@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_anim_c =
- *   "$Id: anim.c 4961 2006-09-21 05:14:18Z mwedel $";
+ *   "$Id: anim.c 7569 2007-11-24 21:19:31Z akirschbaum $";
  */
 
 /*
@@ -44,10 +44,8 @@ void free_all_anim(void) {
 void init_anim(void) {
     char buf[MAX_BUF];
     FILE *fp;
-    static int anim_init=0;
     int num_frames=0,faces[MAX_ANIMATIONS],i;
 
-    if (anim_init) return;
     animations_allocated=9;
     num_animations=0;
     /* Make a default.  New animations start at one, so if something
