@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_arch_c =
- *   "$Id: arch.c 5115 2006-11-15 01:32:04Z qal21 $";
+ *   "$Id: arch.c 7568 2007-11-24 21:17:09Z akirschbaum $";
  */
 
 /*
@@ -30,7 +30,7 @@
 #include <funcpoint.h>
 #include <loader.h>
 
-/* IF set, does a little timing on the archetype load. */
+/* If set, does a little timing on the archetype load. */
 #define TIME_ARCH_LOAD 0
 
 static void add_arch(archetype *at);
@@ -285,6 +285,12 @@ archetype *get_archetype_struct(void) {
 /*
  * Reads/parses the archetype-file, and copies into a linked list
  * of archetype-structures.
+ * Called through load_archetypes()
+ *
+ * Will discard object in archetypes, those are handled by second_arch_pass().
+ *
+ * @param fp
+ * opened file descriptor which will be used to read the archetypes.
  */
 static void first_arch_pass(FILE *fp) {
     object *op;
@@ -352,49 +358,67 @@ static void first_arch_pass(FILE *fp) {
 	op->arch=at;
     }
     free(at);
+    op->arch = NULL; /* arch is checked for temporary archetypes if not NULL. */
     free_object(op);
 }
 
 /*
  * Reads the archetype file once more, and links all pointers between
  * archetypes.
+ *
+ * This also handles putting items in inventory when defined in archetype.
+ *
+ * @param fp
+ * file fron which to read. Won't be rewinded.
  */
 
 static void second_arch_pass(FILE *fp) {
   char buf[MAX_BUF],*variable=buf,*argument,*cp;
   archetype *at=NULL,*other;
+    object* inv;
 
-  while(fgets(buf,MAX_BUF,fp)!=NULL) {
-    if(*buf=='#')
-      continue;
-    if((argument=strchr(buf,' '))!=NULL) {
-	*argument='\0',argument++;
-	cp = argument + strlen(argument)-1;
-	while (isspace(*cp)) {
-	    *cp='\0';
-	    cp--;
-	}
-    }
-    if(!strcmp("Object",variable)) {
-      if((at=find_archetype(argument))==NULL)
-        LOG(llevError,"Warning: failed to find arch %s\n",argument);
-    } else if(!strcmp("other_arch",variable)) {
-      if(at!=NULL&&at->clone.other_arch==NULL) {
-        if((other=find_archetype(argument))==NULL)
-          LOG(llevError,"Warning: failed to find other_arch %s\n",argument);
-        else if(at!=NULL)
-          at->clone.other_arch=other;
+    while(fgets(buf,MAX_BUF,fp)!=NULL) {
+        if(*buf=='#')
+            continue;
+        if((argument=strchr(buf,' '))!=NULL) {
+            *argument='\0',argument++;
+            cp = argument + strlen(argument)-1;
+            while (isspace(*cp)) {
+                *cp='\0';
+                cp--;
+            }
+        }
+        if(!strcmp("Object",variable)) {
+            if((at=find_archetype(argument))==NULL)
+                LOG(llevError,"Warning: failed to find arch %s\n",argument);
+        } else if(!strcmp("other_arch",variable)) {
+            if(at!=NULL&&at->clone.other_arch==NULL) {
+                if((other=find_archetype(argument))==NULL)
+                    LOG(llevError,"Warning: failed to find other_arch %s\n",argument);
+                else if(at!=NULL)
+                    at->clone.other_arch=other;
+            }
+        } else if(!strcmp("randomitems",variable)) {
+            if(at!=NULL) {
+                treasurelist *tl=find_treasurelist(argument);
+                if(tl==NULL)
+                    LOG(llevError,"Failed to link treasure to arch (%s): %s\n",at->name, argument);
+                else
+                    at->clone.randomitems=tl;
+            }
+        } else if (!strcmp("arch", variable)) {
+            inv = create_archetype(argument);
+            load_object(fp, inv, LO_LINEMODE, 0);
+            if (at) {
+                insert_ob_in_ob(inv, &at->clone);
+                /*LOG(llevDebug, "Put %s in %s\n", inv->name, at->clone.name);*/
+            }
+            else {
+                LOG(llevError, "Got an arch %s not inside an Object.\n", argument);
+                free_object(inv);
+            }
+        }
       }
-    } else if(!strcmp("randomitems",variable)) {
-      if(at!=NULL) {
-        treasurelist *tl=find_treasurelist(argument);
-        if(tl==NULL)
-          LOG(llevError,"Failed to link treasure to arch (%s): %s\n",at->name, argument);
-        else
-          at->clone.randomitems=tl;
-      }
-    }
-  }
 }
 
 #ifdef DEBUG
@@ -472,16 +496,16 @@ static void load_archetypes(void) {
  */
 
 object *arch_to_object(archetype *at) {
-  object *op;
-  if(at==NULL) {
-    if(warn_archetypes)
-      LOG(llevError,"Couldn't find archetype.\n");
-    return NULL;
-  }
-  op=get_object();
-  copy_object(&at->clone,op);
-  op->arch=at;
-  return op;
+    object *op;
+    if(at==NULL) {
+        if(warn_archetypes)
+            LOG(llevError,"Couldn't find archetype.\n");
+        return NULL;
+    }
+    op=get_object();
+    copy_object_with_inv(&at->clone,op);
+    op->arch=at;
+    return op;
 }
 
 /*
