@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_init_c =
- *   "$Id: init.c 5555 2007-02-16 00:06:50Z ryo_saeba $";
+ *   "$Id: init.c 6369 2007-05-30 20:07:53Z ryo_saeba $";
  */
 
 /*
@@ -97,6 +97,9 @@ ARMOR_SPEED_LINEAR,
 1, /* no_player_stealing */
 0, /* create_home_portals */
 0, /* allow_denied_spells_writing */
+    0, /* allow_broken_converters */
+    0, /* log_timestamp */
+    NULL, /* log_timestamp_format */
 };
 
 /** perhaps not the best place for this, but needs to be
@@ -189,11 +192,11 @@ void init_library(void) {
     read_smooth();
     init_anim();    /* Must be after we read in the bitmaps */
     init_archetypes();	/* Reads all archetypes from file */
-    init_dynamic ();
     init_attackmess();
     init_clocks();
     init_emergency_mappath();
     init_experience();
+    init_dynamic ();
 }
 
 
@@ -319,13 +322,21 @@ void init_dynamic (void) {
                 strcpy (first_map_ext_path, at->clone.race);
             }
             if (EXIT_PATH (&at->clone)) {
-                strcpy (first_map_path, EXIT_PATH (&at->clone));
+                mapstruct* first;
+                snprintf(first_map_path, sizeof(first_map_path), EXIT_PATH (&at->clone));
+                first = ready_map_name(first_map_path, 0);
+                if (!first) {
+                    LOG(llevError, "Initial map %s can't be found! Please ensure maps are correctly installed.\n", first_map_path);
+                    LOG(llevError, "Unable to continue without initial map.\n");
+                    abort();
+                }
+                delete_map(first);
                 return;
             }
 	}
 	at = at->next;
     }
-    LOG(llevDebug,"You Need a archetype called 'map' and it have to contain start map\n");
+    LOG(llevError,"You need a archetype called 'map' and it have to contain start map\n");
     exit (-1);
 }
 
