@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_item_c =
- *   "$Id: item.c 5544 2007-02-15 19:44:29Z ryo_saeba $";
+ *   "$Id: item.c 7568 2007-11-24 21:17:09Z akirschbaum $";
  */
 
 /*
@@ -65,6 +65,7 @@ body_locations_struct body_locations[NUM_BODY_LOCATIONS] = {
 {"body_hand",	"on your hands",	"on a human's hands"},
 {"body_wrist",	"around your wrists",	"around a human's wrist"},
 {"body_waist",	"around your waist",	"around a human's waist"},
+    {"body_leg",	"around your legs",	"around a human's legs"},
 
 /*{"body_dragon_torso", "your body", "a dragon's body"} */
 };
@@ -1258,7 +1259,7 @@ int is_magical(const object *op) {
 		 return 1;
 
     /* Check for stealty, speed, flying, or just plain magic in the boots */
-    /* Presume any boots that hvae a move_type are special. */
+    /* Presume any boots that have a move_type are special. */
     if (op->type== BOOTS && 
 	((QUERY_FLAG(op, FLAG_STEALTH) || op->move_type ||  op->stats.exp)))
 		return 1;
