@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_living_c =
- *   "$Id: living.c 5138 2006-11-26 18:23:36Z ryo_saeba $";
+ *   "$Id: living.c 6479 2007-06-09 20:59:08Z ryo_saeba $";
  */
 
 /*
@@ -1107,6 +1107,8 @@ void fix_player(object *op) {
 			else /* To nullify the below effect */
 			    ac+=tmp->stats.ac+tmp->magic;
 		    }
+		    if(tmp->stats.dam && tmp->type == BRACERS)
+                        op->stats.dam+=(tmp->stats.dam+tmp->magic);
 		    if(tmp->stats.wc) wc-=(tmp->stats.wc+tmp->magic);
 		    if(tmp->stats.ac) ac-=(tmp->stats.ac+tmp->magic);
 		    if(ARMOUR_SPEED(tmp)&&ARMOUR_SPEED(tmp)/10.0<max)
@@ -1649,7 +1651,7 @@ static void add_player_exp(object *op, sint64 exp, const char *skill_name, int f
 	    if (!skill_obj) {
 		if (flag == SK_EXP_NONE) return;
 		else if (flag == SK_EXP_ADD_SKILL)
-		    give_skill_by_name(op, skill_name);
+		    skill_obj = give_skill_by_name(op, skill_name);
 	    }
 	}
     }
@@ -1759,6 +1761,7 @@ static void subtract_player_exp(object *op, sint64 exp, const char *skill, int f
  * skill_name is the skill that should get the exp added.
  * flag is what to do if player doesn't have the skill.
  * these last two values are only used for players.
+ * @see share_exp() for a party-aware version.
  */
  
 void change_exp(object *op, sint64 exp, const char *skill_name, int flag) {
@@ -1870,3 +1873,58 @@ int did_make_save(const object *op, int level, int bonus)
     return 1;
 }
     
+/**
+ * Gives experience to a player/monster, sharing it with party if applicable.  This
+ * does bounds checking to make sure we don't overflow the max exp.
+ *
+ * The exp passed is typically not modified much by this function -
+ * it is assumed the caller has modified the exp as needed.
+ *
+ * This is merely a wrapper for change_exp().
+ *
+ * @param op
+ * victim to alter.
+ * @param exp
+ * experience to gain (positive) or lose (negative).
+ * @param skill
+ * skill to change. Can be NULL.
+ * @param flag
+ * @li if experience gain, what to do if player doesn't have the skill
+ * @li if experience loss, whether to remove from all skills or only specified skill
+ * @note
+ * flag only applies to op, not other players in same party.
+ */
+void share_exp(object *op, sint64 exp, const char *skill, int flag) {
+    int shares=0,count=0;
+    player *pl;
+    partylist *party;
+
+    if(op->type!=PLAYER || op->contr->party==NULL) {
+        change_exp(op,exp, skill, 0);
+        return;
+    }
+
+    party = op->contr->party;
+
+    for(pl=first_player;pl!=NULL;pl=pl->next) {
+        if(party && pl->ob->contr->party==party && on_same_map(pl->ob, op)) {
+            count++;
+            shares += (pl->ob->level+4);
+        }
+    }
+    if(count==1 || shares>exp)
+        change_exp(op,exp, skill, flag);
+    else {
+        int share=exp/shares,given=0,nexp;
+        for(pl=first_player;pl!=NULL;pl=pl->next) {
+            if(party && pl->ob->contr->party==party && on_same_map(pl->ob, op)) {
+                nexp=(pl->ob->level+4)*share;
+                change_exp(pl->ob,nexp, skill, SK_EXP_TOTAL);
+                given+=nexp;
+            }
+        }
+        exp-=given;
+        /* give any remainder to the player */
+        change_exp(op,exp, skill, flag);
+    }
+}
