@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_map_c =
- *   "$Id: map.c 5806 2007-03-17 22:45:24Z ryo_saeba $";
+ *   "$Id: map.c 7568 2007-11-24 21:17:09Z akirschbaum $";
  */
 
 /*
@@ -591,9 +591,20 @@ void load_objects (mapstruct *m, FILE *fp, int mapflags) {
  * Modified by MSW 2001-07-01 to do in a single pass - reduces code,
  * and we only save the head of multi part objects - this is needed
  * in order to do map tiling properly.
+ *
+ * @param m
+ * map to save.
+ * @param fp
+ * file where regular objects are saved.
+ * @param fp2
+ * file to save unique objects.
+ * @param flag
+ * combination of @ref SAVE_FLAG_xxx "SAVE_FLAG_xxx" flags.
+ * @return
+ * one of @ref SAVE_ERROR_xxx "SAVE_ERROR_xxx" value.
  */
-void save_objects (mapstruct *m, FILE *fp, FILE *fp2, int flag) {
-    int i, j = 0,unique=0;
+int save_objects (mapstruct *m, FILE *fp, FILE *fp2, int flag) {
+    int i, j = 0,unique=0, res;
     object *op,  *otmp;
     /* first pass - save one-part objects */
     for(i = 0; i < MAP_WIDTH(m); i++)
@@ -613,16 +624,20 @@ void save_objects (mapstruct *m, FILE *fp, FILE *fp2, int flag) {
 		if (op->head || op->owner)
 		    continue;
 
-		if (unique || QUERY_FLAG(op, FLAG_UNIQUE))
-		    save_object( fp2 , op, 3);
-		else
-		    if (flag == 0 ||
-			(flag == 2 && (!QUERY_FLAG(op, FLAG_OBJ_ORIGINAL) &&
-			 !QUERY_FLAG(op, FLAG_UNPAID))))
-		    	save_object(fp, op, 3);
+                if (unique || QUERY_FLAG(op, FLAG_UNIQUE))
+                    res = save_object( fp2 , op, SAVE_FLAG_SAVE_UNPAID | SAVE_FLAG_NO_REMOVE);
+                else
+                    if (flag == 0 ||
+                    (flag == SAVE_FLAG_NO_REMOVE && (!QUERY_FLAG(op, FLAG_OBJ_ORIGINAL) &&
+                    !QUERY_FLAG(op, FLAG_UNPAID))))
+                        res = save_object(fp, op, SAVE_FLAG_SAVE_UNPAID | SAVE_FLAG_NO_REMOVE);
 
-	    } /* for this space */
-	} /* for this j */
+                if (res != 0)
+                    return res;
+            } /* for this space */
+        } /* for this j */
+
+    return 0;
 }
 
 /*
@@ -997,8 +1012,9 @@ static int load_map_header(FILE *fp, mapstruct *m)
 		    m->tile_path[tile-1] = strdup_local(path);
 		}
 	    } /* end if tile direction (in)valid */
-	}
-	else {
+	} else  if (!strcmp(key,"background_music")) {
+	    m->background_music = strdup_local(value);
+	} else {
 	    LOG(llevError,"Got unknown value in map header: %s %s\n", key, value);
 	}
     }
@@ -1061,6 +1077,9 @@ mapstruct *load_original_map(const char *filename, int flags) {
 	MAP_DIFFICULTY(m)=calculate_difficulty(m);
     set_map_reset_time(m);
     
+    /* In case other objects press some buttons down */
+    update_buttons(m);
+
     /* Handle for map load event */
     execute_global_event(EVENT_MAPLOAD, m);
     
@@ -1221,16 +1240,18 @@ static void load_unique_objects(mapstruct *m) {
  * The temporary filename will be stored in the mapstructure.
  * If the map is unique, we also save to the filename in the map
  * (this should have been updated when first loaded)
+ * @return
+ * one of @ref SAVE_ERROR_xxx "SAVE_ERROR_xxx" values.
  */
-
 int new_save_map(mapstruct *m, int flag) {
+#define TEMP_EXT ".savefile"
     FILE *fp, *fp2;
-    char filename[MAX_BUF],buf[MAX_BUF], shop[MAX_BUF];
-    int i;
+    char filename[MAX_BUF],buf[MAX_BUF], shop[MAX_BUF], final[MAX_BUF];
+    int i, res;
     
     if (flag && !*m->path) {
-	LOG(llevError,"Tried to save map without path.\n");
-	return -1;
+        LOG(llevError,"Tried to save map without path.\n");
+        return SAVE_ERROR_NO_PATH;
     }
     
     if (flag || (m->unique) || (m->template)) {
@@ -1259,19 +1280,20 @@ int new_save_map(mapstruct *m, int flag) {
     LOG(llevDebug,"Saving map %s\n",m->path);
     m->in_memory = MAP_SAVING;
 
-    /* Compress if it isn't a temporary save.  Do compress if unique */
-    if (m->compressed && (m->unique || m->template || flag)) {
-	char buf[MAX_BUF];
-	strcpy(buf, uncomp[m->compressed][2]);
-	strcat(buf, " > ");
-	strcat(buf, filename);
-	fp = popen(buf, "w");
-    } else
+    if (m->compressed && (m->unique || m->template || flag != SAVE_MODE_NORMAL)) {
+        char buf[MAX_BUF];
+        snprintf(buf, sizeof(buf), "%s > %s%s", uncomp[m->compressed][2], filename, TEMP_EXT);
+        snprintf(final, sizeof(final), filename);
+        fp = popen(buf, "w");
+    } else {
+        snprintf(final, sizeof(final), filename);
+        snprintf(filename, sizeof(filename), "%s%s", final, TEMP_EXT);
 	fp = fopen(filename, "w");
+    }
 
     if(fp == NULL) {
-	LOG(llevError, "Cannot write %s: %s\n", filename, strerror_local(errno));
-	return -1;
+        LOG(llevError, "Cannot open regular objects file %s: %s\n", filename, strerror_local(errno));
+        return SAVE_ERROR_RCREATION;
     }
     
     /* legacy */
@@ -1326,38 +1348,82 @@ int new_save_map(mapstruct *m, int flag) {
      * player)
      */
     fp2 = fp; /* save unique items into fp2 */
-    if ((flag == 0 || flag == 2) && !m->unique && !m->template) {
-	sprintf (buf,"%s.v00",create_items_path (m->path));
-	if ((fp2 = fopen (buf, "w")) == NULL) {
-	    LOG(llevError, "Can't open unique items file %s\n", buf);
-	}
-    if (flag == 2) {
-        // 2 is non destructive save, so map is still valid.
-	    save_objects(m, fp, fp2, 2);
-        m->in_memory = MAP_IN_MEMORY;
-    }
-	else
-	    save_objects (m, fp, fp2, 0);
-	if (fp2 != NULL) {
-	    if (ftell (fp2) == 0) {
-		fclose (fp2);
-		unlink (buf);
-	    } else {
-		fclose (fp2);
-		chmod (buf, SAVE_MODE);
-	    }
-	}
+    if ((flag == SAVE_MODE_NORMAL || flag == SAVE_MODE_OVERLAY) && !m->unique && !m->template) {
+        char final_unique[MAX_BUF];
+
+        snprintf(final_unique, sizeof(final_unique), "%s.v00", create_items_path (m->path));
+        snprintf(buf, sizeof(buf), "%s%s", final_unique, TEMP_EXT);
+        if ((fp2 = fopen (buf, "w")) == NULL) {
+            LOG(llevError, "Can't open unique items file %s\n", buf);
+            return SAVE_ERROR_UCREATION;
+        }
+        if (flag == SAVE_MODE_OVERLAY) {
+            /* SO_FLAG_NO_REMOVE is non destructive save, so map is still valid. */
+            res = save_objects(m, fp, fp2, SAVE_FLAG_NO_REMOVE);
+            if (res < 0) {
+                LOG(llevError, "Save error during object save: %d\n", res);
+                return res;
+            }
+            m->in_memory = MAP_IN_MEMORY;
+        }
+        else {
+            res = save_objects (m, fp, fp2, 0);
+            if (res < 0) {
+                LOG(llevError, "Save error during object save: %d\n", res);
+                return res;
+            }
+            free_all_objects(m);
+        }
+        if (fp2 != NULL) {
+            if (ftell (fp2) == 0) {
+                fclose (fp2);
+                unlink (buf);
+		/* If there are no unique items left on the map, we need to
+		 * unlink the original unique map so that the unique
+		 * items don't show up again.
+		 */
+		unlink(final_unique);
+            } else {
+                fflush(fp2);
+                fclose (fp2);
+                unlink(final_unique); /* failure isn't too bad, maybe the file doesn't exist. */
+                if (rename(buf, final_unique) == -1) {
+                    LOG(llevError, "Couldn't rename unique file %s to %s\n", buf, final_unique);
+                    return SAVE_ERROR_URENAME;
+                }
+                chmod (final_unique, SAVE_MODE);
+            }
+        }
     } else { /* save same file when not playing, like in editor */
-	save_objects(m, fp, fp, 0);
+        res = save_objects(m, fp, fp, 0);
+        if (res < 0) {
+            LOG(llevError, "Save error during object save: %d\n", res);
+            return res;
+        }
+        free_all_objects(m);
     }
 
-    if (m->compressed && (m->unique || m->template || flag))
-	pclose(fp);
-    else
-	fclose(fp);
+    if (m->compressed && (m->unique || m->template || flag != SAVE_MODE_NORMAL)) {
+        fflush(fp);
+        if (pclose(fp) == -1) {
+            LOG(llevError, "pclose error!\n");
+            return SAVE_ERROR_CLOSE;
+        }
+    } else {
+        fflush(fp);
+        if (fclose(fp) != 0) {
+            LOG(llevError, "fclose error!\n");
+            return SAVE_ERROR_CLOSE;
+        }
+    }
+    unlink(final); /* failure isn't too bad, maybe the file doesn't exist. */
+    if (rename(filename, final) == -1) {
+        LOG(llevError, "Couldn't rename regular file %s to %s\n", filename, final);
+        return SAVE_ERROR_RRENAME;
+    }
 
-    chmod (filename, SAVE_MODE);
-    return 0;
+    chmod(final, SAVE_MODE);
+    return SAVE_ERROR_OK;
 }
 
 
@@ -1447,6 +1513,7 @@ void free_map(mapstruct *m,int flag) {
     if (m->maplore) FREE_AND_CLEAR(m->maplore);
     if (m->shopitems) FREE_AND_CLEAR(m->shopitems);
     if (m->shoprace) FREE_AND_CLEAR(m->shoprace);
+    if (m->background_music) FREE_AND_CLEAR(m->background_music);
     if (m->buttons)
 	free_objectlinkpt(m->buttons);
     m->buttons = NULL;
@@ -1607,8 +1674,7 @@ mapstruct *ready_map_name(const char *name, int flags) {
      */
 
     decay_objects(m); /* start the decay */
-    /* In case other objects press some buttons down */
-    update_buttons(m);
+
     if (m->outdoor)
 	    set_darkness_map(m);
     /* run the weather over this map */
@@ -1817,7 +1883,7 @@ static void inline add_face_layer(int low_layer, int high_layer, object *ob, obj
  * through, etc)
  */
 void update_position (mapstruct *m, int x, int y) {
-    object *tmp;
+    object *tmp, *player=NULL;
     uint8 flags = 0, oldflags, light=0;
     object *layers[MAP_LAYERS];
 
@@ -1838,6 +1904,8 @@ void update_position (mapstruct *m, int x, int y) {
         if (QUERY_FLAG(tmp, FLAG_WIZ) && tmp->contr->hidden)
             continue;
 
+	if (tmp->type == PLAYER) player=tmp;
+
         /* This could be made additive I guess (two lights better than
          * one).  But if so, it shouldn't be a simple additive - 2
          * light bulbs do not illuminate twice as far as once since
@@ -1897,6 +1965,8 @@ void update_position (mapstruct *m, int x, int y) {
 	    flags |= P_BLOCKSVIEW;
     } /* for stack of objects */
 
+    if (player) flags |= P_PLAYER;
+
     /* we don't want to rely on this function to have accurate flags, but
      * since we're already doing the work, we calculate them here.
      * if they don't match, logic is broken someplace.
@@ -1907,6 +1977,7 @@ void update_position (mapstruct *m, int x, int y) {
 	    m->path, x, y,
             (oldflags & ~P_NEED_UPDATE), flags);
     }
+
     SET_MAP_FLAGS(m, x, y, flags);
     SET_MAP_MOVE_BLOCK(m, x, y, move_block & ~move_allow);
     SET_MAP_MOVE_ON(m, x, y, move_on);
@@ -1914,8 +1985,14 @@ void update_position (mapstruct *m, int x, int y) {
     SET_MAP_MOVE_SLOW(m, x, y, move_slow);
     SET_MAP_LIGHT(m,x,y,light);
 
+    /* Note that player may be NULL here, which is fine - if no player, need
+     * to clear any value that may be set.
+     */
+    SET_MAP_PLAYER(m, x, y, player);
+
     /* Note it is intentional we copy everything, including NULL values. */
     memcpy(GET_MAP_FACE_OBJS(m, x, y), layers, sizeof(object*) * MAP_LAYERS);
+
 }
 
 
@@ -2125,8 +2202,8 @@ static int adjacent_map(const mapstruct *map1, const mapstruct *map2, int *dx, i
  * This is used by get_player to determine where the other
  * creature is.  get_rangevector takes into account map tiling,
  * so you just can not look the the map coordinates and get the
- * righte value.  distance_x/y are distance away, which
- * can be negativbe.  direction is the crossfire direction scheme
+ * right value.  distance_x/y are distance away, which
+ * can be negative.  direction is the crossfire direction scheme
  * that the creature should head.  part is the part of the
  * monster that is closest.
  * 
