@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_player_c =
- *   "$Id: player.c 5149 2006-12-02 19:17:49Z ryo_saeba $";
+ *   "$Id: player.c 5866 2007-03-24 15:08:23Z ryo_saeba $";
  */
 
 /*
@@ -34,6 +34,9 @@
  */
 void free_player(player *pl) {
 
+    client_spell *info;
+    client_spell* next;
+
     if (first_player!=pl) {
         player *prev=first_player;
         while(prev!=NULL&&prev->next!=NULL&&prev->next!=pl)
@@ -54,8 +57,7 @@ void free_player(player *pl) {
     if (pl->stack_items)
         free( pl->stack_items );
 
-    client_spell *info = pl->spell_state;
-    client_spell* next;
+    info = pl->spell_state;
     while ( info )
     {
         next = info->next;
