@@ -193,8 +193,8 @@ void init_formulae(void) {
   check_formulae();
 }
 
-/* check_formulae()- since we are doing a squential search on the
- * formulae lists now, we have to be carefull that we dont have 2
+/* check_formulae()- since we are doing a sequential search on the
+ * formulae lists now, we have to be careful that we dont have 2
  * formula with the exact same index value. Under the new nbatches
  * code, it is possible to have multiples of ingredients in a cauldron
  * which could result in an index formula mismatch. We *don't* check for
@@ -207,16 +207,17 @@ void check_formulae( void ) {
 
   LOG(llevDebug,"Checking formulae lists...");
 
-  for(fl=formulalist; fl!=NULL; fl = fl->next) { 
-    for (formula=fl->items; formula!=NULL; formula=formula->next) 
-      for (check=formula->next; check!=NULL; check=check->next) 
-        if(check->index==formula->index) { 
-          LOG(llevError," ERROR: On %d ingred list: ", numb);
-          LOG(llevError, "Formulae [%s] of %s and [%s] of %s have matching index id (%d)\n",
-            formula->arch_name[0],formula->title,check->arch_name[0],check->title,formula->index);
-        }
-    numb++;
-  }
+    for(fl=formulalist; fl!=NULL; fl = fl->next) {
+        for (formula=fl->items; formula!=NULL; formula=formula->next)
+            for (check=formula->next; check!=NULL; check=check->next)
+                if(check->index==formula->index && strcmp(check->cauldron, formula->cauldron) == 0) {
+                    /* if the recipes don't have the same facility, then no issue anyway. */
+                    LOG(llevError," ERROR: On %d ingred list:\n", numb);
+                    LOG(llevError, "Formulae [%s] of %s and [%s] of %s have matching index id (%d)\n",
+                        formula->arch_name[0],formula->title,check->arch_name[0],check->title,formula->index);
+                }
+        numb++;
+    }
 
   LOG(llevDebug,"done.\n");
 
