@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_map_c =
- *   "$Id: region.c 5127 2006-11-19 20:15:44Z akirschbaum $";
+ *   "$Id: region.c 6039 2007-04-15 21:45:00Z ryo_saeba $";
  */
 
 /*
@@ -94,7 +94,10 @@ region *get_region_from_string(const char *name) {
     region *reg;
     char *substr;
     char *p;
-    
+   
+    if (first_region == NULL)
+        return NULL;
+
     if (name==NULL) {
     	for (reg=first_region;reg->parent!=NULL;reg=reg->parent);
     	return reg;
