@@ -1,7 +1,7 @@
 
 /*
  * static char *rcs_treasure_c =
- *   "$Id: treasure.c 5295 2007-01-03 21:03:42Z ryo_saeba $";
+ *   "$Id: treasure.c 7568 2007-11-24 21:17:09Z akirschbaum $";
  */
 
 /*
@@ -277,8 +277,6 @@ treasurelist *find_treasurelist(const char *name) {
 
 static void put_treasure (object *op, object *creator, int flags)
 {
-    object *tmp;
-
     /* Bit of a hack - spells should never be put onto the map.  The entire
      * treasure stuff is a problem - there is no clear idea of knowing
      * this is the original object, or if this is an object that should be created
@@ -293,8 +291,6 @@ static void put_treasure (object *op, object *creator, int flags)
         op = insert_ob_in_ob (op, creator);
         if ((flags & GT_APPLY) && QUERY_FLAG (creator, FLAG_MONSTER))
             monster_check_apply(creator, op);
-        if ((flags & GT_UPDATE_INV) && (tmp = get_player_container (creator)) != NULL)
-            esrv_send_item(tmp, op);
     }
 }
 
@@ -426,7 +422,7 @@ void create_treasure(treasurelist *t, object *op, int flag, int difficulty,
 /* This is similar to the old generate treasure function.  However,
  * it instead takes a treasurelist.  It is really just a wrapper around
  * create_treasure.  We create a dummy object that the treasure gets
- * inserted into, and then return that treausre
+ * inserted into, and then return that treasure
  */
 object *generate_treasure(treasurelist *t, int difficulty)
 {
@@ -1255,6 +1251,9 @@ void init_artifacts(void) {
     linked_char *tmp;
     int value, comp;
     artifactlist *al;
+    archetype dummy_archetype;
+
+    memset(&dummy_archetype, 0, sizeof(archetype));
 
     if (has_been_inited) return;
     else has_been_inited = 1;
@@ -1281,43 +1280,45 @@ void init_artifacts(void) {
 		if (art==NULL) {
 		  art=get_empty_artifact();
 		  nrofartifacts++;
-        }
-	    cp = strchr(cp,' ') + 1;
-	    if (!strcmp(cp,"all")) continue;
+		}
+		cp = strchr(cp,' ') + 1;
+		if (!strcmp(cp,"all")) continue;
+                  do {
+		    nrofallowedstr++;
+		    if ((next=strchr(cp, ','))!=NULL)
+		       *(next++) = '\0';
+		    tmp = (linked_char*) malloc(sizeof(linked_char));
+		    tmp->name = add_string(cp);
+		    tmp->next = art->allowed;
+		    art->allowed = tmp;
+		  } while ((cp=next)!=NULL);
 
-	    do {
-		nrofallowedstr++;
-		if ((next=strchr(cp, ','))!=NULL)
-		    *(next++) = '\0';
-		tmp = (linked_char*) malloc(sizeof(linked_char));
-		tmp->name = add_string(cp);
-		tmp->next = art->allowed;
-		art->allowed = tmp;
-		} while ((cp=next)!=NULL);
-	}
-	else if (sscanf(cp, "chance %d", &value))
-	    art->chance = (uint16) value;
-	else if (sscanf(cp, "difficulty %d", &value))
-	    art->difficulty = (uint8) value;
-	else if (!strncmp(cp, "Object",6)) {
-	    art->item = (object *) calloc(1, sizeof(object));
-	    reset_object(art->item);
-	    if (!load_object(fp, art->item,LO_LINEMODE,0))
-		LOG(llevError,"Init_Artifacts: Could not load object.\n");
-	    art->item->name = add_string((strchr(cp, ' ')+1));
-	    al=find_artifactlist(art->item->type);
-	    if (al==NULL) {
-		al = get_empty_artifactlist();
-		al->type = art->item->type;
-		al->next = first_artifactlist;
-		first_artifactlist = al;
-	    }
-	    art->next = al->items;
-	    al->items = art;
-		art = NULL;
-	}
-	else
-	    LOG(llevError,"Unknown input in artifact file: %s\n", buf);
+        }
+        else if (sscanf(cp, "chance %d", &value))
+            art->chance = (uint16) value;
+        else if (sscanf(cp, "difficulty %d", &value))
+            art->difficulty = (uint8) value;
+        else if (!strncmp(cp, "Object",6)) {
+            art->item = (object *) calloc(1, sizeof(object));
+            reset_object(art->item);
+            art->item->arch = &dummy_archetype;
+            if (!load_object(fp, art->item,LO_LINEMODE,0))
+                LOG(llevError,"Init_Artifacts: Could not load object.\n");
+            art->item->arch = NULL;
+            art->item->name = add_string((strchr(cp, ' ')+1));
+            al=find_artifactlist(art->item->type);
+            if (al==NULL) {
+                al = get_empty_artifactlist();
+                al->type = art->item->type;
+                al->next = first_artifactlist;
+                first_artifactlist = al;
+            }
+            art->next = al->items;
+            al->items = art;
+            art = NULL;
+        }
+        else
+            LOG(llevError,"Unknown input in artifact file: %s\n", buf);
     }
 
     close_and_delete(fp, comp);
@@ -1506,6 +1507,17 @@ void add_abilities(object *op, object *change) {
 	    free_string(op->msg);
 	op->msg = add_refcount(change->msg);
     }
+
+    if (change->inv) {
+        object* inv = change->inv;
+        object* copy;
+        while (inv) {
+            copy = get_object();
+            copy_object(inv, copy);
+            insert_ob_in_ob(copy, op);
+            inv = inv->below;
+        }
+    }
 }
 
 static int legal_artifact_combination(object *op, artifact *art) {
