@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_utils_c =
- *   "$Id: utils.c 5544 2007-02-15 19:44:29Z ryo_saeba $";
+ *   "$Id: utils.c 7568 2007-11-24 21:17:09Z akirschbaum $";
  */
 
 /*
@@ -185,7 +185,7 @@ int rndm(int min, int max)
   return(RANDOM()%diff+min);
 }
 
-/* decay and destroy persihable items in a map */
+/* decay and destroy perishable items in a map */
 
 void decay_objects(mapstruct *m)
 {
@@ -209,6 +209,9 @@ void decay_objects(mapstruct *m)
                   QUERY_FLAG(op, FLAG_OVERLAY_FLOOR) ||
                   QUERY_FLAG(op, FLAG_UNPAID) || IS_LIVE(op))
                     continue;
+                if (op->head)
+                    /* Don't try to remove a non head part of a multipart object, remove_ob() would abort(). */
+                    continue;
                 /* otherwise, we decay and destroy */
                 if (IS_WEAPON(op)) {
                     op->stats.dam--;
