@@ -18,7 +18,7 @@ static XtResource resources[] = {
 };
 #undef Offset
 
-static void ClassInitialize();
+static void ClassInitialize(void);
 static void Initialize(Widget req,Widget new,Arg args[],Cardinal *num);
 static void Destroy(Widget w);
 static void Resize(Widget w);
@@ -83,7 +83,7 @@ static int DrawObject(Widget wd, int x, int y, object * tmp)
     if (!tmp)
 	tmp = &empty_archetype->clone;
 
-    FaceDraw (wd, w->crFace.gc, tmp->face, x * FontSize, y * FontSize);
+    DrawPartObject(wd, w->crFace.gc, tmp, x * FontSize, y * FontSize);
     return (0);
 }
 
@@ -131,7 +131,7 @@ static void GetSize(Widget w,Dimension *width,Dimension *height)
  * members
  **********************************************************************/
 
-static void ClassInitialize()
+static void ClassInitialize(void)
 {
   debug0 ("ClassInitialize()\n");
 }
