@@ -42,7 +42,7 @@ static XtResource resources[] = {
  * declarations
  **********************************************************************/
 
-static void ClassInitialize();
+static void ClassInitialize(void);
 static void Initialize(Widget req,Widget new,Arg args[],Cardinal *num);
 static void Destroy(Widget w);
 static void Resize(Widget w);
@@ -167,7 +167,7 @@ static void SetSize(Widget w)
  * y       : index
  * return  : 
  */
-CrListNode GetYNode (Widget w, int y)
+static CrListNode GetYNode (Widget w, int y)
 {
     CrListWidget self = (CrListWidget)w;
     CrListNode node = NULL;
@@ -187,7 +187,7 @@ CrListNode GetYNode (Widget w, int y)
 }
 
 /* hmm */
-CrListCall GetYCall (Widget w, int y)
+static CrListCall GetYCall (Widget w, int y)
 {
     CrListNode node = GetYNode (w, y);
     static struct _CrListCall ret;
@@ -246,7 +246,7 @@ static void Delete(Widget new,XEvent *e,String *argv,Cardinal *argc)
  * members
  **********************************************************************/
 
-static void ClassInitialize()
+static void ClassInitialize(void)
 {
     debug0("CrList-ClassInitialize()\n");
 }
@@ -307,7 +307,7 @@ static void Redisplay(Widget w,XEvent *event,Region region)
     for (y = rect.y; node && (!region || rect.height--); y++) {
 	debug2 ("%s Drawing %s\n", self->core.name,node->name);
 	/*DrawObject(w,0,y,node);*/
-	FaceDraw (w, self->crList.gc, node->face, 0, y * FontSize);
+	DrawFacePart(w, self->crList.gc, node->face, 0, y * FontSize, 0, 0);
 	XDrawString(XtDisplay(w),XtWindow(w),
 		    self->crList.normal_GC 
 		    /*DefaultGCOfScreen(XtScreen(w))*/,
