@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_commands_h =
- *   "$Id: commands.h,v 1.5 2006/03/18 15:05:32 ryo_saeba Exp $";
+ *   "$Id: commands.h 5117 2006-11-15 02:29:53Z qal21 $";
  */
 
 /*
@@ -26,6 +26,9 @@
     The author can be reached via e-mail to mark@pyramid.com
 */
 
+#ifndef COMMANDS_H
+#define COMMANDS_H
+
 /*
  * Crossfire commands
  *	++Jam
@@ -43,9 +46,6 @@
  * be initialized in one of the source files.
  */
 
-#ifndef COMMANDS_H
-#define COMMANDS_H
-
 typedef int (*command_function)(object *op, char *params);
 
 typedef struct {		/* global list's structure */
@@ -115,4 +115,5 @@ extern const int CommandsSize,NewServerCommandSize,
 #define EMOTE_BLEED	52
 #define EMOTE_CRINGE	53
 #define EMOTE_THINK	54
-#endif
+
+#endif /* COMMANDS_H */
