@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_define_h =
- *   "$Id: define.h 5544 2007-02-15 19:44:29Z ryo_saeba $";
+ *   "$Id: define.h 7624 2007-12-01 10:15:18Z ryo_saeba $";
  */
 
 /*
@@ -384,7 +384,7 @@ extern typedata ItemTypes[];
 
 /* Instead of using arbitrary constants for indexing the
  * freearr, add these values.  <= SIZEOFFREE1 will get you
- * within 1 space.  <= SIZEOFFREE2 wll get you withing 
+ * within 1 space.  <= SIZEOFFREE2 will get you withing 
  * 2 spaces, and the entire array (< SIZEOFFREE) is 
  * three spaces
  */
@@ -601,7 +601,7 @@ extern typedata ItemTypes[];
 #define FLAG_ACTIVATE_ON_RELEASE 106	/* connected object is activated when 'released' */
 #define FLAG_IS_WATER            107
 #define FLAG_CONTENT_ON_GEN      108
-#define FLAG_IS_A_TEMPLATE       109 /* Object has no ingame life until instanciated*/
+#define FLAG_IS_A_TEMPLATE       109 /* Object has no ingame life until instantiated*/
 #define FLAG_IS_BUILDABLE        110 /* Can build on item */
 #define FLAG_AFK                 111 /* Player is AFK */
 #define NUM_FLAGS                111 /* Should always be equal to the last
@@ -722,6 +722,11 @@ typedef unsigned char	MoveType;
 #define F_APPROX	32  /* flag to give a guess of item value */
 #define F_SHOP		64  /* consider the effect that the shop that the player is in has */
 
+/**
+ * Maximum item power an item can have. If changed, check object::item_power for overflow issues.
+ */
+#define MAX_WEAPON_ITEM_POWER   100
+
 #define DIRX(xyz)	freearr_x[(xyz)->direction]
 #define DIRY(xyz)	freearr_y[(xyz)->direction]
 
@@ -1014,7 +1019,7 @@ enum apply_flag {
  * and clean up the string. (Lots faster than doing each time...)
  * If you use them and have choice between FAST_STRCAT and FAST_STRNCAT,
  * keep in mind FAST_STRNCAT is faster since length of second argument is
- * kown in advance.
+ * known in advance.
  */
 
  #define PREPARE_FASTCAT(buf__) buf__+strlen(buf__)
