@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_global_h =
- *   "$Id: global.h 5555 2007-02-16 00:06:50Z ryo_saeba $";
+ *   "$Id: global.h 8175 2008-01-12 09:27:41Z akirschbaum $";
  */
 
 /*
@@ -77,6 +77,7 @@ typedef unsigned char	uint8;
 typedef signed char	sint8;
 #define SINT8_MAX	127
 
+typedef const char* sstring;
 
 typedef unsigned short Fontindex;
 
@@ -212,6 +213,7 @@ extern uint32 pticks;		/* used by various function to determine */
  * Misc global variables:
  */
 EXTERN FILE *logfile;			/* Used by server/daemon.c */
+EXTERN int reopen_logfile;
 EXTERN int exiting;			/* True if the game is about to exit */
 EXTERN long nroftreasures;		/* Only used in malloc_info() */
 EXTERN long nrofartifacts;		/* Only used in malloc_info() */
@@ -263,6 +265,7 @@ extern socket_struct *init_sockets;
 
 
 #ifndef __CEXTRACT__
+#include "stringbuffer.h"
 #include "libproto.h"
 #include "sockproto.h"
 #endif
@@ -344,7 +347,7 @@ typedef struct Settings {
     uint8   not_permadeth;  /* if true, death is non-permament */
     uint8   simple_exp;	    /* If true, use the simple experience system */
     int	    reset_loc_time; /* Number of seconds to put player back at home */
-    uint8   set_title;	    /* players can set thier title */
+    uint8   set_title;	    /* players can set their title */
     uint8   resurrection;   /* ressurection possible w/ permadeth on */
     uint8   search_items;   /* search_items command */
     uint8   spell_encumbrance; /* encumbrance effects spells */
@@ -394,6 +397,10 @@ typedef struct Settings {
     uint8   no_player_stealing;     /* If 1, can not steal from other players */
     uint8   create_home_portals;     /* If 1, can create portals in unique maps (apartments) */
     int     allow_denied_spells_writing; /* If set, players can write spells they can't cast. */
+    int     allow_broken_converters; /**< If set, converters will work even if price of generated item is higher
+                                       * than the price of converted items. */
+    int     log_timestamp; /**< If set, log will comport a timestamp. */
+    char*   log_timestamp_format; /**< Format for timestap, if log_timestamp is set. */
 } Settings;
 
 extern Settings settings;
