@@ -125,6 +125,7 @@ extern void calc_perm_exp(object *op);
 extern sint64 check_exp_loss(const object *op, sint64 exp);
 extern sint64 check_exp_adjust(const object *op, sint64 exp);
 extern void change_exp(object *op, sint64 exp, const char *skill_name, int flag);
+extern void share_exp(object *op, sint64 exp, const char *skill_name, int flag);
 extern void apply_death_exp_penalty(object *op);
 extern int did_make_save(const object *op, int level, int bonus);
 /* logger.c */
@@ -152,7 +153,7 @@ extern int blocked_link(object *ob, mapstruct *m, int sx, int sy);
 extern int ob_blocked(const object *ob, mapstruct *m, sint16 x, sint16 y);
 extern void fix_container(object *container);
 extern void load_objects(mapstruct *m, FILE *fp, int mapflags);
-extern void save_objects(mapstruct *m, FILE *fp, FILE *fp2, int flag);
+extern int save_objects(mapstruct *m, FILE *fp, FILE *fp2, int flag);
 extern mapstruct *get_linked_map(void);
 extern void allocate_map(mapstruct *m);
 extern mapstruct *get_empty_map(int sizex, int sizey);
@@ -198,6 +199,7 @@ extern void update_ob_speed(object *op);
 extern void remove_from_active_list(object *op);
 extern void update_object(object *op, int action);
 extern void free_object(object *ob);
+extern void free_object2(object *ob, int free_inventory);
 extern int count_free(void);
 extern int count_used(void);
 extern int count_active(void);
@@ -209,6 +211,7 @@ extern object *insert_ob_in_map(object *op, mapstruct *m, object *originator, in
 extern void replace_insert_ob_in_map(const char *arch_string, object *op);
 extern object *get_split_ob(object *orig_ob, uint32 nr);
 extern object *decrease_ob_nr(object *op, uint32 i);
+extern void increase_ob_nr(object *op, uint32 i);
 extern void add_weight(object *op, signed long weight);
 extern object *insert_ob_in_ob(object *op, object *where);
 extern int check_move_on(object *op, object *originator);
@@ -237,9 +240,9 @@ extern object *find_obj_by_type_subtype(const object *who, int type, int subtype
 extern key_value *get_ob_key_link(const object *ob, const char *key);
 extern const char *get_ob_key_value(const object *op, const char *const key);
 extern int set_ob_key_value(object *op, const char *key, const char *value, int add_key);
-extern object *find_best_weapon_used_match(object *pl, const char *params);
 extern int item_matched_string(object *pl, object *op, const char *name);
 extern void fix_multipart_object(object* tmp);
+extern void copy_object_with_inv(object *src_ob, object *dest_ob);
 /* path.c */
 extern char *path_combine(const char *src, const char *dst);
 extern void path_normalize(char *path);
@@ -388,12 +391,12 @@ extern void strip_endline(char *buf);
 extern void replace(const char *src, const char *key, const char *replacement, char *result, size_t resultsize);
 extern void make_list_like(char *input);
 /* loader.c */
-extern int lex_load(object *op, int map_flags);
+extern int lex_load(int* depth, object** items, int maxdepth, int map_flags, int linemode);
 extern void yyrestart(FILE *input_file);
 extern void yy_load_buffer_state(void);
 extern int yyerror(char *s);
 extern int load_object(FILE *fp, object *op, int bufstate, int map_flags);
 extern int set_variable(object *op, char *buf);
 extern void init_vars(void);
-extern char *get_ob_diff(object *op, object *op2);
-extern void save_object(FILE *fp, object *op, int flag);
+extern void get_ob_diff(StringBuffer *sb, object *op, object *op2);
+extern int save_object(FILE *fp, object *op, int flag);
