@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_object_h =
- *   "$Id: loader.h,v 1.16 2005/10/24 20:48:15 akirschbaum Exp $";
+ *   "$Id: loader.h 5117 2006-11-15 02:29:53Z qal21 $";
  */
 
 /*
@@ -26,6 +26,8 @@
     The authors can be reached via e-mail to crossfire-devel@real-time.com
 */
 
+#ifndef LOADER_H
+#define LOADER_H
 
 #define LL_IGNORED  -1
 #define LL_EOF	    0
@@ -39,3 +41,5 @@
 #define	LO_NOREAD   3
 
 extern int nrofpixmaps;
+
+#endif /* LOADER_H */
