@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_define_h =
- *   "$Id: map.h 5117 2006-11-15 02:29:53Z qal21 $";
+ *   "$Id: map.h 6918 2007-08-12 10:34:43Z ryo_saeba $";
  */
 
 /*
@@ -111,12 +111,46 @@ extern const char *map_layer_name[MAP_LAYERS];
 #define MAP_STYLE	    0x8
 #define MAP_OVERLAY	    0x10
 
+/**
+ * Flags for save_object() and save_objects().
+ * Can be combined for various effects.
+ */
+#define SAVE_FLAG_SAVE_UNPAID   1   /**< If set, unpaid items will be saved. */
+#define SAVE_FLAG_NO_REMOVE     2   /**< If set, objects are not removed while saving. */
+
+/*
+ * How save_map() should save the map. Can't be combined.
+ */
+#define SAVE_MODE_NORMAL    0   /**< No special handling. */
+#define SAVE_MODE_INPLACE   1   /**< Map is saved from where it was loaded.*/
+#define SAVE_MODE_OVERLAY   2   /**< Map is persisted as an overlay. */
+
+
 /* Values for in_memory below.  Should probably be an enumerations */
 #define MAP_IN_MEMORY 1
 #define MAP_SWAPPED 2
 #define MAP_LOADING 3
 #define MAP_SAVING 4
 
+/**
+ * @defgroup SAVE_ERROR_xxx Save error values
+ * Those values are returned by save_map(), save_object() and save_objects().
+ *
+ * Values under -10 are returned by swap_map() only.
+ */
+/*@{*/
+#define SAVE_ERROR_OK                0   /**< No error. */
+#define SAVE_ERROR_RCREATION        -1  /**< Couldn't create the regular save file. */
+#define SAVE_ERROR_UCREATION        -2  /**< Couldn't create the file for unique objects. */
+#define SAVE_ERROR_WRITE            -3  /**< Write error. */
+#define SAVE_ERROR_NO_PATH          -4  /**< Map had no path set. */
+#define SAVE_ERROR_URENAME          -5  /**< Couldn't rename unique temporary file. */
+#define SAVE_ERROR_CLOSE            -6  /**< Close error for regular file. */
+#define SAVE_ERROR_RRENAME          -7  /**< Couldn't rename regular temporary file. */
+#define SAVE_ERROR_NOT_IN_MEMORY    -10 /**< Map to swap isn't in memory. */
+#define SAVE_ERROR_PLAYER           -11 /**< Player on map to save. */
+/*@}*/
+
 /* GET_MAP_FLAGS really shouldn't be used very often - get_map_flags should
  * really be used, as it is multi tile aware.  However, there are some cases
  * where it is known the map is not tiled or the values are known
@@ -126,6 +160,8 @@ extern const char *map_layer_name[MAP_LAYERS];
 #define SET_MAP_FLAGS(M,X,Y,C)	( (M)->spaces[(X) + (M)->width * (Y)].flags = C )
 #define GET_MAP_LIGHT(M,X,Y)	( (M)->spaces[(X) + (M)->width * (Y)].light )
 #define SET_MAP_LIGHT(M,X,Y,L)	( (M)->spaces[(X) + (M)->width * (Y)].light = L )
+#define GET_MAP_PLAYER(M,X,Y)	( (M)->spaces[(X) + (M)->width * (Y)].pl )
+#define SET_MAP_PLAYER(M,X,Y,C)	( (M)->spaces[(X) + (M)->width * (Y)].pl = C )
 
 #define GET_MAP_OB(M,X,Y)	( (M)->spaces[(X) + (M)->width * (Y)].bottom )
 #define GET_MAP_TOP(M,X,Y)	( (M)->spaces[(X) + (M)->width * (Y)].top )
@@ -176,7 +212,7 @@ extern const char *map_layer_name[MAP_LAYERS];
  */
 
 #define AB_NO_PASS       0x04
-/*#define P_PASS_THRU	0x08	*//* */
+#define P_PLAYER	0x08	/* There is a player on this space */
 #define P_IS_ALIVE      0x10	/* something alive is on this space */
 #define P_NO_CLERIC     0x20	/* no clerical spells cast here */
 #define P_NEED_UPDATE	0x40	/* this space is out of date */
@@ -218,6 +254,7 @@ typedef struct MapSpace {
     MoveType	move_slow;	/* What movement types this space slows */
     MoveType	move_on;	/* What movement types are activated */
     MoveType	move_off;	/* What movement types are activated */
+    object	*pl;		/* Player that is on this space */
 } MapSpace;
 
 /*
@@ -240,8 +277,8 @@ typedef struct wmapdef {
     uint32	rainfall;	/* cumulative rainfall */
     uint8 	darkness;	/* indicates level of darkness of map */
     uint8	water;		/* 0-100 percentage of water tiles */
-	/*Dynamic parts*/
-	sint16  realtemp;  /*  temperature at a given calculation step for this tile*/
+    /*Dynamic parts*/
+    sint16  realtemp;		/**< Temperature at a given calculation step for this tile. */
 } weathermap_t;
 
 /*
@@ -310,7 +347,7 @@ typedef struct mapdef {
     uint32 nosmooth:1;  /* if set the content of this map has smoothlevel=0 forced*/
     sint32 timeout;	/* swapout is set to this */
     sint32 swap_time;	/* When it reaches 0, the map will be swapped out */
-    sint16 players;	/* How many plares are on this level right now */
+    sint16 players;	/**< How many plares are on this level right now. Automatically updated by the object handling functions. */
     uint32 in_memory;	/* If not true, the map has been freed and must
 		         * be loaded before used.  The map,omap and map_ob
 		         * arrays will be allocated when the map is loaded */
@@ -342,6 +379,7 @@ typedef struct mapdef {
     struct mapdef *tile_map[4];	/* Next map, linked list */
     char path[HUGE_BUF];	/* Filename of the map */
     struct timeval last_reset_time; /* A timestamp of the last original map loading */
+    char* background_music;
 } mapstruct;
 
 /* This is used by get_rangevector to determine where the other
