@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_newclient_h =
- *   "$Id: newclient.h 5131 2006-11-20 07:45:22Z mwedel $";
+ *   "$Id: newclient.h 6974 2007-08-22 19:47:05Z akirschbaum $";
  */
 
 /*
@@ -219,8 +219,9 @@
 				/* to still do all the flags */
 
 
-#define NDI_UNIQUE	0x100	/* Print this out immediately, don't buffer */
-#define NDI_ALL		0x200	/* Inform all players of this message */
+#define NDI_UNIQUE	0x100	/**< Print this out immediately, don't buffer. */
+#define NDI_ALL		0x200	/**< Inform all players of this message. */
+#define NDI_ALL_DMS 0x400   /**< Inform all logged in DMs. Used in case of errors. Overrides NDI_ALL. */
 
 /* Flags for the item command */
 enum {a_none, a_readied, a_wielded, a_worn, a_active, a_applied};
@@ -258,6 +259,7 @@ enum {a_none, a_readied, a_wielded, a_worn, a_active, a_applied};
 #define UPD_ANIM	0x20
 #define UPD_ANIMSPEED	0x40
 #define UPD_NROF	0x80
+#define UPD_ALL     0xFF
 
 /* The flag values sent by the updspell command */
 #define UPD_SP_MANA	0x01
@@ -326,7 +328,7 @@ enum {a_none, a_readied, a_wielded, a_worn, a_active, a_applied};
 					/* resistances, etc) */
 #define MSG_TYPE_SKILL		    12	/* Messages related to using skills */
 #define MSG_TYPE_APPLY		    13	/* Applying objects */
-#define MSG_TYPE_ATTACK		    14	/* Attack related messges */
+#define MSG_TYPE_ATTACK		    14	/* Attack related messages */
 #define MSG_TYPE_COMMUNICATION	    15	/* Communication between players */
 #define MSG_TYPE_SPELL		    16	/* Spell related info */
 #define MSG_TYPE_ITEM		    17	/* Item related information */
