@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_newserver_h =
- *   "$Id: newserver.h 5117 2006-11-15 02:29:53Z qal21 $";
+ *   "$Id: newserver.h 6098 2007-04-22 13:49:02Z ryo_saeba $";
  */
 
 /*
@@ -119,7 +119,7 @@ typedef struct socket_struct {
     buffer_struct outputbuffer;   /* For undeliverable data */
     uint32  facecache:1;    /* If true, client is caching images */
     uint32  sent_scroll:1;
-    uint32  sound:1;	    /* does the client want sound */
+    uint32  sound;	    /* Client sound mode */
     uint32  exp64:1;	    /* Client wants 64 bit exp data, as well as skill data */
     uint32  newmapcmd:1;    /* Send newmap command when entering new map SMACFIGGEN*/
     uint32  darkness:1;	    /* True if client wants darkness information */
