@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_object_h =
- *   "$Id: object.h 5159 2006-12-09 18:12:21Z ryo_saeba $";
+ *   "$Id: object.h 7568 2007-11-24 21:17:09Z akirschbaum $";
  */
 
 /*
@@ -30,7 +30,7 @@
 #define OBJECT_H
 
 typedef uint32 tag_t;
-#define NUM_BODY_LOCATIONS	12
+#define NUM_BODY_LOCATIONS	13
 #define BODY_ARMS		1
 
 /* See common/item.c */
@@ -172,8 +172,6 @@ typedef struct obj {
     sint8	glow_radius;	/* indicates the glow radius of the object */
     living	stats;		/* Str, Con, Dex, etc */
     sint64	perm_exp;	/* Permanent exp */
-    const char	*current_weapon_script;  /* The script of the currently used weapon. Executed */
-				/* each time the object attacks something */
     struct obj	*current_weapon;   /* Pointer to the weapon currently used */
     uint32	weapontype;	/* type of weapon */
     uint32	tooltype;	/* type of tool or build facility */
@@ -287,7 +285,7 @@ extern object objarray[STARTMAX];
 extern int nrofallocobjects;
 extern int nroffreeobjects;
 
-/* This returns TRUE if the object is somethign that
+/* This returns TRUE if the object is something that
  * should be displayed in the look window
  */
 #define LOOK_OBJ(ob) (!ob->invisible && ob->type!=PLAYER && ob->type!=EVENT_CONNECTOR)
