@@ -1,4 +1,4 @@
-/* $Id: path.h,v 1.1 2005/12/08 21:50:19 akirschbaum Exp $ */
+/* $Id: path.h 5117 2006-11-15 02:29:53Z qal21 $ */
 
 #ifndef PATH_H
 #define PATH_H
@@ -21,4 +21,4 @@ void path_normalize(char *path);
  */
 char *path_combine_and_normalize(const char *src, const char *dst);
 
-#endif
+#endif /* PATH_H */
