@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_player_h =
- *   "$Id: player.h 5149 2006-12-02 19:17:49Z ryo_saeba $";
+ *   "$Id: player.h 6328 2007-05-21 21:18:34Z ryo_saeba $";
  */
 
 /*
@@ -120,6 +120,15 @@ typedef struct party_struct {
     uint32  kills;
 } partylist;
 
+/**
+ * Whether to rejoin party at login or not.
+ */
+typedef enum party_rejoin_mode {
+    party_rejoin_no = 0,        /**< Don't rejoin. */
+    party_rejoin_if_exists = 1, /**< Rejoin if party exists. */
+    party_rejoin_always = 2,    /**< If party doesn't exist, form it. */
+} party_rejoin_mode;
+
 typedef struct pl {
     struct pl	*next;		    /* Pointer to next player, NULL if this is last */
     socket_struct	socket;		    /* Socket information for this player */
@@ -140,11 +149,11 @@ typedef struct pl {
     uint32	count;		    /* Any numbers typed before a command */
     uint32	mode;		    /* Mode of player for pickup. */
 
-    sint8	digestion;	    /* Any bonuses/penalties to digestion */
-    sint8	gen_hp;		    /* Bonuses to regeneration speed of hp */
-    sint8	gen_sp;		    /* Bonuses to regeneration speed of sp */
-    sint8	gen_sp_armour;	    /* Penalty to sp regen from armour */
-    sint8	gen_grace;	    /* Bonuses to regeneration speed of grace */
+    sint16	digestion;	    /* Any bonuses/penalties to digestion */
+    sint16	gen_hp;		    /* Bonuses to regeneration speed of hp */
+    sint16	gen_sp;		    /* Bonuses to regeneration speed of sp */
+    sint16	gen_sp_armour;	    /* Penalty to sp regen from armour */
+    sint16	gen_grace;	    /* Bonuses to regeneration speed of grace */
     sint16	item_power;	    /* Total item power of objects equipped */
     uint8	state;		    /* Input state of the player (name, password, etc */
     uint8	listening;	    /* Which priority will be used in info_all */
@@ -214,6 +223,7 @@ typedef struct pl {
 				    /* but we will have to get password first */
 				    /* so we have to remember which party to */
 				    /* join */
+    party_rejoin_mode rejoin_party; /**< Whether to rejoin or not party at login. */
     char    search_str[MAX_BUF];    /* Item we are looking for */
     sint16  encumbrance;	    /* How much our player is encumbered  */
     Output_Buf	outputs[NUM_OUTPUT_BUFS];   /* holds output strings to client */
@@ -226,6 +236,7 @@ typedef struct pl {
     /* Special DM fields */
     tag_t*  stack_items;    /* Item stack for patch/dump/... commands */
     int     stack_position; /* Current stack position, 0 for no item */
+    char * followed_player; /**< Player the DM is following. */
 } player;
 
 #endif /* PLAYER_H */
