@@ -96,7 +96,7 @@
 #define CFAPI_INT     1
 #define CFAPI_LONG    2
 #define CFAPI_CHAR    3
-#define CFAPI_STRING  4
+#define CFAPI_STRING  4 /* String with a length that must be given too. */
 #define CFAPI_POBJECT 5
 #define CFAPI_PMAP    6
 #define CFAPI_FLOAT   7
@@ -109,6 +109,8 @@
 #define CFAPI_INT16   14
 #define CFAPI_TIME    15
 #define CFAPI_SINT64  16
+#define CFAPI_SSTRING 17 /* Shared string that shouldn't be changed, or const char* */
+#define CFAPI_MOVETYPE 18 /* MoveType */
 
 typedef void* (*f_plug_api) (int* type, ...);
 typedef int   (*f_plug_postinit) (void);
@@ -163,7 +165,6 @@ extern crossfire_plugin* plugins_list;
 #define CFAPI_OBJECT_PROP_CONTAINER         8
 #define CFAPI_OBJECT_PROP_MAP               9
 #define CFAPI_OBJECT_PROP_COUNT             10
-#define CFAPI_OBJECT_PROP_REFCOUNT          11
 #define CFAPI_OBJECT_PROP_NAME              12
 #define CFAPI_OBJECT_PROP_NAME_PLURAL       13
 #define CFAPI_OBJECT_PROP_TITLE             14
@@ -252,7 +253,7 @@ extern crossfire_plugin* plugins_list;
 #define CFAPI_OBJECT_PROP_ARCH_NAME         97
 #define CFAPI_OBJECT_PROP_INVISIBLE         98
 #define CFAPI_OBJECT_PROP_FACE              99
-#define CFAPI_OBJECT_ANIMATION              100
+#define CFAPI_OBJECT_PROP_ANIMATION         100
 #define CFAPI_OBJECT_PROP_NO_SAVE           101
 #define CFAPI_OBJECT_PROP_MOVE_TYPE         102
 #define CFAPI_OBJECT_PROP_MOVE_BLOCK        103
@@ -261,6 +262,7 @@ extern crossfire_plugin* plugins_list;
 #define CFAPI_OBJECT_PROP_MOVE_OFF          106
 #define CFAPI_OBJECT_PROP_MOVE_SLOW         107
 #define CFAPI_OBJECT_PROP_MOVE_SLOW_PENALTY 108
+#define CFAPI_OBJECT_PROP_DURATION          109
 
 #define CFAPI_PLAYER_PROP_IP                150
 #define CFAPI_PLAYER_PROP_MARKED_ITEM       151
@@ -268,6 +270,7 @@ extern crossfire_plugin* plugins_list;
 #define CFAPI_PLAYER_PROP_BED_MAP           153
 #define CFAPI_PLAYER_PROP_BED_X             154
 #define CFAPI_PLAYER_PROP_BED_Y             155
+#define CFAPI_PLAYER_PROP_NEXT              156
 
 #define CFAPI_MAP_PROP_FLAGS                0
 #define CFAPI_MAP_PROP_DIFFICULTY           1
@@ -294,6 +297,7 @@ extern crossfire_plugin* plugins_list;
 #define CFAPI_MAP_PROP_MESSAGE              22
 #define CFAPI_MAP_PROP_NEXT                 23
 #define CFAPI_MAP_PROP_REGION               24
+#define CFAPI_MAP_PROP_UNIQUE               25
 
 #define CFAPI_ARCH_PROP_NAME				0
 #define CFAPI_ARCH_PROP_NEXT				1
