@@ -12,7 +12,7 @@ void flush_output_element(const object *pl, Output_Buf *outputs);
 void new_draw_info(int flags, int pri, const object *pl, const char *buf);
 void new_draw_info_format(int flags, int pri, const object *pl, const char *format, ...);
 void draw_ext_info(int flags, int pri, const object *pl, uint8 type, uint8 subtype, const char *message, const char *oldmessage);
-void draw_ext_info_format(int flags, int pri, const object *pl, uint8 type, uint8 subtype, const char *old_format, const char *new_format, ...);
+void draw_ext_info_format(int flags, int pri, const object *pl, uint8 type, uint8 subtype, const char *new_format, const char *old_format, ...);
 void new_info_map_except(int color, const mapstruct *map, const object *op, const char *str);
 void new_info_map_except2(int color, const mapstruct *map, const object *op1, const object *op2, const char *str);
 void new_info_map(int color, const mapstruct *map, const char *str);
@@ -41,6 +41,7 @@ void mark_item_cmd(uint8 *data, int len, player *pl);
 void look_at(object *op, int dx, int dy);
 void look_at_cmd(char *buf, int len, player *pl);
 void esrv_move_object(object *pl, tag_t to, tag_t tag, long nrof);
+void inscribe_scroll_cmd(char *buf, int len, player *pl);
 /* loop.c */
 void request_info_cmd(char *buf, int len, socket_struct *ns);
 void handle_oldsocket(socket_struct *ns);
@@ -53,6 +54,7 @@ void SockList_AddChar(SockList *sl, char c);
 void SockList_AddShort(SockList *sl, uint16 data);
 void SockList_AddInt(SockList *sl, uint32 data);
 void SockList_AddInt64(SockList *sl, uint64 data);
+void SockList_AddString(SockList *sl, const char *data);
 int GetInt_String(const unsigned char *data);
 short GetShort_String(const unsigned char *data);
 int SockList_ReadPacket(int fd, SockList *sl, int len);
@@ -64,6 +66,9 @@ void write_cs_stats(void);
 /* metaserver.c */
 void metaserver_init(void);
 void metaserver_update(void);
+int metaserver2_init(void);
+size_t metaserver2_writer(void *ptr, size_t size, size_t nmemb, void *data);
+void *metaserver2_thread(void *junk);
 /* request.c */
 void set_up_cmd(char *buf, int len, socket_struct *ns);
 void add_me_cmd(char *buf, int len, socket_struct *ns);
@@ -87,6 +92,7 @@ void draw_client_map2(object *pl);
 void draw_client_map(object *pl);
 void esrv_map_scroll(socket_struct *ns, int dx, int dy);
 void send_plugin_custom_message(object *pl, char *buf);
+void send_exp_table(socket_struct *ns, char *params);
 void send_skill_info(socket_struct *ns, char *params);
 void send_spell_paths(socket_struct *ns, char *params);
 void esrv_update_spells(player *pl);
@@ -96,3 +102,4 @@ void send_tick(player *pl);
 /* sounds.c */
 void play_sound_player_only(player *pl, short soundnum, sint8 x, sint8 y);
 void play_sound_map(const mapstruct *map, int x, int y, short sound_num);
+void send_background_music(player *pl, const char *music);
