@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_spells_h =
- *   "$Id: spells.h,v 1.21 2006/02/10 23:59:26 akirschbaum Exp $";
+ *   "$Id: spells.h 5117 2006-11-15 02:29:53Z qal21 $";
  */
 
 /*
@@ -162,4 +162,4 @@ extern const char* const spellpathnames[NRSPELLPATHS];
  */
 #define EXPLODING_FIREBALL  "exploding_fireball"
 
-#endif
+#endif /* SPELLS_H */
