@@ -1,5 +1,5 @@
 /* alchemy.c */
-int use_alchemy(object* op);
+int use_alchemy(object *op);
 /* apply.c */
 int transport_can_hold(const object *transport, const object *op, int nrof);
 int apply_transport(object *pl, object *transport, int aflag);
@@ -117,7 +117,6 @@ int command_body(object *op, char *params);
 int command_motd(object *op, char *params);
 int command_rules(object *op, char *params);
 int command_news(object *op, char *params);
-int command_bug(object *op, char *params);
 void malloc_info(object *op);
 void current_region_info(object *op);
 void current_map_info(object *op);
@@ -148,7 +147,6 @@ int command_dumpmap(object *op, char *params);
 int command_dumpallmaps(object *op, char *params);
 int command_printlos(object *op, char *params);
 int command_version(object *op, char *params);
-void bug_report(const char *reportstring);
 int command_output_sync(object *op, char *params);
 int command_output_count(object *op, char *params);
 int command_listen(object *op, char *params);
@@ -169,16 +167,16 @@ int command_explore(object *op, char *params);
 int command_sound(object *op, char *params);
 void receive_player_name(object *op, char k);
 void receive_player_password(object *op, char k);
-int explore_mode(void);
 int command_title(object *op, char *params);
 int command_save(object *op, char *params);
 int command_peaceful(object *op, char *params);
 int command_wimpy(object *op, char *params);
 int command_brace(object *op, char *params);
-int command_style_map_info(object *op, char *params);
 int command_kill_pets(object *op, char *params);
 int command_quests(object *pl, char *params);
 int command_passwd(object *pl, char *params);
+int do_harvest(object *pl, int dir, object *skill);
+void list_players(object* op, region* reg, partylist* party);
 /* c_move.c */
 int command_east(object *op, char *params);
 int command_north(object *op, char *params);
@@ -212,6 +210,7 @@ object *drop_object(object *op, object *tmp, uint32 nrof);
 void drop(object *op, object *tmp);
 int command_dropall(object *op, char *params);
 int command_drop(object *op, char *params);
+int command_empty(object *op, char *params);
 int command_examine(object *op, char *params);
 object *find_marked_object(object *op);
 int command_mark(object *op, char *params);
@@ -225,6 +224,7 @@ int command_rename_item(object *op, char *params);
 int command_lock_item(object *op, char *params);
 /* c_party.c */
 partylist *get_firstparty(void);
+partylist *form_party(object *op, const char *params);
 void remove_party(partylist *target_party);
 void obsolete_parties(void);
 int confirm_party_password(object *op);
@@ -232,6 +232,7 @@ void receive_party_password(object *op, char k);
 void send_party_message(object *op, char *msg);
 int command_gsay(object *op, char *params);
 int command_party(object *op, char *params);
+int command_party_rejoin(object *op, char *params);
 /* c_range.c */
 int command_invoke(object *op, char *params);
 int command_cast(object *op, char *params);
@@ -292,12 +293,14 @@ int command_stack_list(object *op, char *params);
 int command_stack_clear(object *op, char *params);
 int command_diff(object *op, char *params);
 int command_insert_into(object *op, char *params);
+int command_style_map_info(object *op, char *params);
+int command_follow(object *op, char *params);
 /* commands.c */
 void init_commands(void);
 command_function find_oldsocket_command(char *cmd);
 command_function find_oldsocket_command2(char *cmd);
 /* daemon.c */
-FILE *become_daemon(const char *filename);
+void become_daemon(void);
 /* disease.c */
 int move_disease(object *disease);
 int infect_object(object *victim, object *disease, int force);
@@ -307,7 +310,7 @@ int cure_disease(object *sufferer, object *caster);
 /* egoitem.c */
 int apply_power_crystal(object *op, object *crystal);
 /* hiscore.c */
-void check_score(object *op);
+void check_score(object *op, int quiet);
 void display_high_score(object *op, int max, const char *match);
 /* gods.c */
 object *find_god(const char *name);
@@ -329,7 +332,7 @@ void fatal_signal(int make_core, int close_sockets);
 racelink *find_racelink(const char *name);
 /* login.c */
 void emergency_save(int flag);
-void delete_character(const char *name, int new);
+void delete_character(const char *name);
 int verify_player(const char *name, char *password);
 int check_name(player *me, const char *name);
 void destroy_object(object *op);
@@ -420,6 +423,7 @@ int plugins_remove_plugin(const char *id);
 crossfire_plugin *plugins_find_plugin(const char *id);
 void plugins_display_list(object *op);
 void *cfapi_system_find_animation(int *type, ...);
+void *cfapi_system_find_face(int *type, ...);
 void *cfapi_system_strdup_local(int *type, ...);
 void *cfapi_system_register_global_event(int *type, ...);
 void *cfapi_system_unregister_global_event(int *type, ...);
@@ -428,6 +432,9 @@ void *cfapi_system_remove_string(int *type, ...);
 void *cfapi_system_check_path(int *type, ...);
 void *cfapi_system_re_cmp(int *type, ...);
 void *cfapi_system_directory(int *type, ...);
+void *cfapi_get_time(int *type, ...);
+void *cfapi_timer_create(int *type, ...);
+void *cfapi_timer_destroy(int *type, ...);
 void *cfapi_log(int *type, ...);
 void *cfapi_map_get_map(int *type, ...);
 void *cfapi_map_has_been_loaded(int *type, ...);
@@ -439,8 +446,8 @@ void *cfapi_map_update_position(int *type, ...);
 void *cfapi_map_delete_map(int *type, ...);
 void *cfapi_map_message(int *type, ...);
 void *cfapi_map_get_object_at(int *type, ...);
-void *cfapi_map_get_flags(int *type, ...);
 void *cfapi_map_present_arch_by_name(int *type, ...);
+void *cfapi_map_change_light(int *type, ...);
 void *cfapi_object_move(int *type, ...);
 void *cfapi_object_get_key(int *type, ...);
 void *cfapi_object_set_key(int *type, ...);
@@ -483,22 +490,20 @@ void *cfapi_object_transfer(int *type, ...);
 void *cfapi_object_find_archetype_inside(int *type, ...);
 void *cfapi_object_drop(int *type, ...);
 void *cfapi_object_take(int *type, ...);
+void *cfapi_object_change_abil(int *type, ...);
 void *cfapi_object_say(int *type, ...);
-void *cfapi_object_speak(int *type, ...);
 void *cfapi_player_find(int *type, ...);
 void *cfapi_player_message(int *type, ...);
-void *cfapi_player_send_inventory(int *type, ...);
+void *cfapi_object_change_exp(int *type, ...);
 void *cfapi_player_can_pay(int *type, ...);
 void *cfapi_object_teleport(int *type, ...);
 void *cfapi_object_pickup(int *type, ...);
-void *cfapi_archetype_get_first(int *type, ...);
 void *cfapi_archetype_get_property(int *type, ...);
 void *cfapi_party_get_property(int *type, ...);
 void *cfapi_region_get_property(int *type, ...);
-void *cfapi_get_time(int *type, ...);
-void *cfapi_timer_create(int *type, ...);
-void *cfapi_timer_destroy(int *type, ...);
 void *cfapi_friendlylist_get_next(int *type, ...);
+void *cfapi_set_random_map_variable(int *type, ...);
+void *cfapi_generate_random_map(int *type, ...);
 command_array_struct *find_plugin_command(char *cmd, object *op);
 int initPlugins(void);
 /* resurrection.c */
@@ -637,14 +642,14 @@ void spell_failure(object *op, int failure, int power, object *skill);
 int cast_spell(object *op, object *caster, int dir, object *spell_ob, char *stringarg);
 void move_spell_effect(object *op);
 void apply_spell_effect(object *spell, object *victim);
-void store_spell_expiry(object* spell);
-void check_spell_expiry(object* spell);
+void store_spell_expiry(object *spell);
+void check_spell_expiry(object *spell);
 /* swamp.c */
 void walk_on_deep_swamp(object *op, object *victim);
 void move_deep_swamp(object *op);
 /* swap.c */
 void read_map_log(void);
-void swap_map(mapstruct *map);
+int swap_map(mapstruct *map);
 void check_active_maps(void);
 void swap_below_max(char *except_level);
 int players_on_map(mapstruct *m, int show_all);
@@ -655,6 +660,7 @@ void remove_door2(object *op);
 object *stop_item(object *op);
 void fix_stopped_item(object *op, mapstruct *map, object *originator);
 object *fix_stopped_arrow(object *op);
+int free_no_drop(object *op);
 void move_arrow(object *op);
 void move_teleporter(object *op);
 void move_player_changer(object *op);
@@ -669,7 +675,7 @@ void cftimer_process_timers(void);
 int cftimer_create(int id, long delay, object *ob, int mode);
 int cftimer_destroy(int id);
 int cftimer_find_free_id(void);
-void cftimer_init();
+void cftimer_init(void);
 /* weather.c */
 void set_darkness_map(mapstruct *m);
 void tick_the_clock(void);
@@ -684,7 +690,6 @@ void start_info(object *op);
 char *crypt_string(char *str, char *salt);
 int check_password(char *typed, char *crypted);
 void enter_player_savebed(object *op);
-void leave_map(object *op);
 void set_map_timeout(mapstruct *oldmap);
 void enter_exit(object *op, object *exit_ob);
 void clean_tmp_files(void);
