@@ -7,8 +7,8 @@
 /* Headers needed.                                                           */
 /*****************************************************************************/
 
-#ifndef TIMERS_H_
-#define TIMERS_H_
+#ifndef TIMERS_H
+#define TIMERS_H
 
 #include <global.h>
 #include <object.h>
@@ -52,7 +52,7 @@ typedef struct _cftimer
 {
     int     mode;
     long    delay;
-    object* ob;
+    tag_t   ob_tag;
 } cftimer;
 
 #define MAX_TIMERS 1000
@@ -64,5 +64,4 @@ cftimer timers_table[MAX_TIMERS];
 #define TIMER_ERR_OBJ      -2
 #define TIMER_ERR_MODE     -3
 
-#endif /* TIMERS_H_ */
-
+#endif /* TIMERS_H */
