@@ -32,7 +32,6 @@ static PyObject* Map_GetName(Crossfire_Map* whoptr, void* closure);
 static PyObject* Map_GetResetTime(Crossfire_Map* whoptr, void* closure);
 static PyObject* Map_GetResetTimeout(Crossfire_Map* whoptr, void* closure);
 static PyObject* Map_GetPlayers(Crossfire_Map* whoptr, void* closure);
-static PyObject* Map_GetLight(Crossfire_Map* whoptr, void* closure);
 static PyObject* Map_GetDarkness(Crossfire_Map* whoptr, void* closure);
 static PyObject* Map_GetWidth(Crossfire_Map* whoptr, void* closure);
 static PyObject* Map_GetHeight(Crossfire_Map* whoptr, void* closure);
@@ -48,6 +47,9 @@ static PyObject* Map_GetWPartX(Crossfire_Map* whoptr, void* closure);
 static PyObject* Map_GetWPartY(Crossfire_Map* whoptr, void* closure);
 static PyObject* Map_GetMessage(Crossfire_Map* whoptr, void* closure);
 static PyObject* Map_GetRegion(Crossfire_Map* whoptr, void* closure);
+static PyObject* Map_GetUnique(Crossfire_Map* whoptr, void* closure);
+
+static int Map_SetPath(Crossfire_Map* whoptr, PyObject* value, void* closure);
 
 static PyObject* Map_Message(Crossfire_Map* map, PyObject* args);
 static PyObject* Map_GetFirstObjectAt(Crossfire_Map* map, PyObject* args);
@@ -55,6 +57,7 @@ static PyObject* Map_CreateObject(Crossfire_Map* map, PyObject* args);
 static PyObject* Map_Check(Crossfire_Map* map, PyObject* args);
 static PyObject* Map_Next(Crossfire_Map* map, PyObject* args);
 static PyObject* Map_Insert(Crossfire_Map* map, PyObject* args);
+static PyObject* Map_ChangeLight(Crossfire_Map* map, PyObject* args);
 
 static int Map_InternalCompare(Crossfire_Map* left, Crossfire_Map* right);
 
@@ -66,13 +69,13 @@ static PyObject *Crossfire_Map_new(PyTypeObject *type, PyObject *args, PyObject
 /* Python binding */
 static PyGetSetDef Map_getseters[] = {
     {"Difficulty",      (getter)Map_GetDifficulty,  NULL, NULL, NULL },
-    {"Path",            (getter)Map_GetPath,        NULL, NULL, NULL },
+    {"Path",            (getter)Map_GetPath,        (setter)Map_SetPath, NULL, NULL },
     {"TempName",        (getter)Map_GetTempName,    NULL, NULL, NULL },
     {"Name",            (getter)Map_GetName,        NULL, NULL, NULL },
     {"ResetTime",       (getter)Map_GetResetTime,   NULL, NULL, NULL },
     {"ResetTimeout",    (getter)Map_GetResetTimeout,NULL, NULL, NULL },
     {"Players",         (getter)Map_GetPlayers,     NULL, NULL, NULL },
-    {"Light",           (getter)Map_GetLight,       NULL, NULL, NULL },
+    {"Light",           (getter)Map_GetDarkness,    NULL, NULL, NULL },
     {"Darkness",        (getter)Map_GetDarkness,    NULL, NULL, NULL },
     {"Width",           (getter)Map_GetWidth,       NULL, NULL, NULL },
     {"Height",          (getter)Map_GetHeight,      NULL, NULL, NULL },
@@ -88,6 +91,7 @@ static PyGetSetDef Map_getseters[] = {
     {"WPartY",          (getter)Map_GetWPartY,      NULL, NULL, NULL },
     {"Message",         (getter)Map_GetMessage,     NULL, NULL, NULL },
     {"Region",          (getter)Map_GetRegion,     NULL, NULL, NULL },
+    {"Unique",          (getter)Map_GetUnique,     NULL, NULL, NULL },
     { NULL, NULL, NULL, NULL, NULL }
 };
 
@@ -98,6 +102,7 @@ static PyMethodDef MapMethods[] = {
     { "Check",    (PyCFunction)Map_Check, METH_VARARGS},
     { "Next",    (PyCFunction)Map_Next, METH_VARARGS},
     { "Insert",  (PyCFunction)Map_Insert, METH_VARARGS},
+    { "ChangeLight", (PyCFunction)Map_ChangeLight, METH_VARARGS},
     {NULL, NULL, 0}
 };
 
