@@ -28,6 +28,7 @@
 static PyObject* Object_GetName(Crossfire_Object* whoptr, void* closure);
 static PyObject* Object_GetNamePl(Crossfire_Object* whoptr, void* closure);
 static PyObject* Object_GetTitle(Crossfire_Object* whoptr, void* closure);
+static PyObject* Object_GetRace(Crossfire_Object* whoptr, void* closure);
 static PyObject* Object_GetMap(Crossfire_Object* whoptr, void* closure);
 static PyObject* Object_GetCha(Crossfire_Object* whoptr, void* closure);
 static PyObject* Object_GetCon(Crossfire_Object* whoptr, void* closure);
@@ -48,7 +49,10 @@ static PyObject* Object_GetWC(Crossfire_Object* whoptr, void* closure);
 static PyObject* Object_GetDam(Crossfire_Object* whoptr, void* closure);
 static PyObject* Object_GetLuck(Crossfire_Object* whoptr, void* closure);
 static PyObject* Object_GetMessage(Crossfire_Object* whoptr, void* closure);
+static PyObject* Object_GetSkill(Crossfire_Object* whoptr, void* closure);
 static PyObject* Object_GetExp(Crossfire_Object* whoptr, void* closure);
+static PyObject* Object_GetPermExp(Crossfire_Object* whoptr, void* closure);
+static PyObject* Object_GetExpMul(Crossfire_Object* whoptr, void* closure);
 static PyObject* Object_GetSlaying(Crossfire_Object* whoptr, void* closure);
 static PyObject* Object_GetCursed(Crossfire_Object* whoptr, void* closure);
 static PyObject* Object_GetDamned(Crossfire_Object* whoptr, void* closure);
@@ -73,6 +77,8 @@ static PyObject* Object_GetLastGrace(Crossfire_Object* whoptr, void* closure);
 static PyObject* Object_GetLastEat(Crossfire_Object* whoptr, void* closure);
 static PyObject* Object_GetLevel(Crossfire_Object* whoptr, void* closure);
 static PyObject* Object_GetFace(Crossfire_Object* whoptr, void* closure);
+static PyObject* Object_GetAnim(Crossfire_Object* whoptr, void* closure);
+static PyObject* Object_GetAnimSpeed(Crossfire_Object* whoptr, void* closure);
 static PyObject* Object_GetAttackType(Crossfire_Object* whoptr, void* closure);
 static PyObject* Object_GetBeenApplied(Crossfire_Object* whoptr, void* closure);
 static PyObject* Object_GetIdentified(Crossfire_Object* whoptr, void* closure);
@@ -81,7 +87,6 @@ static PyObject* Object_GetDM(Crossfire_Object* whoptr, void* closure);
 static PyObject* Object_GetWasDM(Crossfire_Object* whoptr, void* closure);
 static PyObject* Object_GetApplied(Crossfire_Object* whoptr, void* closure);
 static PyObject* Object_GetUnpaid(Crossfire_Object* whoptr, void* closure);
-static PyObject* Object_GetFlying(Crossfire_Object* whoptr, void* closure);
 static PyObject* Object_GetMonster(Crossfire_Object* whoptr, void* closure);
 static PyObject* Object_GetFriendly(Crossfire_Object* whoptr, void* closure);
 static PyObject* Object_GetGenerator(Crossfire_Object* whoptr, void* closure);
@@ -114,8 +119,6 @@ static PyObject* Object_GetCanCastSpell(Crossfire_Object* whoptr, void* closure)
 static PyObject* Object_GetReflectSpells(Crossfire_Object* whoptr, void* closure);
 static PyObject* Object_GetReflectMissiles(Crossfire_Object* whoptr, void* closure);
 static PyObject* Object_GetUnique(Crossfire_Object* whoptr, void* closure);
-static PyObject* Object_GetCanPickUp(Crossfire_Object* whoptr, void* closure);
-static PyObject* Object_GetCanPassThru(Crossfire_Object* whoptr, void* closure);
 static PyObject* Object_GetRunAway(Crossfire_Object* whoptr, void* closure);
 static PyObject* Object_GetScared(Crossfire_Object* whoptr, void* closure);
 static PyObject* Object_GetUndead(Crossfire_Object* whoptr, void* closure);
@@ -141,15 +144,27 @@ static PyObject* Object_GetMoveOff(Crossfire_Object* whoptr, void* closure);
 static PyObject* Object_GetMoveSlow(Crossfire_Object* whoptr, void* closure);
 static PyObject* Object_GetMoveSlowPenalty(Crossfire_Object* whoptr, void* closure);
 static PyObject* Object_GetOwner(Crossfire_Object* whoptr, void* closure);
+static PyObject* Object_GetEnemy(Crossfire_Object* whoptr, void* closure);
+static PyObject* Object_GetCount(Crossfire_Object* whoptr, void* closure);
+static PyObject* Object_GetGodGiven(Crossfire_Object* whoptr, void* closure);
+static PyObject* Object_GetIsPet(Crossfire_Object* whoptr, void* closure);
+static PyObject* Object_GetAttackMovement(Crossfire_Object* whoptr, void* closure);
+static PyObject* Object_GetDuration(Crossfire_Object* whoptr, void* closure);
+static PyObject* Object_GetNoDamage(Crossfire_Object* whoptr, void* closure);
+static PyObject* Object_GetRandomMovement(Crossfire_Object* whoptr, void* closure);
 
 static int Object_SetMessage(Crossfire_Object* whoptr, PyObject* value, void* closure);
+static int Object_SetExp(Crossfire_Object* whoptr, PyObject* value, void* closure);
 static int Object_SetName(Crossfire_Object* whoptr, PyObject* value, void* closure);
 static int Object_SetNamePl(Crossfire_Object* whoptr, PyObject* value, void* closure);
 static int Object_SetTitle(Crossfire_Object* whoptr, PyObject* value, void* closure);
+static int Object_SetRace(Crossfire_Object* whoptr, PyObject* value, void* closure);
 static int Object_SetMap(Crossfire_Object* whoptr, PyObject* value, void* closure);
 static int Object_SetSlaying(Crossfire_Object* whoptr, PyObject* value, void* closure);
+static int Object_SetSkill(Crossfire_Object* whoptr, PyObject* value, void* closure);
 static int Object_SetCursed(Crossfire_Object* whoptr, PyObject* value, void* closure);
 static int Object_SetDamned(Crossfire_Object* whoptr, PyObject* value, void* closure);
+static int Object_SetApplied(Crossfire_Object* whoptr, PyObject* value, void* closure);
 static int Object_SetStr(Crossfire_Object* whoptr, PyObject* value, void* closure);
 static int Object_SetDex(Crossfire_Object* whoptr, PyObject* value, void* closure);
 static int Object_SetCon(Crossfire_Object* whoptr, PyObject* value, void* closure);
@@ -182,7 +197,6 @@ static int Object_SetAttackType(Crossfire_Object* whoptr, PyObject* value, void*
 static int Object_SetUnaggressive(Crossfire_Object* whoptr, PyObject* value, void* closure);
 static int Object_SetPickable(Crossfire_Object* whoptr, PyObject* value, void* closure);
 static int Object_SetInvisible(Crossfire_Object* whoptr, PyObject* value, void* closure);
-static int Object_SetFlying(Crossfire_Object* whoptr, PyObject* value, void* closure);
 static int Object_SetUnpaid(Crossfire_Object* whoptr, PyObject* value, void* closure);
 static int Object_SetFriendly(Crossfire_Object* whoptr, PyObject* value, void* closure);
 static int Object_SetCanSeeInvisible(Crossfire_Object* whoptr, PyObject* value, void* closure);
@@ -201,7 +215,6 @@ static int Object_SetKnownMagical(Crossfire_Object* whoptr, PyObject* value, voi
 static int Object_SetReflectSpells(Crossfire_Object* whoptr, PyObject* value, void* closure);
 static int Object_SetReflectMissiles(Crossfire_Object* whoptr, PyObject* value, void* closure);
 static int Object_SetUnique(Crossfire_Object* whoptr, PyObject* value, void* closure);
-static int Object_SetCanPassThru(Crossfire_Object* whoptr, PyObject* value, void* closure);
 static int Object_SetRunAway(Crossfire_Object* whoptr, PyObject* value, void* closure);
 static int Object_SetScared(Crossfire_Object* whoptr, PyObject* value, void* closure);
 static int Object_SetUndead(Crossfire_Object* whoptr, PyObject* value, void* closure);
@@ -212,16 +225,23 @@ static int Object_SetOnlyAttack(Crossfire_Object* whoptr, PyObject* value, void*
 static int Object_SetMakeInvisible(Crossfire_Object* whoptr, PyObject* value, void* closure);
 static int Object_SetValue(Crossfire_Object* whoptr, PyObject* value, void* closure);
 static int Object_SetFace(Crossfire_Object* whoptr, PyObject* value, void* closure);
+static int Object_SetAnim(Crossfire_Object* whoptr, PyObject* value, void* closure);
+static int Object_SetAnimSpeed(Crossfire_Object* whoptr, PyObject* value, void* closure);
 static int Object_SetNoSave(Crossfire_Object* whoptr, PyObject* value, void* closure);
 static int Object_SetOwner(Crossfire_Object* whoptr, PyObject* value, void* closure);
-
+static int Object_SetEnemy(Crossfire_Object* whoptr, PyObject* value, void* closure);
+static int Object_SetGodGiven(Crossfire_Object* whoptr, PyObject* value, void* closure);
+static int Object_SetIsPet(Crossfire_Object* whoptr, PyObject* value, void* closure);
+static int Object_SetAttackMovement(Crossfire_Object* whoptr, PyObject* value, void* closure);
+static int Object_SetDuration(Crossfire_Object* whoptr, PyObject* value, void* closure);
+static int Object_SetNoDamage(Crossfire_Object* whoptr, PyObject* value, void* closure);
+static int Object_SetRandomMovement(Crossfire_Object* whoptr, PyObject* value, void* closure);
 
 static PyObject* Crossfire_Object_Remove( Crossfire_Object* who, PyObject* args );
 static PyObject* Crossfire_Object_Apply( Crossfire_Object* who, PyObject* args );
 static PyObject* Crossfire_Object_Drop( Crossfire_Object* who, PyObject* args );
 static PyObject* Crossfire_Object_Fix( Crossfire_Object* who, PyObject* args );
 static PyObject* Crossfire_Object_Say( Crossfire_Object* who, PyObject* args );
-static PyObject* Crossfire_Object_Speak( Crossfire_Object* who, PyObject* args );
 static PyObject* Crossfire_Object_Pickup( Crossfire_Object* who, PyObject* args );
 static PyObject* Crossfire_Object_Take( Crossfire_Object* who, PyObject* args );
 static PyObject* Crossfire_Object_Teleport( Crossfire_Object* who, PyObject* args );
@@ -229,6 +249,7 @@ static PyObject* Crossfire_Object_Reposition( Crossfire_Object* who, PyObject* a
 
 static PyObject* Crossfire_Object_QueryName( Crossfire_Object* who, PyObject* args );
 static PyObject* Crossfire_Object_GetResist( Crossfire_Object* who, PyObject* args );
+static PyObject* Crossfire_Object_SetResist( Crossfire_Object* who, PyObject* args );
 static PyObject* Crossfire_Object_ActivateRune( Crossfire_Object* who, PyObject* args );
 static PyObject* Crossfire_Object_CheckTrigger( Crossfire_Object* who, PyObject* args );
 static PyObject* Crossfire_Object_QueryCost( Crossfire_Object* who, PyObject* args );
@@ -248,6 +269,8 @@ static PyObject* Crossfire_Object_ReadKey(Crossfire_Object* who, PyObject* args)
 static PyObject* Crossfire_Object_WriteKey(Crossfire_Object* who, PyObject* args);
 static PyObject* Crossfire_Object_CreateTimer(Crossfire_Object* who, PyObject* args);
 static PyObject* Crossfire_Object_AddExp(Crossfire_Object* who, PyObject* args);
+static PyObject* Crossfire_Object_Move(Crossfire_Object* who, PyObject* args);
+static PyObject* Crossfire_Object_ChangeAbil(Crossfire_Object* who, PyObject* args);
 
 static int Crossfire_Object_InternalCompare(Crossfire_Object* left, Crossfire_Object* right);
 
@@ -260,6 +283,8 @@ static PyGetSetDef Object_getseters[] = {
     { "Name",       (getter)Object_GetName,     (setter)Object_SetName, NULL, NULL },
     { "NamePl",     (getter)Object_GetNamePl,   (setter)Object_SetNamePl, NULL, NULL },
     { "Title",      (getter)Object_GetTitle,    (setter)Object_SetTitle, NULL, NULL },
+    { "Race",       (getter)Object_GetRace,     (setter)Object_SetRace, NULL, NULL },
+    { "Skill",      (getter)Object_GetSkill,    (setter)Object_SetSkill, NULL, NULL },
     { "Map",        (getter)Object_GetMap,      (setter)Object_SetMap, NULL, NULL },
     { "Cha",        (getter)Object_GetCha,      (setter)Object_SetCha, NULL, NULL },
     { "Con",        (getter)Object_GetCon,      (setter)Object_SetCon, NULL, NULL },
@@ -279,7 +304,9 @@ static PyGetSetDef Object_getseters[] = {
     { "WC",         (getter)Object_GetWC,       (setter)Object_SetWC, NULL, NULL },
     { "Dam",        (getter)Object_GetDam,      (setter)Object_SetDam, NULL, NULL },
     { "Luck",       (getter)Object_GetLuck,     NULL, NULL, NULL },
-    { "Exp",        (getter)Object_GetExp,      NULL, NULL, NULL },
+    { "Exp",        (getter)Object_GetExp,      (setter)Object_SetExp, NULL, NULL },
+    { "ExpMul",     (getter)Object_GetExpMul,   NULL, NULL, NULL },
+    { "PermExp",    (getter)Object_GetPermExp,  NULL, NULL, NULL },
     { "Message",    (getter)Object_GetMessage,  (setter)Object_SetMessage, NULL, NULL },
     { "Slaying",    (getter)Object_GetSlaying,  (setter)Object_SetSlaying, NULL, NULL },
     { "Cursed",     (getter)Object_GetCursed,   (setter)Object_SetCursed, NULL, NULL },
@@ -305,15 +332,16 @@ static PyGetSetDef Object_getseters[] = {
     { "LastEat",    (getter)Object_GetLastEat,  (setter)Object_SetLastEat, NULL, NULL },
     { "Level",      (getter)Object_GetLevel,    NULL, NULL, NULL },
     { "Face",       (getter)Object_GetFace,     (setter)Object_SetFace, NULL, NULL },
+    { "Anim",       (getter)Object_GetAnim,     (setter)Object_SetAnim, NULL, NULL },
+    { "AnimSpeed",  (getter)Object_GetAnimSpeed,(setter)Object_SetAnimSpeed, NULL, NULL },
     { "AttackType", (getter)Object_GetAttackType, (setter)Object_SetAttackType, NULL, NULL },
     { "BeenApplied",(getter)Object_GetBeenApplied,NULL, NULL, NULL },
     { "Identified", (getter)Object_GetIdentified, NULL, NULL, NULL },
     { "Alive",      (getter)Object_GetAlive,    NULL, NULL, NULL },
     { "DungeonMaster",(getter)Object_GetDM,     NULL, NULL, NULL },
     { "WasDungeonMaster",(getter)Object_GetWasDM, NULL, NULL, NULL },
-    { "Applied",    (getter)Object_GetApplied,  NULL, NULL, NULL },
+    { "Applied",    (getter)Object_GetApplied,  (setter)Object_SetApplied, NULL, NULL },
     { "Unpaid",     (getter)Object_GetUnpaid,   (setter)Object_SetUnpaid, NULL, NULL },
-    { "Flying",     (getter)Object_GetFlying,   (setter)Object_SetFlying, NULL, NULL },
     { "Monster",    (getter)Object_GetMonster,  NULL, NULL, NULL },
     { "Friendly",   (getter)Object_GetFriendly, (setter)Object_SetFriendly, NULL, NULL },
     { "Generator",  (getter)Object_GetGenerator,NULL, NULL, NULL },
@@ -346,8 +374,6 @@ static PyGetSetDef Object_getseters[] = {
     { "ReflectSpells",  (getter)Object_GetReflectSpells,(setter)Object_SetReflectSpells, NULL, NULL },
     { "ReflectMissiles",(getter)Object_GetReflectMissiles,(setter)Object_SetReflectMissiles, NULL, NULL },
     { "Unique",         (getter)Object_GetUnique,       (setter)Object_SetUnique, NULL, NULL },
-    { "CanPickUp",      (getter)Object_GetCanPickUp,    NULL, NULL, NULL },
-    { "CanPassThru",    (getter)Object_GetCanPassThru,  (setter)Object_SetCanPassThru, NULL, NULL },
     { "RunAway",        (getter)Object_GetRunAway,      (setter)Object_SetRunAway, NULL, NULL },
     { "Scared",         (getter)Object_GetScared,       (setter)Object_SetScared, NULL, NULL },
     { "Undead",         (getter)Object_GetUndead,       (setter)Object_SetUndead, NULL, NULL },
@@ -373,6 +399,14 @@ static PyGetSetDef Object_getseters[] = {
     { "MoveSlow",       (getter)Object_GetMoveSlow,     NULL, NULL, NULL },
     { "MoveSlowPenalty",(getter)Object_GetMoveSlowPenalty,  NULL, NULL, NULL },
     { "Owner",          (getter)Object_GetOwner,        (setter)Object_SetOwner, NULL, NULL },
+    { "Enemy",          (getter)Object_GetEnemy,        (setter)Object_SetEnemy, NULL, NULL },
+    { "Count",          (getter)Object_GetCount,        NULL, NULL, NULL },
+    { "GodGiven",       (getter)Object_GetGodGiven,     (setter)Object_SetGodGiven, NULL, NULL },
+    { "IsPet",          (getter)Object_GetIsPet,     (setter)Object_SetIsPet, NULL, NULL },
+    { "AttackMovement", (getter)Object_GetAttackMovement, (setter)Object_SetAttackMovement, NULL, NULL },
+    { "Duration",       (getter)Object_GetDuration,     (setter)Object_SetDuration, NULL, NULL },
+    { "NoDamage",       (getter)Object_GetNoDamage, (setter)Object_SetNoDamage, NULL, NULL },
+    { "RandomMovement", (getter)Object_GetRandomMovement, (setter)Object_SetRandomMovement, NULL, NULL },
     { NULL, NULL, NULL, NULL, NULL }
 };
 
@@ -388,6 +422,7 @@ static PyMethodDef ObjectMethods[] = {
     { "Reposition",     (PyCFunction)Crossfire_Object_Reposition,   METH_VARARGS},
     { "QueryName",      (PyCFunction)Crossfire_Object_QueryName,    METH_VARARGS},
     { "GetResist",      (PyCFunction)Crossfire_Object_GetResist,    METH_VARARGS},
+    { "SetResist",      (PyCFunction)Crossfire_Object_SetResist,    METH_VARARGS},
     { "ActivateRune",   (PyCFunction)Crossfire_Object_ActivateRune, METH_VARARGS},
     { "CheckTrigger",   (PyCFunction)Crossfire_Object_CheckTrigger, METH_VARARGS},
     { "QueryCost",      (PyCFunction)Crossfire_Object_QueryCost,    METH_VARARGS},
@@ -407,6 +442,8 @@ static PyMethodDef ObjectMethods[] = {
     { "WriteKey",       (PyCFunction)Crossfire_Object_WriteKey,     METH_VARARGS},
     { "CreateTimer",    (PyCFunction)Crossfire_Object_CreateTimer,  METH_VARARGS},
     { "AddExp",         (PyCFunction)Crossfire_Object_AddExp,       METH_VARARGS},
+    { "Move",           (PyCFunction)Crossfire_Object_Move,         METH_VARARGS},
+    { "ChangeAbil",     (PyCFunction)Crossfire_Object_ChangeAbil,   METH_VARARGS},
     {NULL, NULL, 0}
 };
 
