@@ -41,29 +41,44 @@
 
 extern int cf_init_plugin( f_plug_api getHooks );
 extern void cf_system_register_global_event( int event, const char* name, f_plug_api hook );
+extern void cf_system_unregister_global_event( int event, const char* name );
 
 /* General functions */
-extern char*        cf_add_string( char* str );
-extern void         cf_free_string( char* str );
-extern char*        cf_strdup_local(char* txt);
-extern char*        cf_get_maps_directory( char* str );
-extern int          cf_find_animation(char* txt);
-extern void			cf_log( LogLevel logLevel, const char* format, ... );
+extern sstring      cf_add_string(const char* str);
+extern void         cf_free_string(sstring str);
+extern char*        cf_strdup_local(const char* str);
+extern char*        cf_get_maps_directory(const char *name, char* buf, int size);
+extern int          cf_find_animation(const char* txt);
+extern int          cf_find_face(const char* name, int error);
+extern void         cf_log( LogLevel logLevel, const char* format, ... );
 extern void         cf_get_time( timeofday_t* tod );
 extern int          cf_timer_create(object* ob, long delay, int mode);
 extern int          cf_timer_destroy(int id);
+extern const char*  cf_get_directory(int id);
+extern const char*  cf_re_cmp(const char *str, const char *regexp);
 
 /* Objects */
-extern void* cf_object_set_int_property(object* op, int propcode, int value);
-extern void* cf_object_set_object_property(object* op, int propcode, object* value);
-extern void* cf_object_set_float_property(object* op, int propcode, float value);
-extern void* cf_object_get_property(object* op, int propcode);
-extern void         cf_free_object( object* ob );
-extern void         cf_fix_object( object* pl );
-extern char*        cf_query_name( object* ob );
-extern char*        cf_query_name_pl( object* ob );
-extern char*        cf_query_base_name( object* ob, int plural );
-extern object*      cf_insert_ob_in_ob( object* ob, object* where );
+extern void         cf_object_set_int_property(object* op, int propcode, int value);
+extern int          cf_object_get_int_property(object* op, int propcode);
+extern long         cf_object_get_long_property(object* op, long propcode);
+extern void         cf_object_set_movetype_property(object* op, int propcode, MoveType value);
+extern MoveType     cf_object_get_movetype_property(object* op, int propcode);
+extern object*      cf_object_get_object_property(object* op, int propcode);
+extern void         cf_object_set_object_property(object* op, int propcode, object* value);
+extern float        cf_object_get_float_property(object* op, int propcode);
+extern void         cf_object_set_float_property(object* op, int propcode, float value);
+extern mapstruct*   cf_object_get_map_property(object* op, int propcode);
+extern archetype*   cf_object_get_archetype_property(object* op, int propcode);
+extern partylist*   cf_object_get_partylist_property(object* op, int propcode);
+extern sint64       cf_object_get_int64_property(object* op, int propcode);
+extern void         cf_object_set_int64_property(object* op, int propcode, sint64 value);
+extern double       cf_object_get_double_property(object* op, int propcode);
+extern sstring      cf_object_get_sstring_property(object* op, int propcode);
+extern char*        cf_object_get_string_property(object* op, int propcode, char* buf, int size);
+extern void         cf_fix_object(object* op);
+extern char*        cf_query_name(object* ob, char* name, int size);
+extern sstring      cf_query_name_pl( object* ob );
+extern char*        cf_query_base_name(object* ob, int plural, char* name, int size);
 extern const char*  cf_object_get_msg( object* );
 extern void         cf_object_set_weight( object* ob, int weight );
 extern int          cf_object_get_weight( object* ob );
@@ -73,62 +88,71 @@ extern int          cf_object_set_nrof( object*, int nrof );
 extern int          cf_object_get_nrof( object* );
 extern int          cf_object_get_flag( object* ob, int flag );
 extern void         cf_object_set_flag( object* ob, int flag, int value );
-extern object*      cf_object_insert_in_ob( object* ob, object* where );
-extern void*        cf_object_set_string_property(object* op, int propcode, char* value);
-extern void         cf_object_activate_rune( object* op , object* victim);
+extern object*      cf_object_insert_in_ob( object* op, object* where );
+extern void         cf_object_set_string_property(object* op, int propcode, const char* value);
+extern void         cf_spring_trap( object* op , object* victim);
 extern int          cf_object_check_trigger( object* op, object* cause );
-extern int          cf_object_query_money( object* op);
-extern int          cf_object_query_cost( object* op, object* who, int flags);
-extern void*        cf_object_set_string_property(object* op, int propcode, char* value);
-extern int          cf_object_cast_spell(object* caster, object* ctoo, int dir, object* sp, char* flags);
-extern void         cf_object_learn_spell(object* op, object* sp);
+extern int          cf_object_query_money(const object* op);
+extern int          cf_object_query_cost(const object *tmp, object *who, int flag);
+extern int          cf_object_cast_spell(object *op, object *caster,int dir,object *spell_ob, char *stringarg);
+extern void         cf_object_learn_spell(object *op, object *spell, int special_prayer);
 extern void         cf_object_forget_spell(object* op, object* sp);
-extern object*      cf_object_check_for_spell(object* op, char* spellname);
+extern object*      cf_object_check_for_spell(object *op, const char *name);
 extern int          cf_object_cast_ability(object* caster, object* ctoo, int dir, object* sp, char* flags);
 extern int          cf_object_pay_amount(object* op, uint64 amount);
-extern int          cf_object_pay_item(object* op, object* buyer);
-extern void*        cf_object_set_long_property(object* op, int propcode, long value);
-extern int          cf_object_transfer(object* op,int x,int y,int r,object* orig);
+extern int          cf_object_pay_item(object *op,object *pl);
+extern void         cf_object_set_long_property(object* op, int propcode, long value);
+extern int          cf_object_transfer(object *op, int x, int y, int randomly, object *originator);
 extern int          cf_object_out_of_map( object* op, int x, int y);
 extern void         cf_object_drop( object* op, object* author);
 extern void         cf_object_take( object* op, object* author);
 extern void         cf_object_say( object* op, char* msg);
-extern void         cf_object_speak( object* op, char* msg);
 extern object*      cf_object_insert_object(object* op, object* container);
 extern object*      cf_object_present_archname_inside(object* op, char* whatstr);
-extern void         cf_object_apply(object* op, object* author, int flags);
+extern int          cf_object_apply(object* op, object* author, int flags);
 extern void         cf_object_remove(object* op);
-extern void         cf_object_free(object* op);
+extern void         cf_object_free(object* ob);
 extern object*      cf_create_object(void);
 extern object*      cf_create_object_by_name( const char* name );
-extern int          cf_object_change_map(object* op, int x, int y, mapstruct* map);
+extern object*      cf_object_change_map(object* op, mapstruct* m, object* originator, int flag, int x, int y);
 extern int          cf_object_teleport( object* ob, mapstruct* map, int x, int y );
 extern void         cf_object_update( object* op, int flags);
 extern void         cf_object_pickup( object* op, object* what);
-extern char*        cf_object_get_key(object* op, char* keyname);
-extern void         cf_object_set_key(object* op, char* keyname, char* value, int add_key);
+extern const char*  cf_object_get_key(object* op, const char* keyname);
+extern int          cf_object_set_key(object* op, const char* keyname, const char* value, int add_key);
 extern sint16       cf_object_get_resistance(object* op, int rtype);
-extern void         cf_object_move(object* op, int dir, object*originator);
-extern void         cf_object_apply_below(object* op);
+extern void         cf_object_set_resistance(object* op, int rtype, sint16 value);
+extern int          cf_object_move(object* op, int dir, object*originator);
+extern void         cf_object_apply_below(object* pl);
 extern object*      cf_object_clone(object* op, int clonetype);
-extern void*        cf_object_set_experience(object* op, sint64 exp, const char* skill, int arg);
+extern void         cf_object_change_experience(object *op, sint64 exp, const char *skill_name, int flag);
+extern int          cf_object_change_abil(object* op, object* tmp);
 
 /* Maps */
-extern void*        cf_map_get_property(mapstruct* map, int propcode);
-extern void*        cf_map_set_int_property(mapstruct* map, int propcode, int value);
-extern mapstruct*   cf_map_get_map( char* name );
+/*extern void*        cf_map_get_property(mapstruct* map, int propcode);*/
+extern sstring cf_map_get_sstring_property(mapstruct* map, int propcode);
+extern mapstruct* cf_map_get_map_property(mapstruct* map, int propcode);
+extern region* cf_map_get_region_property(mapstruct* map, int propcode);
+extern int cf_map_get_int_property(mapstruct* map, int property);
+
+extern void         cf_map_set_int_property(mapstruct* map, int propcode, int value);
+extern void         cf_map_set_string_property(mapstruct* map, int propcode, const char* value);
+extern mapstruct*   cf_map_get_map(const char* name, int flags);
+extern mapstruct*   cf_get_empty_map(int sizex, int sizey);
 extern mapstruct*   cf_map_get_first(void);
+extern mapstruct*   cf_map_has_been_loaded(const char* name);
 extern void         cf_map_message(mapstruct* m, char* msg, int color);
 extern object*      cf_map_get_object_at(mapstruct* m, int x, int y);
 extern object*      cf_map_insert_object(mapstruct* where, object* op, int x, int y);
 extern object*      cf_map_present_arch_by_name(const char* str, mapstruct* map, int nx, int ny);
-extern int          cf_map_get_flags( mapstruct* map, mapstruct** nmap, sint16 x, sint16 y, sint16* nx, sint16* ny );
-extern object*      cf_map_insert_object_there(mapstruct* where, object* op, object* originator, int flags);
+extern int          cf_map_get_flags(mapstruct *oldmap, mapstruct **newmap, sint16 x, sint16 y, sint16 *nx, sint16 *ny);
+extern object*      cf_map_insert_object_there(object *op, mapstruct *m, object *originator, int flag);
 extern int          cf_map_get_difficulty(mapstruct* map);
 extern int          cf_map_get_reset_time(mapstruct* map);
 extern int          cf_map_get_reset_timeout(mapstruct* map);
 extern int          cf_map_get_players(mapstruct* map);
 extern int          cf_map_get_darkness(mapstruct* map);
+extern int          cf_map_get_light(mapstruct* map);
 extern int          cf_map_get_width(mapstruct* map);
 extern int          cf_map_get_height(mapstruct* map);
 extern int          cf_map_get_enter_x(mapstruct* map);
@@ -141,22 +165,26 @@ extern int          cf_map_get_winddir(mapstruct* map);
 extern int          cf_map_get_sky(mapstruct* map);
 extern int          cf_map_get_wpartx(mapstruct* map);
 extern int          cf_map_get_wparty(mapstruct* map);
+extern int          cf_map_change_light(mapstruct *m, int change);
+
+/* Random maps */
+extern int           cf_random_map_set_variable(RMParms* rp, const char* buf);
+extern mapstruct*    cf_random_map_generate(const char *OutFileName, RMParms *RP, char** use_layout);
 
 /* Players */
-extern char*        cf_player_get_ip(object* op);
+extern sstring      cf_player_get_ip(object* op);
 extern object*      cf_player_get_marked_item(object* op);
 extern void         cf_player_set_marked_item(object* op, object* ob);
-extern player*      cf_player_find(char* txt);
+extern player*      cf_player_find(const char* plname);
 extern void         cf_player_message(object* op, char* txt, int flags);
-extern object*      cf_player_send_inventory(object* op);
-extern void         cf_player_move(player* pl, int dir);
+extern int          cf_player_move(player* pl, int dir);
 extern partylist*   cf_player_get_party(object* op);
 extern void         cf_player_set_party(object* op, partylist* party);
 extern int          cf_player_can_pay(object* op);
 
 /* Archetypes */
 extern archetype*	cf_archetype_get_first(void);
-extern const char*  cf_archetype_get_name(archetype* arch);
+extern sstring      cf_archetype_get_name(archetype* arch);
 extern archetype*   cf_archetype_get_next(archetype* arch);
 extern archetype*   cf_archetype_get_more(archetype* arch);
 extern archetype*   cf_archetype_get_head(archetype* arch);
@@ -182,12 +210,6 @@ extern const char*  cf_region_get_message(region* reg);
 extern object*      cf_friendlylist_get_first(void);
 extern object*      cf_friendlylist_get_next(object* ob);
 
-/* temp */
-extern f_plug_api cfapi_object_get_property;
-extern f_plug_api cfapi_object_set_property;
-extern f_plug_api cfapi_object_apply;
-extern f_plug_api cfapi_object_remove;
-
 #ifdef WIN32
 
 struct timezone {
