@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_alchemy_c =
- *   "$Id: alchemy.c 4961 2006-09-21 05:14:18Z mwedel $";
+ *   "$Id: alchemy.c 7629 2007-12-01 10:58:41Z ryo_saeba $";
  */
 
 /*
@@ -376,17 +376,23 @@ static object * make_item_from_recipe(object *cauldron, recipe *rp) {
         return (object *) NULL;
     }
  
+    /* If item is already in container, we need to remove its weight, since it can change later on. */
+    if (item->env != NULL)
+        sub_weight(cauldron, item->weight * (item->nrof != 0 ? item->nrof : 1));
+
     /* Find the appropriate artifact template...*/
     if(strcmp(rp->title,"NONE")) { 
         if((art=locate_recipe_artifact(rp, rp_arch_index))==NULL) {
             LOG(llevError,"make_alchemy_item(): failed to locate recipe artifact.\n");
             LOG(llevDebug,"  --requested recipe: %s of %s.\n",rp->arch_name[0],rp->title);
             return (object *) NULL;
-	}
-	transmute_materialname(item, art->item);
+        }
+        transmute_materialname(item, art->item);
         give_artifact_abilities(item, art->item);
     }
- 
+    if (item->env != NULL)
+        add_weight(cauldron, item->weight * (item->nrof != 0 ? item->nrof : 1));
+
     if(QUERY_FLAG(cauldron,FLAG_CURSED)) SET_FLAG(item,FLAG_CURSED);
     if(QUERY_FLAG(cauldron,FLAG_DAMNED)) SET_FLAG(item,FLAG_DAMNED);
  
@@ -501,8 +507,11 @@ static void alchemy_failure_effect(object *op,object *cauldron,recipe *rp,int da
 	      	return;
 
 	if((tmp=attempt_recipe(op,cauldron,1,rp,-1,0))) { 
-	    if(!QUERY_FLAG(tmp,FLAG_CURSED)) /* curse it */
+	    if(!QUERY_FLAG(tmp,FLAG_CURSED)) { /* curse it */
 	   	SET_FLAG(tmp,FLAG_CURSED);
+                CLEAR_FLAG(tmp, FLAG_KNOWN_CURSED);
+                CLEAR_FLAG(tmp, FLAG_IDENTIFIED);
+            }
 
 	    /* the apply code for potions already deals with cursed
 	     * potions, so any code here is basically ignored.
@@ -578,8 +587,11 @@ static void alchemy_failure_effect(object *op,object *cauldron,recipe *rp,int da
 	return;
 
     } else if (level<100) {               	/* WHAMMY the CAULDRON */
-	if(!QUERY_FLAG(cauldron,FLAG_CURSED))
+	if(!QUERY_FLAG(cauldron,FLAG_CURSED)) {
 	    SET_FLAG(cauldron,FLAG_CURSED);
+            CLEAR_FLAG(cauldron, FLAG_KNOWN_CURSED);
+            CLEAR_FLAG(cauldron, FLAG_IDENTIFIED);
+        }
 	else cauldron->magic--;
 	cauldron->magic -= random_roll(0, 4, op, PREFER_LOW);
 	if(rndm(0, 1)) {
