@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_c_misc_c =
- *   "$Id: c_misc.c 5802 2007-03-17 22:21:45Z ryo_saeba $";
+ *   "$Id: c_misc.c 7491 2007-11-07 06:27:53Z akirschbaum $";
  */
 
 /*
@@ -31,6 +31,8 @@
 #ifndef __CEXTRACT__
 #include <sproto.h>
 #endif
+#include <assert.h>
+#include <sounds.h>
 
 extern weathermap_t **weathermap;
 
@@ -83,7 +85,7 @@ int command_body(object *op, char *params)
 {
     int i;
 
-    /* Too hard to try and make a header that lines everything up, so just 
+    /* Too hard to try and make a header that lines everything up, so just
      * give a description.
      */
     draw_ext_info(NDI_UNIQUE, 0, op,
@@ -94,7 +96,7 @@ int command_body(object *op, char *params)
 	MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_BODY,
 	"The second column is how many of those locations your body has.", NULL);
 
-    draw_ext_info(NDI_UNIQUE, 0, op, 
+    draw_ext_info(NDI_UNIQUE, 0, op,
 	MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_BODY,
 	"The third column is how many slots in that location are available.", NULL);
 
@@ -103,7 +105,7 @@ int command_body(object *op, char *params)
 	 * if body_info isn't also set.
 	 */
 	if (op->body_info[i] || op->body_used[i]) {
-	    draw_ext_info_format(NDI_UNIQUE, 0, op, 
+	    draw_ext_info_format(NDI_UNIQUE, 0, op,
 		MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_BODY,
 		"[fixed]%-30s %5d %5d",
 		"%-30s %5d %5d",
@@ -247,16 +249,16 @@ void malloc_info(object *op) {
     sum_used+=i;
 
     draw_ext_info_format(NDI_UNIQUE, 0,op, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_MALLOC,
-	"[fixed]%4ld artifacts        %8d", 
-	"%4ld artifacts        %8d", 
+	"[fixed]%4ld artifacts        %8d",
+	"%4ld artifacts        %8d",
 	nrofartifacts, i=(nrofartifacts*sizeof(artifact)));
 
     sum_alloc+=i;
     sum_used +=i;
 
     draw_ext_info_format(NDI_UNIQUE, 0,op, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_MALLOC,
-	  "[fixed]%4ld artifacts strngs %8d", 
-	  "%4ld artifacts strngs %8d", 
+	  "[fixed]%4ld artifacts strngs %8d",
+	  "%4ld artifacts strngs %8d",
 	  nrofallowedstr, i=(nrofallowedstr*sizeof(linked_char)));
 
     sum_alloc += i;
@@ -283,12 +285,12 @@ void malloc_info(object *op) {
 }
 
 /**
- * Pretty much identical to current map_info, but on a bigger scale 
+ * Pretty much identical to current map_info, but on a bigger scale
  * This function returns the name of the players current region, and
  * a description of it. It is there merely for flavour text.
  */
 void current_region_info(object *op) {
-    /* 
+    /*
      * Ok I /suppose/ I should write a seperate function for this, but it isn't
      * going to be /that/ slow, and won't get called much
      */
@@ -311,16 +313,16 @@ void current_map_info(object *op) {
 	return;
 
     draw_ext_info_format(NDI_UNIQUE, 0,op, MSG_TYPE_COMMAND, MSG_SUBTYPE_NONE,
-	"%s (%s) in %s", 
-	"%s (%s) in %s", 
+	"%s (%s) in %s",
+	"%s (%s) in %s",
 	 m->name, m->path, get_name_of_region_for_map(m));
 
     if (QUERY_FLAG(op,FLAG_WIZ)) {
 	draw_ext_info_format(NDI_UNIQUE, 0,op, MSG_TYPE_COMMAND, MSG_SUBTYPE_NONE,
 		"players:%d difficulty:%d size:%dx%d start:%dx%d timeout %ld",
 		"players:%d difficulty:%d size:%dx%d start:%dx%d timeout %ld",
-		 m->players, m->difficulty, 
-		 MAP_WIDTH(m), MAP_HEIGHT(m), 
+		 m->players, m->difficulty,
+		 MAP_WIDTH(m), MAP_HEIGHT(m),
 		 MAP_ENTER_X(m), MAP_ENTER_Y(m),
 		 MAP_TIMEOUT(m));
 
@@ -334,7 +336,7 @@ int command_malloc_verify(object *op, char *parms)
 {
     extern int malloc_verify(void);
 
-    if (!malloc_verify()) 
+    if (!malloc_verify())
 	draw_ext_info(NDI_UNIQUE, 0,op, MSG_TYPE_COMMAND, MSG_SUBTYPE_NONE,
 			  "Heap is corrupted.", NULL);
     else
@@ -349,7 +351,7 @@ int command_whereabouts(object *op, char *params) {
 
     region *reg;
     player *pl;
-    
+
     /*
      * reset the counter on the region, then use it to store the number of
      * players there.
@@ -380,7 +382,7 @@ int command_whereabouts(object *op, char *params) {
 	if(reg->counter>0) {
 	    draw_ext_info_format(NDI_UNIQUE, 0,op, MSG_TYPE_COMMAND, MSG_SUBTYPE_NONE,
 		 "%u players in %s",
-		 "%u players in %s", 
+		 "%u players in %s",
 		 reg->counter, get_region_longname(reg));
 	}
     return 1;
@@ -396,77 +398,98 @@ static int name_cmp (const chars_names *c1, const chars_names *c2) {
     return strcasecmp (c1->namebuf, c2->namebuf);
 }
 
-int command_who (object *op, char *params) {
+/**
+ * Displays the players in a region or party. If both are NULL, all players are listed.
+ *
+ * @param op
+ * who is asking for player list.
+ * @param reg
+ * region to display players of.
+ * @param party
+ * party to list.
+ */
+void list_players(object* op, region* reg, partylist* party) {
     player *pl;
     uint16 i;
-    region *reg;
     char* format;
     int num_players = 0, num_wiz = 0, num_afk = 0, num_bot = 0;
     chars_names *chars = NULL;
-    
-    /* 
-     * The who formats are defined in config to be blank. They should have been
-     * overridden by the settings file, if there are no entries however, it will
-     * have stayed blank. Since this probably isn't what is wanted, we will check if
-     * new formats have been specified, and if not we will use the old defaults.
-     */
-    if (!strcmp(settings.who_format,"")) 
-    	strcpy(settings.who_format, "%N_%T%t%h%d%b%n<%m>");
-    if (!strcmp(settings.who_wiz_format,"")) 
-    	strcpy(settings.who_wiz_format, "%N_%T%t%h%d%b%nLevel %l <%m>(@%i)(%c)");
+
     if (op == NULL || QUERY_FLAG(op, FLAG_WIZ))
-    	format=settings.who_wiz_format;
+        format=settings.who_wiz_format;
     else
-    	format=settings.who_format;
+        format=settings.who_format;
 
-    reg=get_region_from_string(params);
-    
     for (pl=first_player;pl!=NULL;pl=pl->next) {
-	if (pl->ob->map == NULL)
-	    continue;
-	if (pl->hidden && !QUERY_FLAG(op, FLAG_WIZ)) continue;
-	
-	if(!region_is_child_of_region(get_region_by_map(pl->ob->map),reg)) continue;
+        if (pl->ob->map == NULL)
+            continue;
+        if (pl->hidden && !QUERY_FLAG(op, FLAG_WIZ))
+            continue;
 
-	if (pl->state==ST_PLAYING || pl->state==ST_GET_PARTY_PASSWORD) {
+        if(reg && !region_is_child_of_region(get_region_by_map(pl->ob->map),reg))
+            continue;
+        if (party && pl->party != party)
+            continue;
 
-	    num_players++;
-	    chars = (chars_names *) realloc(chars, num_players*sizeof(chars_names));
+        if (pl->state==ST_PLAYING || pl->state==ST_GET_PARTY_PASSWORD) {
+
+            num_players++;
+            chars = (chars_names *) realloc(chars, num_players*sizeof(chars_names));
             if (chars == NULL) {
-	        draw_ext_info(NDI_UNIQUE, 0, op, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_WHO,
-			      "who failed - out of memory!", NULL);
-                return 0;
-            } 
-	    sprintf(chars[num_players-1].namebuf, "%s", pl->ob->name);
-	    chars[num_players-1].login_order = num_players;
+                draw_ext_info(NDI_UNIQUE, 0, op, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_WHO,
+                              "who failed - out of memory!", NULL);
+                return;
+            }
+            sprintf(chars[num_players-1].namebuf, "%s", pl->ob->name);
+            chars[num_players-1].login_order = num_players;
 
-	    /* Check for WIZ's & AFK's*/
-	    if (QUERY_FLAG(pl->ob,FLAG_WIZ))
-		num_wiz++;
+            /* Check for WIZ's & AFK's*/
+            if (QUERY_FLAG(pl->ob,FLAG_WIZ))
+                num_wiz++;
 
-	    if (QUERY_FLAG(pl->ob,FLAG_AFK))
-		num_afk++;
+            if (QUERY_FLAG(pl->ob,FLAG_AFK))
+                num_afk++;
 
-	    if (pl->socket.is_bot)
-		num_bot++;
-	}
+            if (pl->socket.is_bot)
+                num_bot++;
+        }
     }
     if (first_player != (player *) NULL) {
-    	if (reg == NULL) 
-	    draw_ext_info_format(NDI_UNIQUE, 0, op, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_WHO,
-		     "Total Players (%d) -- WIZ(%d) AFK(%d) BOT(%d)", 
-		     "Total Players (%d) -- WIZ(%d) AFK(%d) BOT(%d)", 
-		     num_players, num_wiz, num_afk, num_bot);
-	    else
-     		draw_ext_info_format(NDI_UNIQUE, 0, op, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_WHO,
-		     "Total Players in %s (%d) -- WIZ(%d) AFK(%d) BOT(%d)", 
-		     "Total Players in %s (%d) -- WIZ(%d) AFK(%d) BOT(%d)", 
-		     reg->longname?reg->longname:reg->name, num_players, num_wiz, num_afk, num_bot);
+        if (reg == NULL && party == NULL)
+            new_draw_info_format(NDI_UNIQUE, 0, op,
+                "Total Players (%d) -- WIZ(%d) AFK(%d) BOT(%d)",
+                num_players, num_wiz, num_afk, num_bot);
+        else if (party == NULL)
+            new_draw_info_format(NDI_UNIQUE, 0, op,
+                "Total Players in %s (%d) -- WIZ(%d) AFK(%d) BOT(%d)",
+                reg->longname?reg->longname:reg->name, num_players, num_wiz, num_afk, num_bot);
+        else
+            new_draw_info_format(NDI_UNIQUE, 0, op,
+                "Total Players in party %s (%d) -- WIZ(%d) AFK(%d) BOT(%d)",
+                party->partyname, num_players, num_wiz, num_afk, num_bot);
     }
     qsort (chars, num_players, sizeof(chars_names), (int (*)(const void *, const void *))name_cmp);
     for (i=0;i<num_players;i++)
-	display_who_entry(op, find_player(chars[i].namebuf), format);
-    free(chars);    
+        display_who_entry(op, find_player(chars[i].namebuf), format);
+    free(chars);
+}
+
+/**
+ * 'who' command.
+ *
+ * @param op
+ * player requesting the information.
+ * @param params
+ * optional region to limit the information to.
+ * @return
+ * 1.
+ */
+int command_who (object *op, char *params) {
+    region *reg;
+    reg=get_region_from_string(params);
+
+    list_players(op, reg, NULL);
+
     return 1;
 }
 
@@ -524,9 +547,9 @@ void display_who_entry(object *op, player *pl, const char *format) {
  * %	a literal %
  * _	a literal underscore
  */
-  
+
 void get_who_escape_code_value(char *return_val, const char letter, player *pl) {
-    
+
     switch (letter) {
 	case 'N' :	strcpy(return_val, pl->ob->name);
 			break;
@@ -543,7 +566,7 @@ void get_who_escape_code_value(char *return_val, const char letter, player *pl)
 	case 'c' :	sprintf(return_val,"%d",pl->ob->count);
 			break;
 
-	case 'n' :	strcpy(return_val, "\n"); 
+	case 'n' :	strcpy(return_val, "\n");
 			break;
 
 	case 'h' :	strcpy(return_val,pl->peaceful?"":" <Hostile>");
@@ -565,7 +588,7 @@ void get_who_escape_code_value(char *return_val, const char letter, player *pl)
 			break;
 
 	case 'M' :	strcpy(return_val,pl->ob->map->name?pl->ob->map->name:"Untitled");
-			break;		
+			break;
 
 	case 'r' :	strcpy(return_val,get_name_of_region_for_map(pl->ob->map));
 			break;
@@ -576,10 +599,10 @@ void get_who_escape_code_value(char *return_val, const char letter, player *pl)
 	case 'i' :	strcpy(return_val,pl->socket.host);
 			break;
 
-	case '%' :	strcpy(return_val, "%"); 
+	case '%' :	strcpy(return_val, "%");
 			break;
 
-	case '_' :	strcpy(return_val, "_"); 
+	case '_' :	strcpy(return_val, "_");
 			break;
     }
 }
@@ -587,15 +610,15 @@ void get_who_escape_code_value(char *return_val, const char letter, player *pl)
 
 int command_afk (object *op, char *params)
 {
-    if QUERY_FLAG(op,FLAG_AFK) { 
+    if QUERY_FLAG(op,FLAG_AFK) {
 	CLEAR_FLAG(op,FLAG_AFK);
-	draw_ext_info(NDI_UNIQUE, 0, op, MSG_TYPE_COMMAND, MSG_SUBTYPE_NONE, 
+	draw_ext_info(NDI_UNIQUE, 0, op, MSG_TYPE_COMMAND, MSG_SUBTYPE_NONE,
 		      "You are no longer AFK", NULL);
-    }    
-    else 
+    }
+    else
     {
 	SET_FLAG(op,FLAG_AFK);
-	draw_ext_info(NDI_UNIQUE, 0, op, MSG_TYPE_COMMAND, MSG_SUBTYPE_NONE, 
+	draw_ext_info(NDI_UNIQUE, 0, op, MSG_TYPE_COMMAND, MSG_SUBTYPE_NONE,
 		      "You are now AFK", NULL);
     }
     return 1;
@@ -657,14 +680,20 @@ int command_weather (object *op, char *params)
     int wx, wy, temp, sky;
     char buf[MAX_BUF];
 
-    if (settings.dynamiclevel < 1)
+    if (settings.dynamiclevel < 1) {
+	draw_ext_info(NDI_UNIQUE, 0, op, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_WEATHER,
+	    "The weather is perpetually great around here.", NULL);
 	return 1;
+    }
 
     if (op->map == NULL)
 	return 1;
 
-    if (worldmap_to_weathermap(op->x, op->y, &wx, &wy, op->map) != 0)
+    if (worldmap_to_weathermap(op->x, op->y, &wx, &wy, op->map) != 0) {
+	draw_ext_info(NDI_UNIQUE, 0, op, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_WEATHER,
+	    "You can't see the weather from here.", NULL);
 	return 1;
+    }
 
     if (QUERY_FLAG(op, FLAG_WIZ)) {
 	/* dump the weather, Dm style! Yo! */
@@ -741,18 +770,18 @@ int command_weather (object *op, char *params)
     }
     if (weathermap[wx][wy].windspeed < 5)
 	draw_ext_info_format(NDI_UNIQUE, 0, op, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_WEATHER,
-	     "There is a mild breeze coming from the %s.", 
-	     "There is a mild breeze coming from the %s.", 
+	     "There is a mild breeze coming from the %s.",
+	     "There is a mild breeze coming from the %s.",
 	     buf);
     else if (weathermap[wx][wy].windspeed < 10)
 	draw_ext_info_format(NDI_UNIQUE, 0, op, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_WEATHER,
-	      "There is a strong breeze coming from the %s.", 
-	      "There is a strong breeze coming from the %s.", 
+	      "There is a strong breeze coming from the %s.",
+	      "There is a strong breeze coming from the %s.",
 	      buf);
     else if (weathermap[wx][wy].windspeed < 15)
 	draw_ext_info_format(NDI_UNIQUE, 0, op, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_WEATHER,
-	     "There is a light wind coming from the %s.", 
-	     "There is a light wind coming from the %s.", 
+	     "There is a light wind coming from the %s.",
+	     "There is a light wind coming from the %s.",
 	     buf);
     else if (weathermap[wx][wy].windspeed < 25)
 	draw_ext_info_format(NDI_UNIQUE, 0, op, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_WEATHER,
@@ -761,13 +790,13 @@ int command_weather (object *op, char *params)
 	     buf);
     else if (weathermap[wx][wy].windspeed < 35)
 	draw_ext_info_format(NDI_UNIQUE, 0, op, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_WEATHER,
-	     "There is a heavy wind coming from the %s.", 
-	     "There is a heavy wind coming from the %s.", 
+	     "There is a heavy wind coming from the %s.",
+	     "There is a heavy wind coming from the %s.",
 	     buf);
     else
 	draw_ext_info_format(NDI_UNIQUE, 0, op, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_WEATHER,
-	     "The wind from the %s is incredibly strong!", 
-	     "The wind from the %s is incredibly strong!", 
+	     "The wind from the %s is incredibly strong!",
+	     "The wind from the %s is incredibly strong!",
 	     buf);
 
     sky = weathermap[wx][wy].sky;
@@ -861,8 +890,8 @@ int command_debug (object *op, char *params)
     }
     settings.debug = (enum LogLevel) FABS(i);
     draw_ext_info_format(NDI_UNIQUE, 0,op, MSG_TYPE_COMMAND, MSG_SUBTYPE_NONE,
-		 "Debug level set to %d.", 
-		 "Debug level set to %d.", 
+		 "Debug level set to %d.",
+		 "Debug level set to %d.",
 		 i);
     return 1;
 }
@@ -897,11 +926,11 @@ int command_wizpass (object *op, char *params)
 	i =onoff_value(params);
 
     if (i) {
-	draw_ext_info(NDI_UNIQUE, 0,op, MSG_TYPE_COMMAND, MSG_SUBTYPE_NONE, 
+	draw_ext_info(NDI_UNIQUE, 0,op, MSG_TYPE_COMMAND, MSG_SUBTYPE_NONE,
 		      "You will now walk through walls.\n", NULL);
 	SET_FLAG(op, FLAG_WIZPASS);
     } else {
-	draw_ext_info(NDI_UNIQUE, 0,op, MSG_TYPE_COMMAND, MSG_SUBTYPE_NONE, 
+	draw_ext_info(NDI_UNIQUE, 0,op, MSG_TYPE_COMMAND, MSG_SUBTYPE_NONE,
 		      "You will now be stopped by walls.\n", NULL);
 	CLEAR_FLAG(op, FLAG_WIZPASS);
     }
@@ -921,11 +950,11 @@ int command_wizcast (object *op, char *params)
 	i = onoff_value(params);
 
     if (i) {
-	draw_ext_info(NDI_UNIQUE, 0,op, MSG_TYPE_COMMAND, MSG_SUBTYPE_NONE, 
+	draw_ext_info(NDI_UNIQUE, 0,op, MSG_TYPE_COMMAND, MSG_SUBTYPE_NONE,
 		      "You can now cast spells anywhere.", NULL);
 	SET_FLAG(op, FLAG_WIZCAST);
     } else {
-	draw_ext_info(NDI_UNIQUE, 0,op, MSG_TYPE_COMMAND, MSG_SUBTYPE_NONE, 
+	draw_ext_info(NDI_UNIQUE, 0,op, MSG_TYPE_COMMAND, MSG_SUBTYPE_NONE,
 		      "You now cannot cast spells in no-magic areas.", NULL);
 	CLEAR_FLAG(op, FLAG_WIZCAST);
     }
@@ -989,18 +1018,18 @@ int command_output_sync(object *op, char *params)
     int val;
 
     if (!params) {
-	draw_ext_info_format(NDI_UNIQUE, 0, op, MSG_TYPE_COMMAND, MSG_SUBTYPE_NONE, 
-	     "Output sync time is presently %d", 
-	     "Output sync time is presently %d", 
+	draw_ext_info_format(NDI_UNIQUE, 0, op, MSG_TYPE_COMMAND, MSG_SUBTYPE_NONE,
+	     "Output sync time is presently %d",
+	     "Output sync time is presently %d",
 	     op->contr->outputs_sync);
 	return 1;
     }
     val=atoi(params);
     if (val>0) {
 	op->contr->outputs_sync = val;
-	draw_ext_info_format(NDI_UNIQUE, 0, op, MSG_TYPE_COMMAND, MSG_SUBTYPE_NONE, 
-	     "Output sync time now set to %d", 
-	     "Output sync time now set to %d", 
+	draw_ext_info_format(NDI_UNIQUE, 0, op, MSG_TYPE_COMMAND, MSG_SUBTYPE_NONE,
+	     "Output sync time now set to %d",
+	     "Output sync time now set to %d",
 	     op->contr->outputs_sync);
     }
     else
@@ -1015,7 +1044,7 @@ int command_output_count(object *op, char *params)
     int val;
 
     if (!params) {
-	draw_ext_info_format(NDI_UNIQUE, 0, op, MSG_TYPE_COMMAND, MSG_SUBTYPE_NONE, 
+	draw_ext_info_format(NDI_UNIQUE, 0, op, MSG_TYPE_COMMAND, MSG_SUBTYPE_NONE,
 	     "Output count is presently %d",
 	     "Output count is presently %d",
 	     op->contr->outputs_count);
@@ -1024,9 +1053,9 @@ int command_output_count(object *op, char *params)
     val=atoi(params);
     if (val>0) {
 	op->contr->outputs_count = val;
-	draw_ext_info_format(NDI_UNIQUE, 0, op, MSG_TYPE_COMMAND, MSG_SUBTYPE_NONE, 
-	     "Output count now set to %d", 
-	     "Output count now set to %d", 
+	draw_ext_info_format(NDI_UNIQUE, 0, op, MSG_TYPE_COMMAND, MSG_SUBTYPE_NONE,
+	     "Output count now set to %d",
+	     "Output count now set to %d",
 	     op->contr->outputs_count);
     }
     else
@@ -1041,19 +1070,19 @@ int command_listen (object *op, char *params)
     int i;
 
     if(params==NULL || !sscanf(params, "%d", &i)) {
-	draw_ext_info_format(NDI_UNIQUE, 0, op, MSG_TYPE_COMMAND, MSG_SUBTYPE_NONE, 
-	     "Set listen to what (presently %d)?", 
-	     "Set listen to what (presently %d)?", 
+	draw_ext_info_format(NDI_UNIQUE, 0, op, MSG_TYPE_COMMAND, MSG_SUBTYPE_NONE,
+	     "Set listen to what (presently %d)?",
+	     "Set listen to what (presently %d)?",
 	     op->contr->listening);
 	return 1;
     }
     if (i < 0) {
-        draw_ext_info_format(NDI_UNIQUE, 0, op, MSG_TYPE_COMMAND, MSG_SUBTYPE_NONE, 
+        draw_ext_info_format(NDI_UNIQUE, 0, op, MSG_TYPE_COMMAND, MSG_SUBTYPE_NONE,
             "Verbose level should be positive.", NULL);
         return;
     }
     op->contr->listening=(char) i;
-    draw_ext_info_format(NDI_UNIQUE, 0, op, MSG_TYPE_COMMAND, MSG_SUBTYPE_NONE, 
+    draw_ext_info_format(NDI_UNIQUE, 0, op, MSG_TYPE_COMMAND, MSG_SUBTYPE_NONE,
 	 "Your verbose level is now %d.",
 	 "Your verbose level is now %d.",
 	  i);
@@ -1132,7 +1161,7 @@ int command_players(object *op, char *paramss)
     char buf[MAX_BUF];
     char *t;
     DIR *Dir;
-  
+
     sprintf(buf,"%s/%s/",settings.localdir,settings.playerdir);
     t=buf+strlen(buf);
     if ((Dir=opendir(buf))!=NULL) {
@@ -1162,7 +1191,7 @@ int command_players(object *op, char *paramss)
 				  tm->tm_mday,
 				  tm->tm_hour,
 				  tm->tm_min,
-				  tm->tm_sec);	     
+				  tm->tm_sec);
 		    }
 		}
 	    }
@@ -1213,11 +1242,11 @@ int command_applymode(object *op, char *params)
 	return 1;
     }
 
-    if (!strcmp(params,"nochoice")) 
+    if (!strcmp(params,"nochoice"))
 	op->contr->unapply=unapply_nochoice;
-    else if (!strcmp(params,"never")) 
+    else if (!strcmp(params,"never"))
 	op->contr->unapply=unapply_never;
-    else if (!strcmp(params,"always")) 
+    else if (!strcmp(params,"always"))
 	op->contr->unapply=unapply_always;
     else {
 	draw_ext_info_format(NDI_UNIQUE, 0, op, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_CONFIG,
@@ -1294,18 +1323,18 @@ int command_petmode(object *op, char *params)
 	return 1;
     }
 
-    if (!strcmp(params,"normal")) 
+    if (!strcmp(params,"normal"))
 	op->contr->petmode=pet_normal;
-    else if (!strcmp(params,"sad")) 
+    else if (!strcmp(params,"sad"))
 	op->contr->petmode=pet_sad;
-    else if (!strcmp(params,"defend")) 
+    else if (!strcmp(params,"defend"))
 	op->contr->petmode=pet_defend;
-    else if (!strcmp(params,"arena")) 
+    else if (!strcmp(params,"arena"))
 	op->contr->petmode=pet_arena;
     else {
 	draw_ext_info_format(NDI_UNIQUE, 0, op, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_CONFIG,
-	    "petmode: Unknown options %s, valid options are normal, sad (seek and destroy), defend, arena", 
-	    "petmode: Unknown options %s, valid options are normal, sad (seek and destroy), defend, arena", 
+	    "petmode: Unknown options %s, valid options are normal, sad (seek and destroy), defend, arena",
+	    "petmode: Unknown options %s, valid options are normal, sad (seek and destroy), defend, arena",
 	     params);
 	return 0;
     }
@@ -1333,10 +1362,10 @@ int command_showpets(object *op, char *params)
 		    draw_ext_info(NDI_UNIQUE, 0, op, MSG_TYPE_COMMAND, MSG_SUBTYPE_NONE,
 				  "Pets:", NULL);
 		draw_ext_info_format(NDI_UNIQUE, 0, op,  MSG_TYPE_COMMAND, MSG_SUBTYPE_NONE,
-			     "%d  %s - level %d", 
-			     "%d  %s - level %d", 
+			     "%d  %s - level %d",
+			     "%d  %s - level %d",
 			     ++counter, ob->name, ob->level );
-	    }	
+	    }
 	    else if (!have_shown_pet && ++counter==target) {
 	    	draw_ext_info_format(NDI_UNIQUE, 0, op, MSG_TYPE_COMMAND, MSG_SUBTYPE_NONE,
 				     "[fixed]level %d %s",
@@ -1353,7 +1382,7 @@ int command_showpets(object *op, char *params)
 				     "Str %d",
 				     ob->stats.Str);
 		draw_ext_info_format(NDI_UNIQUE, 0, op, MSG_TYPE_COMMAND, MSG_SUBTYPE_NONE,
-				     "[fixed]Dex %d", 
+				     "[fixed]Dex %d",
 				     "Dex %d",
 				     ob->stats.Dex);
 		draw_ext_info_format(NDI_UNIQUE, 0, op, MSG_TYPE_COMMAND, MSG_SUBTYPE_NONE,
@@ -1377,14 +1406,14 @@ int command_showpets(object *op, char *params)
 				     "Pow %d",
 				     ob->stats.Pow);
 		draw_ext_info_format(NDI_UNIQUE, 0, op, MSG_TYPE_COMMAND, MSG_SUBTYPE_NONE,
-				     "[fixed]wc %d  damage %d ac %d ", 
-				     "wc %d  damage %d ac %d ", 
+				     "[fixed]wc %d  damage %d ac %d ",
+				     "wc %d  damage %d ac %d ",
 				     ob->stats.wc, ob->stats.dam, ob->stats.ac);
 		have_shown_pet=1;
 	    }
 	}
     }
-    if (counter == 0) 
+    if (counter == 0)
     	draw_ext_info(NDI_UNIQUE, 0, op, MSG_TYPE_COMMAND, MSG_SUBTYPE_NONE,
 		      "you have no pets.", NULL);
     else if (target !=0 && have_shown_pet==0)
@@ -1406,11 +1435,11 @@ int command_usekeys(object *op, char *params)
 	return 1;
     }
 
-    if (!strcmp(params,"inventory")) 
+    if (!strcmp(params,"inventory"))
 	op->contr->usekeys=key_inventory;
-    else if (!strcmp(params,"keyrings")) 
+    else if (!strcmp(params,"keyrings"))
 	op->contr->usekeys=keyrings;
-    else if (!strcmp(params,"containers")) 
+    else if (!strcmp(params,"containers"))
 	op->contr->usekeys=containers;
     else {
 	draw_ext_info_format(NDI_UNIQUE, 0, op, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_CONFIG,
@@ -1438,8 +1467,8 @@ int command_resistances(object *op, char *params)
 	if (i==ATNR_INTERNAL) continue;
 
 	draw_ext_info_format(NDI_UNIQUE, 0, op, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_INFO,
-			     "[fixed]%-20s %+5d", 
-			     "%-20s %+5d", 
+			     "[fixed]%-20s %+5d",
+			     "%-20s %+5d",
 			     attacktype_desc[i], op->resist[i]);
     }
 
@@ -1456,8 +1485,8 @@ int command_resistances(object *op, char *params)
                 for ( attack = 0; attack < NROFATTACKS; attack++ ) {
                     if ( atnr_is_dragon_enabled( attack ) ) {
                         draw_ext_info_format( NDI_UNIQUE, 0, op, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_INFO,
-					     "%s: %d", 
-					     "%s: %d", 
+					     "%s: %d",
+					     "%s: %d",
 					     change_resist_msg[ attack ], tmp->resist[ attack ] );
 		    }
 		}
@@ -1479,7 +1508,7 @@ static void help_topics(object *op, int what)
     struct dirent *de;
     char filename[MAX_BUF], line[HUGE_BUF];
     int namelen;
-  
+
     switch (what) {
 	case 1:
 	    sprintf(filename, "%s/wizhelp", settings.datadir);
@@ -1523,7 +1552,7 @@ static void show_commands(object *op, int what)
     command_array_struct *ap;
     extern command_array_struct Commands[], WizCommands[];
     extern const int CommandsSize, WizCommandsSize;
-  
+
     switch (what) {
 	case 1:
 	    ap =WizCommands;
@@ -1596,7 +1625,7 @@ int command_help (object *op, char *params)
 	    help_topics(op, 1);
 	return 0;
     }
-  
+
     /*
      * Commands list
      */
@@ -1616,8 +1645,8 @@ int command_help (object *op, char *params)
      */
     if (strchr(params, '.') || strchr(params, ' ') || strchr(params, '/')) {
 	draw_ext_info_format(NDI_UNIQUE, 0,op, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_INFO,
-			     "Illegal characters in '%s'", 
-			     "Illegal characters in '%s'", 
+			     "Illegal characters in '%s'",
+			     "Illegal characters in '%s'",
 			     params);
 	return 0;
     }
@@ -1667,8 +1696,8 @@ int command_help (object *op, char *params)
     nohelp:
 
     draw_ext_info_format(NDI_UNIQUE, 0,op, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_INFO,
-	 "No help available on '%s'", 
-	 "No help available on '%s'", 
+	 "No help available on '%s'",
+	 "No help available on '%s'",
 	 params);
 
     return 0;
@@ -1712,40 +1741,45 @@ int command_quit (object *op, char *params)
 }
 
 /**
- * don't allow people to exit explore mode.  It otherwise becomes
+ * Player wants to enter explore mode, that is never-dying mode.
+ *
+ * Don't allow people to exit explore mode.  It otherwise becomes
  * really easy to abuse this.
  */
 int command_explore (object *op, char *params)
 {
-    if (settings.explore_mode == FALSE)
-	return 1;
+    if (settings.explore_mode == FALSE) {
+        draw_ext_info(NDI_UNIQUE, 0,op, MSG_TYPE_COMMAND, MSG_SUBTYPE_NONE,
+            "Explore mode is disabled on this server, sorry.", NULL);
+        return 1;
+    }
     /*
      * I guess this is the best way to see if we are solo or not.  Actually,
      * are there any cases when first_player->next==NULL and we are not solo?
      */
     if ((first_player!=op->contr) || (first_player->next!=NULL)) {
-	draw_ext_info(NDI_UNIQUE, 0,op, MSG_TYPE_COMMAND, MSG_SUBTYPE_NONE,
-		      "You can not enter explore mode if there are other players", NULL);
+        draw_ext_info(NDI_UNIQUE, 0,op, MSG_TYPE_COMMAND, MSG_SUBTYPE_NONE,
+            "You can not enter explore mode if there are other players", NULL);
     } else if (op->contr->explore)
-	draw_ext_info(NDI_UNIQUE, 0,op, MSG_TYPE_COMMAND, MSG_SUBTYPE_NONE,
-		      "There is no return from explore mode", NULL);
+        draw_ext_info(NDI_UNIQUE, 0,op, MSG_TYPE_COMMAND, MSG_SUBTYPE_NONE,
+            "There is no return from explore mode", NULL);
     else {
-	op->contr->explore=1;
-	draw_ext_info(NDI_UNIQUE, 0,op, MSG_TYPE_COMMAND, MSG_SUBTYPE_NONE,
-		      "You are now in explore mode", NULL);
+        op->contr->explore=1;
+        draw_ext_info(NDI_UNIQUE, 0,op, MSG_TYPE_COMMAND, MSG_SUBTYPE_NONE,
+            "You are now in explore mode", NULL);
     }
     return 1;
 }
 
 int command_sound (object *op, char *params)
 {
-    if (op->contr->socket.sound) {
-        op->contr->socket.sound=0;
+    if (!(op->contr->socket.sound & SND_MUTE)) {
+        op->contr->socket.sound=op->contr->socket.sound | SND_MUTE;
         draw_ext_info(NDI_UNIQUE, 0,op, MSG_TYPE_COMMAND, MSG_SUBTYPE_NONE,
 		      "Sounds are turned off", NULL);
     }
     else {
-        op->contr->socket.sound=1;
+        op->contr->socket.sound=op->contr->socket.sound & ~SND_MUTE;
         draw_ext_info(NDI_UNIQUE, 0,op, MSG_TYPE_COMMAND, MSG_SUBTYPE_NONE,
 		      "The sounds are enabled.", NULL);
     }
@@ -1803,6 +1837,7 @@ void receive_player_password(object *op,char k) {
             get_name(op);
             return;
         }
+        LOG(llevInfo,"LOGIN: New player named %s from ip %s\n", op->name, op->contr->socket.host);
         display_motd(op);
         draw_ext_info(NDI_UNIQUE, 0,op, MSG_TYPE_COMMAND, MSG_SUBTYPE_NONE,
 		      "\nWelcome, Brave New Warrior!\n", NULL);
@@ -1853,7 +1888,7 @@ void receive_player_password(object *op,char k) {
 int command_title (object *op, char *params)
 {
     char buf[MAX_BUF];
-    
+
     if (settings.set_title == FALSE) {
 	draw_ext_info(NDI_UNIQUE, 0, op, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_CONFIG,
 		      "You cannot change your title.", NULL);
@@ -1866,7 +1901,7 @@ int command_title (object *op, char *params)
 		      "Dragons cannot change titles.", NULL);
         return 1;
     }
-    
+
     if(params == NULL) {
 	if(op->contr->own_title[0]=='\0')
 	    sprintf(buf,"Your title is '%s'.", op->contr->title);
@@ -1940,8 +1975,8 @@ int command_wimpy (object *op, char *params)
 	return 1;
     }
     draw_ext_info_format(NDI_UNIQUE, 0,op, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_CONFIG,
-		 "Your new wimpy level is %d.", 
-		 "Your new wimpy level is %d.", 
+		 "Your new wimpy level is %d.",
+		 "Your new wimpy level is %d.",
 		 i);
     op->run_away = i;
     return 1;
@@ -1971,7 +2006,7 @@ int command_kill_pets(object *op, char *params)
 {
     objectlink *obl, *next;
     int counter=0, removecount=0;
-    if (params == NULL) { 
+    if (params == NULL) {
     	terminate_all_pets(op);
 	draw_ext_info(NDI_UNIQUE, 0, op, MSG_TYPE_COMMAND, MSG_SUBTYPE_NONE,
 		      "Your pets have been killed.", NULL);
@@ -1990,10 +2025,10 @@ int command_kill_pets(object *op, char *params)
 		removecount++;
             }
 	}
-	if (removecount!=0) 
+	if (removecount!=0)
 	    draw_ext_info_format(NDI_UNIQUE, 0, op, MSG_TYPE_COMMAND, MSG_SUBTYPE_NONE,
-			 "killed %d pets.", 
-			 "killed %d pets.", 
+			 "killed %d pets.",
+			 "killed %d pets.",
 			 removecount);
 	else
 	    draw_ext_info(NDI_UNIQUE, 0, op, MSG_TYPE_COMMAND, MSG_SUBTYPE_NONE,
@@ -2075,3 +2110,135 @@ int command_passwd(object *pl, char *params)
     pl->contr->state = ST_CHANGE_PASSWORD_OLD;
     return 1;
 }
+
+/**
+ * Player is trying to harvest something.
+ * @param pl
+ * player trying to harvest.
+ * @param dir
+ * direction.
+ * @param skill
+ * skill being used.
+ * @return
+ * 0
+ */
+int do_harvest(object* pl, int dir, object* skill) {
+    sint16 x, y;
+    int count = 0, proba; /* Probability to get the item, 100 based. */
+    int level, exp;
+    object* found[10]; /* Found items that can be harvested. */
+    mapstruct* map;
+    object* item, *inv;
+    const char *trace, *ttool, *tspeed, *race, *tool, *slevel, *sexp;
+    float speed;
+
+    x = pl->x + freearr_x[dir];
+    y = pl->y + freearr_y[dir];
+    map = pl->map;
+
+    if (!pl->type == PLAYER)
+        return 0;
+
+    if (!map)
+        return 0;
+
+    if (get_map_flags(map, &map, x, y, &x, &y) & P_OUT_OF_MAP) {
+        draw_ext_info_format(NDI_WHITE, 0, pl, MSG_TYPE_SKILL, MSG_TYPE_SKILL_FAILURE, "You can't %s anything here.", NULL, skill->slaying);
+        return 0;
+    }
+
+    if (!pl->chosen_skill || pl->chosen_skill->skill != skill->skill)
+        return 0;
+
+    trace = get_ob_key_value(pl->chosen_skill, "harvest_race");
+    ttool = get_ob_key_value(pl->chosen_skill, "harvest_tool");
+    tspeed = get_ob_key_value(pl->chosen_skill, "harvest_speed");
+    if (!trace || strcmp(trace, "") == 0 || !ttool || strcmp(ttool, "") == 0 || !tspeed || strcmp(tspeed, "") == 0) {
+        draw_ext_info_format(NDI_WHITE, 0, pl, MSG_TYPE_SKILL, MSG_TYPE_SKILL_FAILURE, "You can't %s anything here.", NULL, skill->slaying);
+        LOG(llevError, "do_harvest: tool %s without harvest_[race|tool|speed]\n", pl->chosen_skill->name);
+        return 0;
+    }
+
+    item = GET_MAP_OB(map, x, y);
+    while (item && count < 10) {
+        for (inv = item->inv; inv; inv = inv->below) {
+            if (get_ob_key_value(inv, "harvestable") == NULL)
+                continue;
+            race = get_ob_key_value(inv, "harvest_race");
+            tool = get_ob_key_value(inv, "harvest_tool");
+            slevel = get_ob_key_value(inv, "harvest_level");
+            sexp = get_ob_key_value(inv, "harvest_exp");
+            if (race && (!slevel || !sexp)) {
+                LOG(llevError, "do_harvest: item %s without harvest_[level|exp]\n", inv->name);
+                continue;
+            }
+            if (race == trace && (!tool || tool == ttool))
+                found[count++] = inv;
+        }
+        item = item->above;
+    }
+    if (count == 0) {
+        draw_ext_info_format(NDI_WHITE, 0, pl, MSG_TYPE_SKILL, MSG_TYPE_SKILL_FAILURE, "You can't %s anything here.", NULL, skill->slaying);
+        return 0;
+    }
+
+    inv = found[rndm(0, count - 1)];
+    assert(inv);
+
+    slevel = get_ob_key_value(inv, "harvest_level");
+    sexp = get_ob_key_value(inv, "harvest_exp");
+    level = atoi(slevel);
+    exp = atoi(sexp);
+
+    speed = atof(tspeed);
+    if (speed < 0)
+        speed = - speed * pl->speed;
+    pl->speed_left -= speed;
+
+
+    /* Now we found something to harvest, randomly try to get it. */
+    if (level > skill->level + 10) {
+        draw_ext_info_format(NDI_WHITE, 0, pl, MSG_TYPE_SKILL, MSG_TYPE_SKILL_FAILURE, "You fail to %s anything.", NULL, skill->slaying);
+        return 0;
+    }
+
+    if (level >= skill->level)
+        /* Up to 10 more levels, 1 to 11 percent probability. */
+        proba = 10 + skill->level - level;
+    else if (skill->level <= level + 10)
+        proba = 10 + (skill->level - level) * 2;
+    else
+        proba = 30;
+
+    if (proba <= random_roll(0, 100, pl, 1)) {
+        draw_ext_info_format(NDI_WHITE, 0, pl, MSG_TYPE_SKILL, MSG_TYPE_SKILL_FAILURE, "You fail to %s anything.", NULL, skill->slaying);
+        return 0;
+    }
+
+    /* Ok, got it. */
+    item = get_object();
+    copy_object_with_inv(inv, item);
+    set_ob_key_value(item, "harvestable", NULL, 0);
+    if (QUERY_FLAG(item, FLAG_MONSTER)) {
+        int spot = find_free_spot(item, pl->map, pl->x, pl->y, 0, SIZEOFFREE);
+        if (spot == -1) {
+            /* Better luck next time...*/
+            remove_ob(item);
+            draw_ext_info_format(NDI_WHITE, 0, pl, MSG_TYPE_SKILL, MSG_TYPE_SKILL_FAILURE, "You fail to %s anything.", NULL, skill->slaying);
+        return 0;
+        }
+        item->x = pl->x + freearr_x[spot];
+        item->y = pl->y + freearr_y[spot];
+        insert_ob_in_map(item, pl->map, NULL, 0);
+        draw_ext_info_format(NDI_WHITE, 0, pl, MSG_TYPE_SKILL, MSG_TYPE_SKILL_FAILURE, "You %s a %s!", NULL, skill->slaying, item->name);
+    }
+    else {
+        item = insert_ob_in_ob(item, pl);
+        draw_ext_info_format(NDI_WHITE, 0, pl, MSG_TYPE_SKILL, MSG_TYPE_SKILL_FAILURE, "You %s some %s", NULL, skill->slaying, item->name);
+    }
+
+    /* Get exp */
+    change_exp(pl, exp, skill->name, SK_EXP_ADD_SKILL);
+
+    return 0;
+}
