@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_c_object_c =
- *   "$Id: c_object.c 5669 2007-03-04 10:47:14Z ryo_saeba $";
+ *   "$Id: c_object.c 7634 2007-12-01 11:42:44Z ryo_saeba $";
  */
 /*
     CrossFire, A Multiplayer game for X-windows
@@ -548,6 +548,7 @@ static void pick_up_object (object *pl, object *op, object *tmp, int nrof)
     }
     if (QUERY_FLAG (tmp, FLAG_NO_DROP))
 	return;
+
     if(QUERY_FLAG(tmp,FLAG_WAS_WIZ) && !QUERY_FLAG(pl, FLAG_WAS_WIZ)) {
 	new_draw_info(NDI_UNIQUE, 0,pl, "The object disappears in a puff of smoke!");
 	new_draw_info(NDI_UNIQUE, 0,pl, "It must have been an illusion.");
@@ -560,19 +561,24 @@ static void pick_up_object (object *pl, object *op, object *tmp, int nrof)
     
     if (nrof > tmp_nrof || nrof == 0)
 	nrof = tmp_nrof;
+
     /* Figure out how much weight this object will add to the player */
     weight = tmp->weight * nrof;
     if (tmp->inv) weight += tmp->carrying * (100 - tmp->stats.Str) / 100;
+
     if (pl->stats.Str <= MAX_STAT)
         effective_weight_limit = weight_limit[pl->stats.Str];
     else
         effective_weight_limit = weight_limit[MAX_STAT];
+
     if ((pl->weight + pl->carrying + weight) > effective_weight_limit) {
 	new_draw_info(0, 0,pl,"That item is too heavy for you to pick up.");
 	return;
     }
+
     if (settings.real_wiz == FALSE && QUERY_FLAG(pl, FLAG_WAS_WIZ))
 	SET_FLAG(tmp, FLAG_WAS_WIZ);
+
     if (nrof != tmp_nrof) {
 	object *tmp2 = tmp;
         tag_t tmp2_tag = tmp2->count;
@@ -581,24 +587,16 @@ static void pick_up_object (object *pl, object *op, object *tmp, int nrof)
 	    new_draw_info(NDI_UNIQUE, 0,pl, errmsg);
 	    return;
 	}
-	/* Tell a client what happened rest of objects */
-	if (pl->type == PLAYER) {
-	    if (was_destroyed (tmp2, tmp2_tag))
-		esrv_del_item (pl->contr, tmp2_tag);
-	    else
-		esrv_send_item (pl, tmp2);
-	}
     } else {
 	/* If the object is in a container, send a delete to the client.
 	 * - we are moving all the items from the container to elsewhere,
 	 * so it needs to be deleted.
 	 */
         if ( ! QUERY_FLAG (tmp, FLAG_REMOVED)) {
-	    if (tmp->env && pl->type==PLAYER) 
-	        esrv_del_item (pl->contr, tmp->count);
 	    remove_ob(tmp); /* Unlink it */
 	}
     }
+
     if(QUERY_FLAG(tmp, FLAG_UNPAID))
 	(void) sprintf(buf,"%s will cost you %s.", query_name(tmp),
 		query_cost_string(tmp,pl,F_BUY | F_SHOP));
@@ -619,13 +617,15 @@ static void pick_up_object (object *pl, object *op, object *tmp, int nrof)
      */
     if(pl->type!=PLAYER) return;
 
-    esrv_send_item (pl, tmp);
+    /* Additional weight changes speed, etc */
+    fix_player(pl);
+
     /* These are needed to update the weight for the container we
      * are putting the object in.
      */
     if (op!=pl) {
 	esrv_update_item (UPD_WEIGHT, pl, op);
-	esrv_send_item (pl, pl);
+	esrv_update_item (UPD_WEIGHT, pl, pl);
     }
 
     /* Update the container the object was in */
@@ -837,10 +837,13 @@ void put_object_in_sack (object *op, object *sack, object *tmp, uint32 nrof)
 	return;
     }
     if (tmp->type == CONTAINER && tmp->inv) {
-
+        if (tmp->slaying)
+            return;
 	/* Eneq(@csd.uu.se): If the object to be dropped is a container
+	 * and does not require a key to be opened,
 	 * we instead move the contents of that container into the active
 	 * container, this is only done if the object has something in it.
+	 * If object is container but need a key, just don't do anything
 	 */
 	sack2 = tmp;
 	new_draw_info_format(NDI_UNIQUE, 0,op, "You move the items from %s into %s.",
@@ -881,11 +884,6 @@ void put_object_in_sack (object *op, object *sack, object *tmp, uint32 nrof)
 	    new_draw_info(NDI_UNIQUE, 0,op, errmsg);
 	    return;
 	}
-	/* Tell a client what happened other objects */ 
-	if (was_destroyed (tmp2, tmp2_tag))
-	      esrv_del_item (op->contr, tmp2_tag);
-	else	/* this can proably be replaced with an update */
-	      esrv_send_item (op, tmp2);
     } else
 	remove_ob(tmp);
 
@@ -893,17 +891,11 @@ void put_object_in_sack (object *op, object *sack, object *tmp, uint32 nrof)
 		  query_name(tmp), query_name(sack));
     tmp_tag = tmp->count;
     tmp2 = insert_ob_in_ob(tmp, sack);
-    fix_player(op); /* This is overkill, fix_player() is called somewhere */
+
+    if (!QUERY_FLAG(op, FLAG_NO_FIX_PLAYER))
+	fix_player(op); /* This is overkill, fix_player() is called somewhere */
 		  /* in object.c */
 
-    /* If an object merged (and thus, different object), we need to
-     * delete the original.
-     */
-    if (tmp2 != tmp)
-	esrv_del_item (op->contr, tmp_tag);
-
-    esrv_send_item (op, tmp2);
-
     /* If a transport, need to update all the players in the transport
      * the view of what is in it.
      */
@@ -952,16 +944,6 @@ object *drop_object (object *op, object *tmp, uint32 nrof)
 	    new_draw_info(NDI_UNIQUE, 0,op, errmsg);
 	    return NULL;
 	}
-	/* Tell a client what happened rest of objects.  tmp2 is now the
-	 * original object
-	 */
-	 if (op->type == PLAYER)
-	 {
-                if (was_destroyed (tmp2, tmp2_tag))
-                        esrv_del_item (op->contr, tmp2_tag);
-                else
-                        esrv_send_item (op, tmp2);
-        }
     } else
       remove_ob (tmp);
 
@@ -970,14 +952,14 @@ object *drop_object (object *op, object *tmp, uint32 nrof)
         return NULL;
 
     if (QUERY_FLAG (tmp, FLAG_STARTEQUIP)) {
-      sprintf(buf,"You drop the %s.", query_name(tmp));
-      new_draw_info(NDI_UNIQUE, 0,op,buf);
-      new_draw_info(NDI_UNIQUE, 0,op,"The gods who lent it to you retrieves it.");
-      if (op->type==PLAYER)
-	esrv_del_item (op->contr, tmp->count);
-      free_object(tmp);
-      fix_player(op);
-      return NULL;
+	sprintf(buf,"You drop the %s.", query_name(tmp));
+	new_draw_info(NDI_UNIQUE, 0,op,buf);
+	new_draw_info(NDI_UNIQUE, 0,op,"The gods who lent it to you retrieves it.");
+
+	free_object(tmp);
+	if (!QUERY_FLAG(op, FLAG_NO_FIX_PLAYER))
+	    fix_player(op);
+	return NULL;
     }
 
 /*  If SAVE_INTERVAL is commented out, we never want to save
@@ -1000,15 +982,12 @@ object *drop_object (object *op, object *tmp, uint32 nrof)
     tmp->x = op->x;
     tmp->y = op->y;
 
-    if (op->type == PLAYER)
-        esrv_del_item (op->contr, tmp->count);
     tmp_tag = tmp->count;
     insert_ob_in_map(tmp, op->map, op,0);
     if (!was_destroyed(tmp, tmp_tag) && !QUERY_FLAG(tmp, FLAG_UNPAID) && tmp->type != MONEY && is_in_shop(op)) {
         sell_item(tmp, op);
     }
 
-
     SET_FLAG (op, FLAG_NO_APPLY);
     remove_ob(op);
     insert_ob_in_map(op, op->map, op, INS_NO_MERGE | INS_NO_WALK_ON);
@@ -1017,14 +996,13 @@ object *drop_object (object *op, object *tmp, uint32 nrof)
     /* Call this before we update the various windows/players.  At least
      * that we, we know the weight is correct.
      */
+    if (!QUERY_FLAG(op, FLAG_NO_FIX_PLAYER)) {
     fix_player(op); /* This is overkill, fix_player() is called somewhere */
 		    /* in object.c */
 
-    if (op->type == PLAYER)
-    {
-	op->contr->socket.update_look = 1;
-	/* Need to update the weight for the player */
-	esrv_send_item (op, op);
+	/* Need to update weight of player */
+	if (op->type == PLAYER)
+	    esrv_update_item(UPD_WEIGHT, op, op);
     }
     return tmp;
 }
@@ -1108,6 +1086,7 @@ void drop(object *op, object *tmp)
 int command_dropall (object *op, char *params) {
 
   object * curinv, *nextinv;
+    int count=0;
 
   if(op->inv == NULL) {
     new_draw_info(NDI_UNIQUE, 0,op,"Nothing to drop!");
@@ -1126,88 +1105,115 @@ int command_dropall (object *op, char *params) {
     in a shop. --Tero.Pelander@utu.fi
   */
 
-  if(params==NULL) {
-    while(curinv != NULL) {
-      nextinv = curinv->below;
-      while (nextinv && nextinv->type==MONEY)
-	nextinv = nextinv->below;
-      if(! QUERY_FLAG(curinv,FLAG_INV_LOCKED) && curinv->type != MONEY &&
-	 curinv->type != FOOD && curinv->type != KEY && 
-	 curinv->type != SPECIAL_KEY && curinv->type != GEM &&
-	 !curinv->invisible &&
-	 (curinv->type!=CONTAINER || op->container!=curinv))
-	{
-	 drop(op,curinv);
-       }
-      curinv = nextinv;
-    }
-  }
+    if (op->contr) count=op->contr->count;
 
-  else if(strcmp(params, "weapons") == 0) {
-    while(curinv != NULL) {
-      nextinv = curinv->below;
-      while (nextinv && nextinv->type==MONEY)
-	nextinv = nextinv->below;
-      if(! QUERY_FLAG(curinv,FLAG_INV_LOCKED) && ((curinv->type == WEAPON) ||
-	 (curinv->type == BOW) || (curinv->type == ARROW)))
-	{
-	  drop(op,curinv);
-	}
-      curinv = nextinv;
+    /* Set this so we don't call it for _every_ object that
+     * is dropped.
+     */
+    SET_FLAG(op, FLAG_NO_FIX_PLAYER);
+    /*
+     * This is the default.  Drops everything not locked or considered
+     * not something that should be dropped.
+     * Care must be taken that the next item pointer is not to money as
+     * the drop() routine will do unknown things to it when dropping
+     * in a shop. --Tero.Pelander@utu.fi
+     */
+    if(params==NULL) {
+        while(curinv != NULL) {
+            nextinv = curinv->below;
+            while (nextinv && nextinv->type==MONEY)
+                nextinv = nextinv->below;
+            if(! QUERY_FLAG(curinv,FLAG_INV_LOCKED) && curinv->type != MONEY
+                && curinv->type != FOOD && curinv->type != KEY
+                && curinv->type != SPECIAL_KEY && curinv->type != GEM
+                && !curinv->invisible
+                && (curinv->type!=CONTAINER || op->container!=curinv))
+            {
+                drop(op,curinv);
+		if (op->contr) op->contr->count=count;
+            }
+            curinv = nextinv;
+        }
     }
-  }
-  
-  else if(strcmp(params, "armor") == 0 || strcmp(params, "armour") == 0) {
-    while(curinv != NULL) {
-      nextinv = curinv->below;
-      while (nextinv && nextinv->type==MONEY)
-	nextinv = nextinv->below;
-      if(! QUERY_FLAG(curinv,FLAG_INV_LOCKED) && ((curinv->type == ARMOUR) ||
-	 curinv->type == SHIELD || curinv->type==HELMET))
-	{
-	  drop(op,curinv);
-	}
-      curinv = nextinv;
+    else if(strcmp(params, "weapons") == 0) {
+        while(curinv != NULL) {
+            nextinv = curinv->below;
+            while (nextinv && nextinv->type==MONEY)
+                nextinv = nextinv->below;
+            if(! QUERY_FLAG(curinv,FLAG_INV_LOCKED)
+                && ((curinv->type == WEAPON)
+                || (curinv->type == BOW) || (curinv->type == ARROW)))
+            {
+                drop(op,curinv);
+		if (op->contr) op->contr->count=count;
+            }
+            curinv = nextinv;
+        }
     }
-  }
+    else if(strcmp(params, "armor") == 0 || strcmp(params, "armour") == 0)
+    {
+        while(curinv != NULL)
+        {
+            nextinv = curinv->below;
+            while (nextinv && nextinv->type==MONEY)
+                nextinv = nextinv->below;
+            if(! QUERY_FLAG(curinv,FLAG_INV_LOCKED)
+                && ((curinv->type == ARMOUR)
+                || curinv->type == SHIELD || curinv->type==HELMET))
+            {
+                drop(op,curinv);
+		if (op->contr) op->contr->count=count;
+            }
+            curinv = nextinv;
+        }
+    }
+    else if(strcmp(params, "misc") == 0)
+    {
+        while(curinv != NULL) {
+            nextinv = curinv->below;
+            while (nextinv && nextinv->type==MONEY)
+                nextinv = nextinv->below;
+            if(! QUERY_FLAG(curinv,FLAG_INV_LOCKED)
+                && ! QUERY_FLAG(curinv,FLAG_APPLIED))
+            {
+                switch(curinv->type) {
+                    case HORN:
+                    case BOOK:
+                    case SPELLBOOK:
+                    case GIRDLE:
+                    case AMULET:
+                    case RING:
+                    case CLOAK:
+                    case BOOTS:
+                    case GLOVES:
+                    case BRACERS:
+                    case SCROLL:
+                    case ARMOUR_IMPROVER:
+                    case WEAPON_IMPROVER:
+                    case WAND:
+                    case ROD:
+                    case POTION:
+                        drop(op,curinv);
+                        curinv = nextinv;
+			if (op->contr) op->contr->count=count;
+                        break;
+                    default:
+                        curinv = nextinv;
+                        break;
+                }
+            }
+            curinv = nextinv;
+        }
+    }
+    op->contr->socket.update_look=1;
+    CLEAR_FLAG(op, FLAG_NO_FIX_PLAYER);
+    /* call it now, once */
+    fix_player(op);
+    /* Need to update weight of player.  Likewise, only do it once */
+    if (op->type == PLAYER)
+	esrv_update_item(UPD_WEIGHT, op, op);
 
-  else if(strcmp(params, "misc") == 0) {
-    while(curinv != NULL) {
-      nextinv = curinv->below;
-      while (nextinv && nextinv->type==MONEY)
-	nextinv = nextinv->below;
-      if(! QUERY_FLAG(curinv,FLAG_INV_LOCKED) && ! QUERY_FLAG(curinv,FLAG_APPLIED)) {
-	switch(curinv->type) {
-	case HORN:
-	case BOOK:
-	case SPELLBOOK:
-	case GIRDLE:
-	case AMULET:
-	case RING:
-	case CLOAK:
-	case BOOTS:
-	case GLOVES:
-	case BRACERS:
-	case SCROLL:
-	case ARMOUR_IMPROVER:
-	case WEAPON_IMPROVER:
-	case WAND:
-	case ROD:
-	case POTION:
-	  drop(op,curinv);
-	  curinv = nextinv;
-	  break;
-	default:
-	  curinv = nextinv;
-	  break;
-	}
-      }
-      curinv = nextinv;
-    }
-  }
-  op->contr->socket.update_look=1;
-/*  draw_look(op);*/
-  return 0;
+    return 0;
 }
 
 /* Object op wants to drop object(s) params.  params can be a
@@ -1253,6 +1259,95 @@ int command_drop (object *op, char *params)
     return 0;
 }
 
+/**
+ * Put all contents of the container on the ground below the player or in opened container, except locked items.
+ *
+ * @param container
+ * what to empty.
+ * @param pl
+ * player to drop for.
+ */
+static void empty_container(object* container, object* pl) {
+    object* inv;
+    object* next;
+    int left = 0;
+
+    if (!container->inv)
+        return;
+
+    for (inv = container->inv; inv; inv = next) {
+        next = inv->below;
+        if (QUERY_FLAG(inv, FLAG_INV_LOCKED)) {
+            /* you can have locked items in container. */
+            left++;
+            continue;
+        }
+        drop(pl, inv);
+        if (inv->below == next)
+            /* item couldn't be dropped for some reason. */
+            left++;
+    }
+    esrv_update_item(UPD_WEIGHT, pl, container);
+
+    if (left)
+        draw_ext_info_format(NDI_UNIQUE, 0, pl, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_SUCCESS, "You empty the %s except %d items.", NULL, query_name(container), left);
+    else
+        draw_ext_info_format(NDI_UNIQUE, 0, pl, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_SUCCESS, "You empty the %s.", NULL, query_name(container));
+}
+
+/**
+ * 'empty' command.
+ *
+ * @param op
+ * player.
+ * @param params
+ * item specifier.
+ * @return
+ * 0.
+ */
+int command_empty(object *op, char *params) {
+    object* inv;
+    object* container;
+
+    if (!params) {
+        draw_ext_info(NDI_UNIQUE,0,op,MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_ERROR,
+            "Empty what?", NULL);
+        return 0;
+    }
+
+    if (strcmp(params, "all") == 0) {
+        for (inv = op->inv; inv; inv = inv->below)
+            if (inv->type == CONTAINER)
+                empty_container(inv, op);
+        return 0;
+    }
+
+    container = find_best_object_match(op, params);
+    if (!container) {
+        draw_ext_info(NDI_UNIQUE,0,op,MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_ERROR,
+            "No such item.", NULL);
+        return 0;
+    }
+    if (container->type != CONTAINER) {
+        draw_ext_info(NDI_UNIQUE,0,op,MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_ERROR,
+            "This is not a container!", NULL);
+        return 0;
+    }
+    empty_container(container, op);
+
+    return 0;
+}
+
+/**
+ * 'examine' command.
+ *
+ * @param op
+ * player.
+ * @param params
+ * optional item specifier.
+ * @return
+ * 0.
+ */
 int command_examine (object *op, char *params)
 {
   if (!params) {
@@ -1511,7 +1606,7 @@ void examine(object *op, object *tmp) {
 
     if(tmp->weight) {
 	sprintf(buf,tmp->nrof>1?"They weigh %3.3f kg.":"It weighs %3.3f kg.",
-            tmp->weight*(tmp->nrof?tmp->nrof:1)/1000.0);
+            tmp->weight*((float)(tmp->nrof?tmp->nrof:1)/1000.0));
 	new_draw_info(NDI_UNIQUE, 0,op,buf);
     }
 
@@ -1943,12 +2038,8 @@ int command_rename_item(object *op, char *params)
   tag = item->count;
   tmp = merge_ob(item, NULL);
   if (tmp == NULL) {
-    /* object was not merged */
+    /* object was not merged - if it was, merge_ob handles updating for us. */
     esrv_update_item(UPD_NAME, op, item);
-  } else {
-    /* object was merged into tmp */
-    esrv_del_item(op->contr, tag);
-    esrv_send_item(op, tmp);
   }
 
   return 1;
@@ -1993,12 +2084,9 @@ int command_lock_item(object *op, char *params) {
     tag = item->count;
     tmp = merge_ob(item, NULL);
     if (tmp == NULL) {
-        /* object was not merged */
+        /* object was not merged, if it was merge_ob handles updates for us */
         esrv_update_item(UPD_FLAGS, op, item);
-    } else {
-        /* object was merged into tmp */
-        esrv_del_item(op->contr, tag);
-        esrv_send_item(op, tmp);
     }
     return 1;
 }
+
