@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_c_party_c =
- *   "$Id: c_party.c 5127 2006-11-19 20:15:44Z akirschbaum $";
+ *   "$Id: c_party.c 7377 2007-10-20 20:54:45Z ryo_saeba $";
  */
 
 /*
@@ -42,10 +42,18 @@ partylist* get_firstparty(void)
 
 void remove_party(partylist *target_party);
 
-/* Forms the party struct for a party called 'params'. it is the responsibility
- * of the caller to ensure that the name is unique, and that it is placed in the 
- * main party list correctly */
-static partylist *form_party(object *op, const char *params) {
+/**
+ * Forms the party struct for a party called 'params'. it is the responsibility
+ * of the caller to ensure that the name is unique.
+ * New item is placed on the party list.
+ * @param op
+ * party creator.
+ * @param params
+ * party name.
+ * @return
+ * new party.
+ */
+partylist *form_party(object *op, const char *params) {
 
     partylist * newparty;
 
@@ -59,6 +67,16 @@ static partylist *form_party(object *op, const char *params) {
     new_draw_info_format(NDI_UNIQUE, 0, op,
 	"You have formed party: %s",newparty->partyname);	
     op->contr->party=newparty;
+
+    if (lastparty) {
+        lastparty->next=newparty;
+        lastparty = lastparty->next;
+    }
+    else {
+        firstparty=newparty;
+        lastparty=firstparty;
+    }
+
     return newparty;
 }
 
@@ -78,6 +96,8 @@ void remove_party(partylist *target_party) {
     
     /* special case-ism for parties at the beginning and end of the list */
     if (target_party==firstparty) {
+        if (lastparty == target_party)
+	    lastparty = NULL;
 	firstparty=firstparty->next;
 	if (target_party->partyleader) free(target_party->partyleader);
 	if (target_party->partyname) free(target_party->partyname);
@@ -322,12 +342,10 @@ int command_party (object *op, char *params)
         	return 1;
 	    }
 	}
-    lastparty->next=form_party(op, params);
-    lastparty = lastparty->next;
+        form_party(op, params);
     }
     else {
-	firstparty=form_party(op, params);
-	lastparty=firstparty;
+        form_party(op, params);
     }
     /* 
      * The player might have previously been a member of a party, if so, he will be leaving
@@ -358,31 +376,15 @@ int command_party (object *op, char *params)
     op->contr->party=NULL;
     return 1;
   }
-  if(strcmp(params, "who")==0) {
-    player *pl;
-    tmpparty = op->contr->party;
-    if(op->contr->party==NULL) {
-      new_draw_info(NDI_UNIQUE, 0,op,"You are not a member of any party.");
-      return 1;
-    }
-    new_draw_info_format(NDI_UNIQUE, 0, op,
-	"Members of party: %s.", op->contr->party->partyname);
-    for(pl=first_player;pl!=NULL;pl=pl->next)
-      if(pl->ob->contr->party==op->contr->party) {
-	  if (settings.set_title == TRUE) {
-	      if(pl->ob->contr->own_title[0]!='\0')
-		  sprintf(buf,"%3d %s the %s",
-		      pl->ob->level,pl->ob->name,pl->ob->contr->own_title);
-	      else
-		  sprintf(buf,"%3d %s the %s",
-		      pl->ob->level,pl->ob->name,pl->ob->contr->title);
-	  } else
-	      sprintf(buf,"%3d %s the %s",
-		  pl->ob->level,pl->ob->name,pl->ob->contr->title);
-        new_draw_info(NDI_UNIQUE, 0,op,buf);
-      }
-    return 1;
-  } /* leave */
+    if(strcmp(params, "who")==0) {
+        if(op->contr->party==NULL) {
+            new_draw_info(NDI_UNIQUE, 0,op, 
+                "You are not a member of any party.");
+            return 1;
+        }
+        list_players(op, NULL, op->contr->party);
+        return 1;
+    } /* leave */
 
   if(strncmp(params, "passwd ", 7) == 0) {
     partylist *tmplist;
@@ -525,3 +527,41 @@ int command_party (object *op, char *params)
 #endif
   return 1;
 }
+
+static const char* rejoin_modes[] = {
+    "no",
+    "if_exists",
+    "always",
+    NULL
+};
+
+/**
+ * Handles the 'party_rejoin' command.
+ * @param op
+ * player.
+ * @param params
+ * optional parameters.
+ * @return
+ * 1.
+ */
+int command_party_rejoin(object *op, char *params) {
+    int mode;
+    if (!params) {
+        draw_ext_info_format(NDI_UNIQUE, 0, op, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_SUCCESS,
+            "party rejoin: %s", NULL, rejoin_modes[op->contr->rejoin_party]);
+        return 1;
+    }
+    for (mode = 0; rejoin_modes[mode] != NULL; mode++) {
+        if (strcmp(rejoin_modes[mode], params) == 0) {
+            op->contr->rejoin_party = mode;
+            draw_ext_info_format(NDI_UNIQUE, 0, op, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_SUCCESS,
+                "party rejoin is now: %s", NULL, rejoin_modes[op->contr->rejoin_party]);
+            return 1;
+        }
+    }
+    if (strlen(params) > 50)
+        params[50] = '\0';
+    draw_ext_info_format(NDI_UNIQUE, 0, op, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_ERROR,
+        "invalid mode: %s", NULL, params);
+    return 1;
+}
