@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_c_wiz_c =
- *   "$Id: c_wiz.c 5813 2007-03-17 23:38:55Z ryo_saeba $";
+ *   "$Id: c_wiz.c 8175 2008-01-12 09:27:41Z akirschbaum $";
  */
 
 /*
@@ -262,7 +262,7 @@ int command_kick(object *op, const char *params) {
             new_draw_info_format(NDI_UNIQUE|NDI_ALL|NDI_RED, 5, op,
                              "%s is kicked out of the game.", op->name);
             strcpy(op->contr->killer, "left");
-            check_score(op); /* Always check score */
+            check_score(op,0); /* Always check score */
 
             /*
              * not sure how the player would be freed, but did see
@@ -295,15 +295,10 @@ int command_overlay_save(object *op, char *params) {
         return 1;
     }
 
-    new_save_map(op->map, 2);
-    // No need to save again the map and reset it. OVerlay saving is non destructive.
-    // new_save_map(op->map, 0);
-    // This fixes bug #1553636 (Crashbug: reset/swaped map after use of "overlay_save")
-    // Ryo 2006-10-22
-    new_draw_info(NDI_UNIQUE, 0, op, "Current map has been saved as an"
-        " overlay.");
-
-    //ready_map_name(op->map->path, 0);
+    if (new_save_map(op->map, SAVE_MODE_OVERLAY) < 0)
+    new_draw_info(NDI_UNIQUE, 0, op, "Overlay save error!");
+    else
+    new_draw_info(NDI_UNIQUE, 0, op, "Current map has been saved as an overlay.");
 
     return 1;
 }
@@ -319,7 +314,7 @@ int command_overlay_reset(object *op, char* params) {
             new_draw_info(NDI_UNIQUE, 0, op, "Overlay couldn't be removed.");
     else
         new_draw_info(NDI_UNIQUE, 0, op, "No overlay for current map.");
-    
+
     return 1;
 }
 
@@ -370,7 +365,7 @@ int command_shutdown(object *op, char *params) {
      * the op won't get kicked off, so we do it ourselves
      */
     command_kick(op, NULL);
-    check_score(op); /* Always check score */
+    check_score(op,0); /* Always check score */
     (void)save_player(op, 0);
     play_again(op);
     cleanup();
@@ -424,8 +419,6 @@ int command_generate (object *op, char *params)
             ;
         if (tmp != NULL) {
             tmp = insert_ob_in_ob(tmp, op);
-            if (op->type == PLAYER)
-                esrv_send_item(op, tmp);
         }
     }
 
@@ -588,7 +581,8 @@ int command_teleport(object *op, char *params) {
  */
 int command_create(object *op, char *params) {
     object *tmp = NULL;
-    int nrof, i, magic, set_magic = 0, set_nrof = 0, gotquote, gotspace;
+    int i, magic, set_magic = 0, set_nrof = 0, gotquote, gotspace;
+    uint32 nrof;
     char buf[MAX_BUF], *cp, *bp = buf, *bp2, *bp3, *bp4, *endline;
     archetype *at, *at_spell = NULL;
     artifact *art = NULL;
@@ -607,7 +601,7 @@ int command_create(object *op, char *params) {
     /* We need to know where the line ends */
     endline = bp+strlen(bp);
 
-    if (sscanf(bp, "%d ", &nrof)) {
+    if (sscanf(bp, "%u ", &nrof)) {
         if ((bp = strchr(params, ' ')) == NULL) {
             new_draw_info(NDI_UNIQUE, 0, op,
                 "Usage: create [nr] [magic] <archetype> [ of <artifact>]"
@@ -616,7 +610,7 @@ int command_create(object *op, char *params) {
         }
         bp++;
         set_nrof = 1;
-        LOG(llevDebug, "%s creates: (%d) %s\n", op->name, nrof, bp);
+        LOG(llevDebug, "%s creates: (%u) %s\n", op->name, nrof, bp);
     }
     if (sscanf(bp, "%d ", &magic)) {
         if ((bp = strchr(bp, ' ')) == NULL) {
@@ -713,7 +707,7 @@ int command_create(object *op, char *params) {
      * just create one object, do all the processing, and then determine
      * if that one object should be inserted or if we need to make copies.
      */
-    tmp = arch_to_object(at);
+    tmp = object_create_arch(at);
     if (settings.real_wiz == FALSE)
         SET_FLAG(tmp, FLAG_WAS_WIZ);
     if (set_magic)
@@ -802,8 +796,17 @@ int command_create(object *op, char *params) {
             tmp->nrof = nrof;
         tmp->map = op->map;
 
-        tmp = insert_ob_in_ob(tmp, op);
-        esrv_send_item(op, tmp);
+        if (at->clone.randomitems != NULL && !at_spell)
+            create_treasure(at->clone.randomitems, tmp, GT_APPLY,
+                op->map->difficulty, 0);
+
+        /* Multipart objects can't be in inventory, put'em on floor. */
+        if (!tmp->more) {
+            tmp = insert_ob_in_ob(tmp, op);
+        }
+        else {
+            insert_ob_in_map_at(tmp, op->map, op, 0, op->x, op->y);
+        }
 
         /* Let's put this created item on stack so dm can access it easily. */
         dm_stack_push(op->contr, tmp->count);
@@ -882,7 +885,6 @@ int command_create(object *op, char *params) {
             if (at->clone.randomitems != NULL && !at_spell)
                 create_treasure(at->clone.randomitems, head, GT_APPLY,
                           op->map->difficulty, 0);
-            esrv_send_item(op, head);
         }
 
         /* free the one we used to copy */
@@ -961,7 +963,7 @@ int command_mon_aggr(object *op, char *params) {
  * This function is severely broken - it has tons of hardcoded values,
  */
 int command_possess(object *op, char *params) {
-    object *victim, *curinv, *nextinv;
+    object *victim;
     player *pl;
     int i;
     char buf[MAX_BUF];
@@ -984,14 +986,6 @@ int command_possess(object *op, char *params) {
         return 1;
     }
 
-    /* clear out the old inventory */
-    curinv = op->inv;
-    while (curinv != NULL) {
-        nextinv = curinv->below;
-        esrv_del_item(op->contr, curinv->count);
-        curinv = nextinv;
-    }
-
     /* make the switch */
     pl = op->contr;
     victim->contr = pl;
@@ -999,13 +993,6 @@ int command_possess(object *op, char *params) {
     victim->type = PLAYER;
     SET_FLAG(victim, FLAG_WIZ);
 
-    /* send the inventory to the client */
-    curinv = victim->inv;
-    while (curinv != NULL) {
-        nextinv = curinv->below;
-        esrv_send_item(victim, curinv);
-        curinv = nextinv;
-    }
     /* basic patchup */
     /* The use of hard coded values is terrible.  Note
      * that really, to be fair, this shouldn't get changed at
@@ -1287,6 +1274,8 @@ int command_abil(object *op, char *params) {
 int command_reset (object *op, char *params) {
     mapstruct *m;
     object *dummy = NULL, *tmp = NULL;
+    const char* path;
+    int res = 0;
 
     if (params == NULL) {
         new_draw_info(NDI_UNIQUE, 0, op, "Reset what map [name]?");
@@ -1294,8 +1283,10 @@ int command_reset (object *op, char *params) {
     }
 
     if (strcmp(params, ".") == 0)
-        params = op->map->path;
-    m = has_been_loaded(params);
+        path = op->map->path;
+    else
+        path = path_combine_and_normalize(op->map->path, params);
+    m = has_been_loaded(path);
     if (m == NULL) {
         new_draw_info(NDI_UNIQUE, 0, op, "No such map.");
         return 1;
@@ -1344,7 +1335,7 @@ int command_reset (object *op, char *params) {
             op->map = NULL;
             tmp = op;
         }
-        swap_map(m);
+        res = swap_map(m);
     }
 
     if (m->in_memory == MAP_SWAPPED) {
@@ -1359,7 +1350,9 @@ int command_reset (object *op, char *params) {
             free_object(dummy);
         }
         return 1;
-    } else {
+    }
+
+    if (res < 0 || m->in_memory != MAP_SWAPPED) {
         player *pl;
         int playercount = 0;
 
@@ -1369,18 +1362,21 @@ int command_reset (object *op, char *params) {
             free_object(dummy);
         }
 
-        new_draw_info(NDI_UNIQUE, 0, op,
-            "Reset failed, couldn't swap map, the following players are on it:");
-        for (pl = first_player; pl != NULL; pl = pl->next) {
-            if (pl->ob->map == m && pl->ob != op) {
-                new_draw_info_format(NDI_UNIQUE, 0, op, "%s", pl->ob->name);
-                playercount++;
+        if (res < 0 && res != SAVE_ERROR_PLAYER)
+            /* no need to warn if player on map, code below checks that. */
+            new_draw_info_format(NDI_UNIQUE | NDI_RED, 0, op, "Reset failed, error code: %d.", res);
+        else {
+            new_draw_info(NDI_UNIQUE, 0, op, "Reset failed, couldn't swap map, the following players are on it:");
+            for (pl = first_player; pl != NULL; pl = pl->next) {
+                if (pl->ob->map == m && pl->ob != op) {
+                    new_draw_info(NDI_UNIQUE, 0, op, pl->ob->name);
+                    playercount++;
+                }
             }
-        }
-        if (!playercount)
-            new_draw_info(NDI_UNIQUE, 0, op,
-                "hmm, I don't see any other players on this map, something else is the problem.");
-        return 1;
+            if (!playercount)
+                new_draw_info(NDI_UNIQUE, 0, op, "hmm, I don't see any other players on this map, something else is the problem.");
+            return 1;
+	}
     }
 }
 
@@ -1388,6 +1384,8 @@ int command_nowiz(object *op, char *params) { /* 'noadm' is alias */
     CLEAR_FLAG(op, FLAG_WIZ);
     CLEAR_FLAG(op, FLAG_WIZPASS);
     CLEAR_FLAG(op, FLAG_WIZCAST);
+    if (op->contr->followed_player)
+        FREE_AND_CLEAR_STR(op->contr->followed_player);
 
     if (settings.real_wiz == TRUE)
         CLEAR_FLAG(op, FLAG_WAS_WIZ);
@@ -1661,6 +1659,13 @@ int command_listplugins(object *op, char *params)
  * Loads the given plugin. The DM specifies the name of the library to load (no
  * pathname is needed). Do not ever attempt to load the same plugin more than
  * once at a time, or bad things could happen.
+ *
+ * @param op
+ * DM loading a plugin.
+ * @param params
+ * should be the plugin's name, eg cfpython.so
+ * @return
+ * 1
  */
 int command_loadplugin(object *op, char *params) {
     char buf[MAX_BUF];
@@ -1674,8 +1679,10 @@ int command_loadplugin(object *op, char *params) {
     strcat(buf, "/plugins/");
     strcat(buf, params);
     LOG(llevDebug, "Requested plugin file is %s\n", buf);
-    if (plugins_init_plugin(buf) == 0)
+    if (plugins_init_plugin(buf) == 0) {
+        LOG(llevInfo, "DM %s loaded plugin %s\n", op->name, params);
         new_draw_info(NDI_UNIQUE, 0, op, "Plugin successfully loaded.");
+    }
     else
         new_draw_info(NDI_UNIQUE, 0, op, "Could not load plugin.");
     return 1;
@@ -1685,6 +1692,13 @@ int command_loadplugin(object *op, char *params) {
  * Unloads the given plugin. The DM specified the ID of the library to unload.
  * Note that some things may behave strangely if the correct plugins are not
  * loaded.
+ *
+ * @param op
+ * DM unloading a plugin.
+ * @param params
+ * should be the plugin's internal name, eg Python
+ * @return
+ * 1
  */
 int command_unloadplugin(object *op, char *params)
 {
@@ -1693,8 +1707,10 @@ int command_unloadplugin(object *op, char *params)
         return 1;
     }
 
-    if (plugins_remove_plugin(params) == 0)
+    if (plugins_remove_plugin(params) == 0) {
+        LOG(llevInfo, "DM %s unloaded plugin %s\n", op->name, params);
         new_draw_info(NDI_UNIQUE, 0, op, "Plugin successfully removed.");
+    }
     else
         new_draw_info(NDI_UNIQUE, 0, op, "Could not remove plugin.");
     return 1;
@@ -1951,7 +1967,8 @@ int command_stack_clear(object *op, char *params) {
  */
 int command_diff(object *op, char *params) {
     object *left, *right, *top;
-    const char *diff;
+    char *diff;
+    StringBuffer *sb;
     int left_from, right_from;
 
     top = NULL;
@@ -1992,14 +2009,15 @@ int command_diff(object *op, char *params) {
         }
     }
 
-    diff = get_ob_diff(left, right);
-
-    if (!diff) {
+    sb = stringbuffer_new();
+    get_ob_diff(sb, left, right);
+    diff = stringbuffer_finish(sb);
+    if (*diff == '\0') {
         new_draw_info(NDI_UNIQUE, 0, op, "Objects are the same.");
-        return 0;
+    } else {
+        new_draw_info(NDI_UNIQUE, 0, op, diff);
     }
-
-    new_draw_info(NDI_UNIQUE, 0, op, diff);
+    free(diff);
     return 0;
 }
 
@@ -2058,9 +2076,8 @@ int command_insert_into(object* op, char *params)
         remove_ob(right);
     inserted = insert_ob_in_ob(right,left);
     if (left->type == PLAYER) {
-        if (inserted == right)
-            esrv_send_item(left,right);
-        else
+        if (inserted != right)
+            /* item was merged, so updating name and such. */
             esrv_update_item(UPD_WEIGHT|UPD_NAME|UPD_NROF,left,inserted);
     }
     new_draw_info_format(NDI_UNIQUE, 0, op, "Inserted %s in %s", query_name(inserted),query_name(left));
@@ -2080,25 +2097,25 @@ int command_style_map_info(object *op, char *params)
 	mapmem += MAP_WIDTH(mp)*MAP_HEIGHT(mp)*(sizeof(object *)+sizeof(MapSpace)) + sizeof(mapstruct);
 	for (x=0; x<MAP_WIDTH(mp); x++) {
 	    for (y=0; y<MAP_HEIGHT(mp); y++) {
-		for (tmp=get_map_ob(mp, x, y); tmp!=NULL; tmp=tmp->above) 
+		for (tmp=get_map_ob(mp, x, y); tmp!=NULL; tmp=tmp->above)
 		    objects_used++;
 	    }
 	}
     }
     draw_ext_info_format(NDI_UNIQUE, 0, op, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_MAPS,
-		 "[fixed]Style maps loaded:    %d", 
-		 "Style maps loaded:    %d", 
+		 "[fixed]Style maps loaded:    %d",
+		 "Style maps loaded:    %d",
 		 maps_used);
     draw_ext_info(NDI_UNIQUE, 0, op, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_MAPS,
-		  "[fixed]Memory used, not", 
+		  "[fixed]Memory used, not",
 		  "Memory used, not");
     draw_ext_info_format(NDI_UNIQUE, 0, op, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_MAPS,
-		 "[fixed]including objects:    %d", 
-		 "including objects:    %d", 
+		 "[fixed]including objects:    %d",
+		 "including objects:    %d",
 		 mapmem);
     draw_ext_info_format(NDI_UNIQUE, 0, op, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_MAPS,
-		 "Style objects:        %d", 
-		 "Style objects:        %d", 
+		 "Style objects:        %d",
+		 "Style objects:        %d",
 		 objects_used);
     draw_ext_info_format(NDI_UNIQUE, 0, op, MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_MAPS,
 		 "Mem for objects:      %d",
@@ -2107,3 +2124,32 @@ int command_style_map_info(object *op, char *params)
     return 0;
 }
 
+/**
+ * DM wants to follow a player, or stop following a player.
+ */
+int command_follow(object* op, char* params) {
+    player* other;
+    if (!params) {
+        if (op->contr->followed_player != NULL) {
+            draw_ext_info_format(NDI_UNIQUE, 0, op, MSG_TYPE_ADMIN, MSG_TYPE_ADMIN_DM, "You stop following %s.", NULL, op->contr->followed_player);
+            FREE_AND_CLEAR_STR(op->contr->followed_player);
+        }
+        return;
+    }
+
+    other = find_player_partial_name(params);
+    if (!other) {
+        draw_ext_info(NDI_UNIQUE, 0, op, MSG_TYPE_ADMIN, MSG_TYPE_ADMIN_DM, "No such player or ambiguous name.", NULL);
+        return;
+    }
+    if (other == op->contr) {
+        draw_ext_info(NDI_UNIQUE, 0, op, MSG_TYPE_ADMIN, MSG_TYPE_ADMIN_DM, "You can't follow yourself.", NULL);
+        return;
+    }
+
+    if (op->contr->followed_player)
+        FREE_AND_CLEAR_STR(op->contr->followed_player);
+
+    op->contr->followed_player = add_string(other->ob->name);
+    draw_ext_info_format(NDI_UNIQUE, 0, op, MSG_TYPE_ADMIN, MSG_TYPE_ADMIN_DM, "Following %s.", NULL, op->contr->followed_player);
+}
