@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_commands_c =
- *   "$Id: commands.c 5803 2007-03-17 22:22:29Z ryo_saeba $";
+ *   "$Id: commands.c 7134 2007-09-09 18:31:23Z ryo_saeba $";
  */
 
 /*
@@ -50,6 +50,7 @@ command_array_struct Commands[] = {
 
   {"sound", command_sound,	0.0},
   {"party", command_party,	0.0},
+  {"party_rejoin", command_party_rejoin,	0.0},
   {"gsay", command_gsay,	1.0},
 
 #ifdef DEBUG
@@ -70,6 +71,7 @@ command_array_struct Commands[] = {
   {"dmhide", command_dmhide,		0.0}, /* Like dm, but don't tell a dm arrived, hide player */
   {"drop", command_drop,	1.0},
   {"dropall", command_dropall,	1.0},
+  {"empty", command_empty,	1.0},
   {"examine", command_examine,	0.5},
   {"explore", command_explore,  0.0},
   {"fix_me", command_fix_me,	0.0},
@@ -236,6 +238,7 @@ command_array_struct WizCommands [] = {
   {"dumpallmaps", command_dumpallmaps,0.0},
   {"dumpallobjects", command_dumpallobjects,0.0},
   {"dumpmap", command_dumpmap,0.0},
+  {"follow", command_follow,0.0},
   {"forget_spell", command_forget_spell, 0.0},
   {"free", command_free,0.0},
   {"freeze", command_freeze,0.0},
