@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_daemon_c =
- *   "$Id: daemon.c 4961 2006-09-21 05:14:18Z mwedel $";
+ *   "$Id: daemon.c 6191 2007-05-05 21:04:41Z ryo_saeba $";
  */
 
 /*
@@ -65,20 +65,14 @@
 #include <stdio.h>
 #include <sys/file.h>
 
-FILE *become_daemon (const char *filename)
+void become_daemon()
 {
-  FILE *logfile;
   register int i;
   int forkresult;
  
-  if((logfile=fopen(filename,"a"))==NULL){
-    fprintf(stderr, "Couldn't create logfile %s: %s\n", filename, strerror_local(errno));
-    exit(0);
-  } 
   fputs("\n========================\n",logfile);    
   fputs("Begin New Server Session\n",logfile);    
   fputs("========================\n\n",logfile);    
-  fflush(logfile);
     /*
      * fork so that the process goes into the background automatically. Also
      * has a nice side effect of having the child process get inherited by
@@ -136,5 +130,4 @@ FILE *become_daemon (const char *filename)
       setpgrp (0, getpid());
 #  endif
 #endif
-  return(logfile);
 }
