@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_disease_c =
- *   "$Id: disease.c 5052 2006-10-28 19:29:09Z ryo_saeba $";
+ *   "$Id: disease.c 6313 2007-05-20 07:10:05Z mwedel $";
  */
 /*
     CrossFire, A Multiplayer game for X-windows
@@ -26,7 +26,7 @@
 */
 
 /*  This file contains all the code implementing diseases,
-    except for odds and ends in attack.c and in 
+    except for odds and ends in attack.c and in
 	 living.c*/
 
 
@@ -42,30 +42,30 @@ title           Message         When the "disease" "infects" something, it will
                                 print "title victim!!!" to the player who owns
 				the "disease".
 wc+             Infectiousness  How well the plague spreads person-to-person
-magic+          Range           range of infection 
+magic+          Range           range of infection
 Stats*          Disability      What stats are reduced by the disease (str con...)
-maxhp+          Persistence     How long the disease can last OUTSIDE the host. 
-value           TimeLeft        Counter for persistence 
-dam^            Damage          How much damage it does (%?). 
-maxgrace+       Duration        How long before the disease is naturally cured. 
-food            DurCount        Counter for Duration 
+maxhp+          Persistence     How long the disease can last OUTSIDE the host.
+value           TimeLeft        Counter for persistence
+dam^            Damage          How much damage it does (%?).
+maxgrace+       Duration        How long before the disease is naturally cured.
+food            DurCount        Counter for Duration
 
-speed           Speed           How often the disease moves. 
-last_sp^        Lethargy        Percentage of max speed--10 = 10% speed. 
+speed           Speed           How often the disease moves.
+last_sp^        Lethargy        Percentage of max speed--10 = 10% speed.
 
-maxsp^          Mana deplete    Saps mana. 
+maxsp^          Mana deplete    Saps mana.
 ac^             Progressiveness How the diseases increases in severity.
-last_eat*^      Deplete food    saps food if negative 
+last_eat*^      Deplete food    saps food if negative
 last_heal       GrantImmunity   If nonzero, disease does NOT grant immunity
-                                when it runs out                              
+                                when it runs out
 
-exp             experience      experience awarded when plague cured 
-hp*^            ReduceRegen     reduces regeneration of disease-bearer 
-sp*^            ReduceSpRegen   reduces spellpoint regeneration 
+exp             experience      experience awarded when plague cured
+hp*^            ReduceRegen     reduces regeneration of disease-bearer
+sp*^            ReduceSpRegen   reduces spellpoint regeneration
 
-name            Name            Name of the plague 
+name            Name            Name of the plague
 msg             message         What the plague says when it strikes.
-race            those affected  species/race the plague strikes (* = everything) 
+race            those affected  species/race the plague strikes (* = everything)
 level           Plague Level    General description of the plague's deadliness
 armour          Attenuation     reduction in wc per generation of disease.
                                 This builds in a self-limiting factor.
@@ -87,7 +87,7 @@ Stats are stat modifications.  These should typically be negative.
 
 maxhp is how long the disease will persist if the host dies and "drops" it,
       in "disease moves", i.e., moves of the disease.  If negative, permanent.
-      
+
 
 value is the counter for maxhp, it starts at maxhp and drops...
 
@@ -147,17 +147,18 @@ static int grant_immunity(object *disease);
 
 
 
-/*  IMPLEMENTATION NOTES  
-	 
+/*  IMPLEMENTATION NOTES
+
 	 Diseases may be contageous.  They are objects which exist in a player's
-inventory.  They themselves do nothing, except modify Symptoms, or 
+inventory.  They themselves do nothing, except modify Symptoms, or
 spread to other live objects.  Symptoms are what actually damage the player:
 these are their own object. */
 
 /* check if victim is susceptible to disease. */
 static int is_susceptible_to_disease(object *victim, object *disease)
 {
-    if(!QUERY_FLAG(victim,FLAG_ALIVE)) return 0;
+    /* Non living and DMs are immune. */
+    if(!QUERY_FLAG(victim,FLAG_ALIVE) || QUERY_FLAG(victim, FLAG_WIZ)) return 0;
 
     if(strstr(disease->race, "*") && !QUERY_FLAG(victim, FLAG_UNDEAD))
 	return 1;
@@ -165,18 +166,22 @@ static int is_susceptible_to_disease(object *victim, object *disease)
     if((disease->race == undead_name) && QUERY_FLAG(victim, FLAG_UNDEAD))
 	return 1;
 
-    if((victim->race && strstr(disease->race, victim->race)) || 
+    if((victim->race && strstr(disease->race, victim->race)) ||
        strstr(disease->race, victim->name))
 	return 1;
-  
+
     return 0;
 }
 
 int move_disease(object *disease) {
     /*  first task is to determine if the disease is inside or outside of someone.
-     * If outside, we decrement 'value' until we're gone. 
+     * If outside, we decrement 'value' until we're gone.
      */
 
+    /* DMs don't infect, and don't suffer either. */
+    if (disease->env && QUERY_FLAG(disease->env, FLAG_WIZ))
+        return 0;
+
     if(disease->env==NULL) { /* we're outside of someone */
 	if ( disease->stats.maxhp > 0 ) disease->value--;
 	if(disease->value==0) {
@@ -214,7 +219,7 @@ int move_disease(object *disease) {
  * disease level and player level and whatnot, a player could get
  * more than one symtpom to a disease.
  */
- 
+
 static int remove_symptoms(object *disease) {
     object *symptom, *victim=NULL;
 
@@ -232,11 +237,11 @@ static object * find_symptom(object *disease) {
   object *walk;
 
   /* check the inventory for symptoms */
-  for(walk=disease->env->inv;walk;walk = walk->below) 
+  for(walk=disease->env->inv;walk;walk = walk->below)
 	 if(!strcmp(walk->name,disease->name)&&walk->type==SYMPTOM) return walk;
   return NULL;
 }
-  
+
 /*  searches around for more victims to infect */
 static int check_infection(object *disease) {
     int x,y,range, mflags;
@@ -245,12 +250,12 @@ static int check_infection(object *disease) {
     sint16 i, j, i2, j2;
 
     range = abs(disease->magic);
-    if(disease->env) { 
+    if(disease->env) {
 	x = disease->env->x;
 	y = disease->env->y;
 	map=disease->env->map;
     }
-    else { 
+    else {
 	x = disease->x;
 	y = disease->y;
 	map = disease->map;
@@ -285,10 +290,16 @@ int infect_object(object *victim, object *disease, int force) {
     if(!QUERY_FLAG(victim,FLAG_MONSTER) && !(victim->type==PLAYER)) return 0;
 
     /* check and see if victim can catch disease:  diseases
-     *  are specific 
+     *  are specific
      */
     if(!is_susceptible_to_disease(victim, disease)) return 0;
 
+    /* If disease is on battleground, only infect other victims on battleground.
+       Not checking results in spectators being infected, which could lead to PK. */
+    if ((disease->map && op_on_battleground(disease, NULL, NULL)) || (disease->env && op_on_battleground(disease->env, NULL, NULL)))
+        if (!op_on_battleground(victim, NULL, NULL))
+            return 0;
+
     /* roll the dice on infection before doing the inventory check!  */
     if(!force && (random_roll(0, 126, victim, PREFER_HIGH) >= disease->stats.wc))
 	return 0;
@@ -304,7 +315,7 @@ int infect_object(object *victim, object *disease, int force) {
      * they were cast in that same order.  Instead, change it so that
      * if you diseased, you can't get diseased more.
      */
-    
+
     for(/* tmp initialized in if, above */;tmp;tmp=tmp->below) {
 	if(tmp->type == SIGN && !strcmp(tmp->name,disease->name) && tmp->level >= disease->level)
 		return 0;  /*Immune! */
@@ -320,7 +331,7 @@ int infect_object(object *victim, object *disease, int force) {
     new_disease->stats.wc -= disease->last_grace;  /* self-limiting factor */
 
     /* Unfortunately, set_owner does the wrong thing to the skills pointers
-     *  resulting in exp going into the owners *current* chosen skill. 
+     *  resulting in exp going into the owners *current* chosen skill.
      */
 
     if(get_owner(disease)) {
@@ -328,7 +339,7 @@ int infect_object(object *victim, object *disease, int force) {
 
 	/* Only need to update skill if different */
 	if (new_disease->skill != disease->skill) {
-	    if (new_disease->skill) free_string(new_disease->skill); 
+	    if (new_disease->skill) free_string(new_disease->skill);
 	    if (disease->skill) new_disease->skill = add_refcount(disease->skill);
 	}
     }
@@ -351,9 +362,9 @@ int infect_object(object *victim, object *disease, int force) {
     if(new_disease->owner && new_disease->owner->type==PLAYER) {
 	 char buf[128];
 	 /* if the disease has a title, it has a special infection message
-	  * This messages is printed in the form MESSAGE victim 
+	  * This messages is printed in the form MESSAGE victim
 	  */
-	 if(new_disease->title) 
+	 if(new_disease->title)
 	    sprintf(buf,"%s %s!!",disease->title,victim->name);
 	 else
 	    sprintf(buf,"You infect %s with your disease, %s!",victim->name,new_disease->name);
@@ -363,7 +374,7 @@ int infect_object(object *victim, object *disease, int force) {
 	 else
 	    new_draw_info(0, 4, new_disease->owner, buf);
     }
-    if(victim->type==PLAYER) 
+    if(victim->type==PLAYER)
 	 new_draw_info(NDI_UNIQUE | NDI_RED,0,victim,"You suddenly feel ill.");
 
     return 1;
@@ -387,9 +398,13 @@ static int do_symptoms(object *disease) {
      * to be found, but this should at least prevent the infinite loops.
      */
 
-    if(victim == NULL || victim==disease) 
+    if(victim == NULL || victim==disease)
 	return 0;/* no-one to inflict symptoms on */
 
+    /* DMs don't suffer from diseases. */
+    if (QUERY_FLAG(victim, FLAG_WIZ))
+        return 0;
+
     symptom = find_symptom(disease);
     if(symptom==NULL) {
 	/* no symptom?  need to generate one! */
@@ -408,7 +423,7 @@ static int do_symptoms(object *disease) {
 		if(!strcmp(tmp->name,disease->name) && tmp->level >= disease->level)
 		    return 0;  /*Immune! */
 	}
-		
+
 	new_symptom = create_archetype(ARCH_SYMPTOM);
 
 	/* Something special done with dam.  We want diseases to be more
@@ -425,8 +440,8 @@ static int do_symptoms(object *disease) {
 	    new_symptom->stats.dam = dam;
 	}
 
- 
-	new_symptom->stats.maxsp = disease->stats.maxsp; 
+
+	new_symptom->stats.maxsp = disease->stats.maxsp;
 	new_symptom->stats.food = new_symptom->stats.maxgrace;
 
 	FREE_AND_COPY(new_symptom->name, disease->name);
@@ -460,7 +475,7 @@ static int do_symptoms(object *disease) {
 	insert_ob_in_ob(new_symptom,victim);
 	return 1;
     }
-  
+
     /* now deal with progressing diseases:  we increase the debility
      * caused by the symptoms.
      */
@@ -524,6 +539,7 @@ int move_symptom(object *symptom) {
     object *victim = symptom->env;
     object *new_ob;
     int sp_reduce;
+    tag_t tag = symptom->count;
 
     if(victim == NULL || victim->map==NULL) {  /* outside a monster/player, die immediately */
         remove_ob(symptom);
@@ -534,13 +550,24 @@ int move_symptom(object *symptom) {
     if(symptom->stats.dam > 0)  hit_player(victim,symptom->stats.dam,symptom,symptom->attacktype,1);
     else hit_player(victim,MAX(1,-victim->stats.maxhp * symptom->stats.dam / 100.0),symptom,symptom->attacktype,1);
 
+    /* In most cases, if the victim has been freed, the logic that
+     * does that will also free the symptom, so check for that.
+     */
+    if(QUERY_FLAG(victim, FLAG_FREED)) {
+	if (!was_destroyed(symptom,tag)) {
+	    remove_ob(symptom);
+	    free_object(symptom);
+	}
+        return 0;
+    }
+
     if(symptom->stats.maxsp>0) sp_reduce = symptom->stats.maxsp;
     else sp_reduce = MAX(1,victim->stats.maxsp * symptom->stats.maxsp/100.0);
     victim->stats.sp = MAX(0,victim->stats.sp - sp_reduce);
 
-    /* create the symptom "other arch" object and drop it here 
+    /* create the symptom "other arch" object and drop it here
      * under every part of the monster
-     * The victim may well have died. 
+     * The victim may well have died.
      */
 
     if(victim->map==NULL) return 0;
@@ -551,14 +578,10 @@ int move_symptom(object *symptom) {
         for(/*tmp initialized above */;tmp!=NULL;tmp=tmp->more) {
             char name[MAX_BUF];
             new_ob = arch_to_object(symptom->other_arch);
-            strcpy(name, victim->name);
-            strncat(name, "'s ", MAX_BUF);
-            strncat(name, new_ob->name, MAX_BUF);
+            snprintf(name, sizeof(name), "%s's %s", victim->name, new_ob->name);
             FREE_AND_COPY(new_ob->name, name);
             if (new_ob->name_pl != NULL) {
-                strcpy(name, victim->name);
-                strncat(name, "'s ", MAX_BUF);
-                strncat(name, new_ob->name_pl, MAX_BUF);
+                snprintf(name, sizeof(name), "%s's %s", victim->name, new_ob->name_pl);
                 FREE_AND_COPY(new_ob->name_pl, name);
             }
             new_ob->x = tmp->x;
@@ -579,7 +602,7 @@ int move_symptom(object *symptom) {
 int check_physically_infect(object *victim, object *hitter) {
   object *walk;
   /* search for diseases, give every disease a chance to infect */
-  for(walk=hitter->inv;walk!=NULL;walk=walk->below) 
+  for(walk=hitter->inv;walk!=NULL;walk=walk->below)
 	 if(walk->type==DISEASE) infect_object(victim,walk,0);
   return 1;
 }
@@ -598,7 +621,7 @@ static object *find_disease(object *victim) {
   return NULL;
 }
 #endif
-	
+
 /* do the cure disease stuff, from the spell "cure disease" */
 
 int cure_disease(object *sufferer,object *caster) {
@@ -625,7 +648,7 @@ int cure_disease(object *sufferer,object *caster) {
 		    remove_symptoms(disease);
 		    remove_ob(disease);
 		    cure=1;
-		    if(caster) change_exp(caster,disease->stats.exp, 
+		    if(caster) change_exp(caster,disease->stats.exp,
 					  caster->chosen_skill?caster->chosen_skill->skill:NULL, 0);
 		    free_object(disease);
 	    }
@@ -647,7 +670,7 @@ int cure_disease(object *sufferer,object *caster) {
  * MSW 2006-06-02
  */
 
-/* reduces disease progression:  reduce_symptoms 
+/* reduces disease progression:  reduce_symptoms
  * return true if we actually reduce a disease.
  */
 
@@ -670,6 +693,5 @@ static int reduce_symptoms(object *sufferer, int reduction) {
 	  new_draw_info(NDI_UNIQUE,0,sufferer,"Your illness seems less severe.");
     return success;
 }
-  
-#endif
 
+#endif
