@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_gods_c =
- *   "$Id: gods.c 5564 2007-02-18 18:34:13Z ryo_saeba $";
+ *   "$Id: gods.c 7624 2007-12-01 10:15:18Z ryo_saeba $";
  */
 
 /*
@@ -56,8 +56,8 @@ static const char *get_god_for_race(const char *race);
 static int lookup_god_by_name(const char *name) {
     int godnr=-1;
     size_t nmlen = strlen(name);
- 
-    if(name && strcmp(name,"none")) { 
+
+    if(name && strcmp(name,"none")) {
 	godlink *gl;
 	for(gl=first_god;gl;gl=gl->next)
 	    if(!strncmp(name,gl->name,MIN(strlen(gl->name),nmlen)))
@@ -73,7 +73,7 @@ static int lookup_god_by_name(const char *name) {
 object *find_god(const char *name) {
     object *god=NULL;
 
-    if(name) { 
+    if(name) {
 	godlink *gl;
 
 	for(gl=first_god;gl;gl=gl->next)
@@ -86,8 +86,8 @@ object *find_god(const char *name) {
 /**
  * Determines if op worships a god.
  * Returns the godname if they do or "none" if they have no god.
- * In the case of an NPC, if they have no god, we try and guess 
- * who they should worship based on their race. If that fails we 
+ * In the case of an NPC, if they have no god, we try and guess
+ * who they should worship based on their race. If that fails we
  * give them a random one.
  */
 
@@ -103,7 +103,7 @@ const char *determine_god(object *op) {
     }
 
     if(op->type!= PLAYER && QUERY_FLAG(op,FLAG_ALIVE)) {
-	
+
 	/* find a god based on race */
 	if(!op->title) {
 	    if (op->race !=NULL) {
@@ -172,18 +172,18 @@ static int same_string (const char *s1, const char *s2)
 static void follower_remove_similar_item (object *op, object *item)
 {
     object *tmp, *next;
-    
+
     if (op && op->type == PLAYER && op->contr) {
         /* search the inventory */
         for (tmp = op->inv; tmp != NULL; tmp = next) {
 	    next = tmp->below;   /* backup in case we remove tmp */
-        
+
 	    if (tmp->type == item->type
 		&& same_string (tmp->name, item->name)
 		&& same_string (tmp->title, item->title)
 		&& same_string (tmp->msg, item->msg)
 		&& same_string (tmp->slaying, item->slaying)) {
-	        
+
 	        /* message */
 		if (tmp->nrof > 1)
 		    new_draw_info_format(NDI_UNIQUE,0,op,
@@ -191,7 +191,7 @@ static void follower_remove_similar_item (object *op, object *item)
 		else
 		    new_draw_info_format(NDI_UNIQUE,0,op,
 		      "The %s crumbles to dust!", query_short_name(tmp));
-	        
+
 	        remove_ob(tmp);    /* remove obj from players inv. */
 		esrv_del_item(op->contr, tmp->count); /* notify client */
 		free_object(tmp);  /* free object */
@@ -241,7 +241,7 @@ static void follower_remove_given_items (object *pl, object *op, object *god)
                                      "The %s crumbles to dust!",
                                      "The %s crumbles to dust!",
                                      query_short_name(tmp));
-                
+
             remove_ob(tmp);    /* remove obj from players inv. */
             esrv_del_item(pl->contr, tmp->count); /* notify client */
             free_object(tmp);
@@ -290,9 +290,7 @@ static int god_gives_present (object *op, object *god, treasure *tr)
      * Mark what god gave it, so it can be taken vengefully later!
      */
     set_ob_key_value(tmp, "divine_giver_name", god->name, TRUE);
-    tmp = insert_ob_in_ob (tmp, op);
-    if (op->type == PLAYER)
-        esrv_send_item (op, tmp);
+    insert_ob_in_ob (tmp, op);
     return 1;
 }
 
@@ -361,15 +359,15 @@ void pray_at_altar(object *pl, object *altar, object *skill) {
                                 "Foul Priest! %s punishes you!",pl_god->name);
 		tmp=create_archetype(LOOSE_MANA);
 		cast_magic_storm(pl,tmp, pl_god->level+20);
-	    } else 
+	    } else
 		new_draw_info_format(NDI_UNIQUE|NDI_NAVY,0,pl,
                                 "Foolish heretic! %s is livid!",pl_god->name);
-	} else 
+	} else
 	    new_draw_info_format(NDI_UNIQUE|NDI_NAVY,0,pl,
                                 "Heretic! %s is angered!",pl_god->name);
- 
+
 	/* whether we will be successfull in defecting or not -
-	 * we lose experience from the clerical experience obj 
+	 * we lose experience from the clerical experience obj
 	 */
 
 	loss = angry * (skill->stats.exp / 10);
@@ -378,7 +376,7 @@ void pray_at_altar(object *pl, object *altar, object *skill) {
 		   skill?skill->skill:"none", SK_SUBTRACT_SKILL_EXP);
 
 	/* May switch Gods, but its random chance based on our current level
-	 * note it gets harder to swap gods the higher we get 
+	 * note it gets harder to swap gods the higher we get
 	 */
 	if((angry==1) && !(random_roll(0, skill->level, pl, PREFER_LOW))) {
 	    become_follower(pl,&altar->other_arch->clone);
@@ -387,7 +385,7 @@ void pray_at_altar(object *pl, object *altar, object *skill) {
 	    new_draw_info(NDI_UNIQUE|NDI_NAVY,0,pl,
                                "A divine force pushes you off the altar.");
 	    move_player(pl,absdir(pl->facing + 4)); /* back him off the way he came. */
-	} 
+	}
     }
 }
 
@@ -466,12 +464,12 @@ void become_follower (object *op, object *new_god) {
     int undeadified = 0; /* Turns to true if changing god can changes the undead
                           * status of the player.*/
     old_god = find_god(determine_god(op));
-    
+
     /* take away any special god-characteristic items. */
     for(item=op->inv;item!=NULL;item=next) {
 	next = item->below;
 	/* remove all invisible startequip items which are
-	 *  not skill, exp or force 
+	 *  not skill, exp or force
  	 */
         if(QUERY_FLAG(item,FLAG_STARTEQUIP) && item->invisible &&
 	   (item->type != SKILL) && (item->type != EXPERIENCE) &&
@@ -485,7 +483,7 @@ void become_follower (object *op, object *new_god) {
 	    free_object(item);
 	}
     }
-    
+
     /* remove any items given by the old god */
     if (old_god) {
         /* Changed to use the new "divine_giver_name" key_value
@@ -495,10 +493,10 @@ void become_follower (object *op, object *new_god) {
          */
         follower_remove_given_items(op, op, old_god);
     }
-    
+
     if(!op||!new_god) return;
 
-    if(op->race && new_god->slaying && strstr(op->race,new_god->slaying)) { 
+    if(op->race && new_god->slaying && strstr(op->race,new_god->slaying)) {
 	new_draw_info_format(NDI_UNIQUE|NDI_NAVY,0,op,"Fool! %s detests your kind!",
 			     new_god->name);
         if(random_roll(0, op->level-1, op, PREFER_LOW)-5>0) {
@@ -524,7 +522,7 @@ void become_follower (object *op, object *new_god) {
 
     /* Player has no skill - give them the skill */
     if (!skop) {
-	/* The arhetype should always be defined - if we crash here because it doesn't,
+	/* The archetype should always be defined - if we crash here because it doesn't,
 	 * things are really messed up anyways.
 	 */
 	skop = give_skill_by_name(op, get_archetype_by_type_subtype(SKILL, SK_PRAYING)->clone.skill);
@@ -543,12 +541,12 @@ void become_follower (object *op, object *new_god) {
         undeadified=1;
     }
 
-    if(skop->title) { /* get rid of old god */ 
+    if(skop->title) { /* get rid of old god */
 	new_draw_info_format(NDI_UNIQUE,0,op,
 	       "%s's blessing is withdrawn from you.",skop->title);
 	/* The point of this is to really show what abilities the player just lost */
 	if (sk_applied||undeadified) {
-	    CLEAR_FLAG(skop,FLAG_APPLIED); 
+	    CLEAR_FLAG(skop,FLAG_APPLIED);
 	    (void) change_abil(op,skop);
 	}
        free_string(skop->title);
@@ -563,7 +561,7 @@ void become_follower (object *op, object *new_god) {
     memcpy(skop->resist, new_god->resist, sizeof(new_god->resist));
 
     /* make sure that certain immunities do NOT get passed
-     * to the follower! 
+     * to the follower!
      */
     for (i=0; i<NROFATTACKS; i++)
       if (skop->resist[i] > 30 && (i==ATNR_FIRE || i==ATNR_COLD ||
@@ -679,7 +677,7 @@ static void update_priest_flag (object *god, object *exp_ob, uint32 flag) {
 	 * the god places on it, this may make sense.  But I don't think
 	 * there is any case like that.
 	 */
-	 
+
 /*        if (!(QUERY_FLAG(&(exp_ob->arch->clone),flag)))*/
 	CLEAR_FLAG(exp_ob,flag);
       };
@@ -730,7 +728,7 @@ static int god_removes_curse (object *op, int remove_damnation)
             CLEAR_FLAG (tmp, FLAG_CURSED);
             CLEAR_FLAG (tmp, FLAG_KNOWN_CURSED);
             if (op->type == PLAYER)
-                esrv_send_item (op, tmp);
+                esrv_update_item (UPD_FLAGS, op, tmp);
         }
     }
 
@@ -775,11 +773,17 @@ static int god_enchants_weapon (object *op, object *god, object *tr, object *ski
     if (weapon == NULL || god_examines_item (god, weapon) <= 0)
         return 0;
 
+    if (weapon->item_power >= MAX_WEAPON_ITEM_POWER) {
+        new_draw_info_format (NDI_UNIQUE, 0, op, "%s considers your %s is not worthy to be enchanted any more.",
+            god->name, weapon->name);
+        return 0;
+    }
+
     /* First give it a title, so other gods won't touch it */
     if ( ! weapon->title) {
         sprintf (buf, "of %s", god->name);
         weapon->title = add_string (buf);
-        if (op->type == PLAYER) 
+        if (op->type == PLAYER)
 	    esrv_update_item (UPD_NAME, op, weapon);
         new_draw_info (NDI_UNIQUE, 0, op, "Your weapon quivers as if struck!");
     }
@@ -790,6 +794,7 @@ static int god_enchants_weapon (object *op, object *god, object *tr, object *ski
         new_draw_info_format (NDI_UNIQUE, 0, op,
                 "Your %s now hungers to slay enemies of your god!",
                 weapon->name);
+	weapon->item_power++;
         return 1;
     }
 
@@ -798,6 +803,7 @@ static int god_enchants_weapon (object *op, object *god, object *tr, object *ski
     if ((attacktype & god->attacktype) != god->attacktype) {
         new_draw_info (NDI_UNIQUE, 0, op, "Your weapon suddenly glows!");
         weapon->attacktype = attacktype | god->attacktype;
+	weapon->item_power++;
         return 1;
     }
 
@@ -809,6 +815,7 @@ static int god_enchants_weapon (object *op, object *god, object *tr, object *ski
         weapon->magic++;
         if (op->type == PLAYER)
             esrv_update_item (UPD_NAME, op, weapon);
+	weapon->item_power++;
         return 1;
     }
 
@@ -818,9 +825,9 @@ static int god_enchants_weapon (object *op, object *god, object *tr, object *ski
 
 /**
  * Every once in a while the god will intervene to help the worshiper.
- * Later, this fctn can be used to supply quests, etc for the 
- * priest. -b.t. 
- * called from pray_at_altar() currently. 
+ * Later, this fctn can be used to supply quests, etc for the
+ * priest. -b.t.
+ * called from pray_at_altar() currently.
  */
 
 static void god_intervention (object *op, object *god, object *skill)
@@ -828,7 +835,7 @@ static void god_intervention (object *op, object *god, object *skill)
     treasure *tr;
 
     if ( ! god || ! god->randomitems) {
-        LOG (llevError, 
+        LOG (llevError,
 	     "BUG: god_intervention(): no god or god without randomitems\n");
         return;
     }
@@ -984,7 +991,7 @@ static void god_intervention (object *op, object *god, object *skill)
             fix_player (op);
             return;
         }
-  
+
         /* Voices */
         if (item->type == BOOK && item->invisible
             && strcmp (item->name, "voice_behind") == 0)
@@ -1031,7 +1038,7 @@ static void god_intervention (object *op, object *god, object *skill)
 
         /* Other gifts */
         if ( ! item->invisible) {
-          if (god_gives_present (op, god, tr)) 
+          if (god_gives_present (op, god, tr))
                 return;
             else
                 continue;
@@ -1058,7 +1065,7 @@ static int god_examines_priest (object *op, object *god) {
     }
 
     /* well, well. Looks like we screwed up. Time for god's revenge */
-    if(reaction<0) { 
+    if(reaction<0) {
 	int loss = 10000000;
 	int angry = abs(reaction);
 
@@ -1094,9 +1101,9 @@ static int god_examines_item(object *god, object *item) {
   if(!item->title) return 1; /* unclaimed item are ok */
 
   sprintf(buf,"of %s",god->name);
-  if(!strcmp(item->title,buf)) return 1; /* belongs to that God */ 
+  if(!strcmp(item->title,buf)) return 1; /* belongs to that God */
 
-  if(god->title) { /* check if we have any enemy blessed item*/ 
+  if(god->title) { /* check if we have any enemy blessed item*/
     sprintf(buf,"of %s",god->title);
     if(!strcmp(item->title,buf)) {
       if(item->env) {
@@ -1108,7 +1115,7 @@ static int god_examines_item(object *god, object *item) {
     }
   }
 
-  return 0; /* item is sacred to a non-enemy god/or is otherwise magical */ 
+  return 0; /* item is sacred to a non-enemy god/or is otherwise magical */
 }
 
 
@@ -1119,19 +1126,19 @@ static int god_examines_item(object *god, object *item) {
  */
 
 /**
- * Returns priest's god's id. 
+ * Returns priest's god's id.
  * Straight calls lookup_god_by_name
  */
 
 static int get_god(object *priest) {
-  int godnr=lookup_god_by_name(determine_god(priest)); 
+  int godnr=lookup_god_by_name(determine_god(priest));
 
   return godnr;
 }
 #endif
 
 /**
- * Returns a string that is the name of the god that should be natively worshipped by a 
+ * Returns a string that is the name of the god that should be natively worshipped by a
  * creature of who has race *race
  * if we can't find a god that is appropriate, we return NULL
  */
@@ -1157,7 +1164,7 @@ static const char *get_god_for_race(const char *race) {
 
 int tailor_god_spell(object *spellop, object *caster) {
     object *god=find_god(determine_god(caster));
-    int caster_is_spell=0; 
+    int caster_is_spell=0;
 
     if (caster->type==SPELL_EFFECT || caster->type == SPELL) caster_is_spell=1;
 
@@ -1178,14 +1185,14 @@ int tailor_god_spell(object *spellop, object *caster) {
     }
 
     /* either holy word or godpower attacks will set the slaying field */
-    if (spellop->attacktype & AT_HOLYWORD || spellop->attacktype & AT_GODPOWER) { 
+    if (spellop->attacktype & AT_HOLYWORD || spellop->attacktype & AT_GODPOWER) {
          if (spellop->slaying) {
              free_string(spellop->slaying);
              spellop->slaying = NULL;
          }
          if(!caster_is_spell)
-            spellop->slaying = add_string(god->slaying);
-	 else if(caster->slaying) 
+            spellop->slaying = god->slaying ? add_string(god->slaying) : NULL;
+	 else if(caster->slaying)
 	    spellop->slaying = add_string(caster->slaying);
     }
 
@@ -1194,17 +1201,17 @@ int tailor_god_spell(object *spellop, object *caster) {
          spellop->attacktype=spellop->attacktype|god->attacktype;
 
     /* tack on the god's name to the spell */
-    if(spellop->attacktype&AT_HOLYWORD||spellop->attacktype&AT_GODPOWER) { 
-         if(spellop->title) 
+    if(spellop->attacktype&AT_HOLYWORD||spellop->attacktype&AT_GODPOWER) {
+         if(spellop->title)
 	   free_string(spellop->title);
          spellop->title=add_string(god->name);
          if(spellop->title){
-	    char buf[MAX_BUF]; 
+	    char buf[MAX_BUF];
 	    sprintf(buf,"%s of %s",spellop->name,spellop->title);
 	    FREE_AND_COPY(spellop->name, buf);
 	    FREE_AND_COPY(spellop->name_pl, buf);
 	}
-    } 
+    }
 
     return 1;
 }
