@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_hiscore_c =
- *   "$Id: hiscore.c 4961 2006-09-21 05:14:18Z mwedel $";
+ *   "$Id: hiscore.c 7278 2007-09-28 05:18:42Z mwedel $";
  */
 
 /*
@@ -240,7 +240,17 @@ static score *add_score(score *new_score) {
   return NULL;
 }
 
-void check_score(object *op) {
+/**
+ * Checks if player should enter the hiscore, and if so writes her into the list.
+ *
+ * @param op
+ * player to check.
+ * @quiet
+ * If set, don't print anything out - used for periodic updates during game
+ * play or when player unexpected quits - don't need to print anything
+ * in those cases
+ */
+void check_score(object *op, int quiet) {
     score new_score;
     score *old_score;
 
@@ -248,22 +258,32 @@ void check_score(object *op) {
 	return;
 
     if(!op->contr->name_changed) {
-	if(op->stats.exp>0) {
+	if(op->stats.exp>0 && !quiet) {
 	    new_draw_info(NDI_UNIQUE, 0,op,"As you haven't changed your name, you won't");
 	    new_draw_info(NDI_UNIQUE, 0,op,"get into the high-score list.");
 	}
 	return;
     }
     if(QUERY_FLAG(op,FLAG_WAS_WIZ)) {
-	new_draw_info(NDI_UNIQUE, 0,op,"Since you have been in wizard mode,");
-	new_draw_info(NDI_UNIQUE, 0,op,"you can't enter the high-score list.");
+	if (!quiet) {
+	    new_draw_info(NDI_UNIQUE, 0,op,"Since you have been in wizard mode,");
+	    new_draw_info(NDI_UNIQUE, 0,op,"you can't enter the high-score list.");
+	}
 	return;
     }
     if (op->contr->explore) {
-	new_draw_info(NDI_UNIQUE, 0,op,"Since you were in explore mode,");
-	new_draw_info(NDI_UNIQUE, 0,op,"you can't enter the high-score list.");
+	if (!quiet) {
+	    new_draw_info(NDI_UNIQUE, 0,op,"Since you were in explore mode,");
+	    new_draw_info(NDI_UNIQUE, 0,op,"you can't enter the high-score list.");
+	}
 	return;
     }
+    if (!op->stats.exp) {
+	if (!quiet)
+	    new_draw_info(NDI_UNIQUE, 0,op, "You don't deserve to save your character yet.");
+	return;
+    }
+
     strncpy(new_score.name,op->name,BIG_NAME);
     new_score.name[BIG_NAME-1] = '\0';
     strncpy(new_score.title,op->contr->own_title,BIG_NAME);
@@ -287,12 +307,20 @@ void check_score(object *op) {
     new_score.maxsp=(int) op->stats.maxsp;
     new_score.maxgrace=(int) op->stats.maxgrace;
     if((old_score=add_score(&new_score))==NULL) {
-	new_draw_info(NDI_UNIQUE, 0,op,"Error in the highscore list.");
+	if (!quiet)
+	    new_draw_info(NDI_UNIQUE, 0,op,"Error in the highscore list.");
 	return;
     }
+    /* Everything below here is just related to print messages
+     * to the player.  If quiet is set, we can just return
+     * now.
+     */
+    if (quiet) return;
+
     if(new_score.position == -1) {
 	new_score.position = HIGHSCORE_LENGTH+1; /* Not strictly correct... */
-	if(!strcmp(old_score->name,new_score.name))
+
+	if(!strcmp(old_score->name,new_score.name)) 
 	    new_draw_info(NDI_UNIQUE, 0,op,"You didn't beat your last highscore:");
 	else
 	    new_draw_info(NDI_UNIQUE, 0,op,"You didn't enter the highscore list:");
