@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_init_c =
- *   "$Id: init.c 5555 2007-02-16 00:06:50Z ryo_saeba $";
+ *   "$Id: init.c 7636 2007-12-01 11:52:48Z ryo_saeba $";
  */
 
 /*
@@ -46,6 +46,8 @@ static racelink *get_racelist(void);
 /** global weathermap */
 weathermap_t **weathermap;
 
+static char default_daemon_log[] = "logfile";
+
 static void set_logfile(char *val) { settings.logfilename=val; }
 static void call_version(void) { version(NULL); exit(0); }
 static void showscores(void) { display_high_score(NULL,9999,NULL); exit(0); }
@@ -62,7 +64,7 @@ static void set_dumpmon7(void) {settings.dumpvalues=7; }
 static void set_dumpmon8(void) {settings.dumpvalues=8; }
 static void set_dumpmon9(void) {settings.dumpvalues=9; }
 static void set_dumpmont(char *name) {settings.dumpvalues=10; settings.dumparg=name; }
-static void set_daemon(void) {settings.daemonmode=1; }
+static void set_daemon(void) {settings.daemonmode=1; if(settings.logfilename[0] == '\0') {settings.logfilename=default_daemon_log;}}
 static void set_datadir(char *path) { settings.datadir=path; }
 static void set_confdir(char *path) { settings.confdir=path; }
 static void set_localdir(char *path) { settings.localdir=path; }
@@ -135,6 +137,7 @@ struct Command_Line_Options options[] = {
 {"-templatedir", 1, 1, set_templatedir},
 {"-tmpdir", 1, 1, set_tmpdir},
 {"-log", 1, 1, set_logfile},
+{"-detach", 0, 1, set_daemon},
 
 #ifdef WIN32
 /* Windows service stuff */
@@ -147,7 +150,6 @@ struct Command_Line_Options options[] = {
  * as they don't require much of anything to bet set up.
  */
 {"-csport", 1, 2, set_csport},
-{"-detach", 0, 2, set_daemon},
 
 /** Start of pass 3 information. In theory, by pass 3, all data paths
  * and defaults should have been set up. 
@@ -404,6 +406,8 @@ static void load_settings(void)
 	    if (has_val) strcpy(settings.meta_host, cp);
 	    else 
 		LOG(llevError,"load_settings: metaserver_host must have a value.\n");
+	} else if (!strcasecmp(buf,"port")) {
+	    set_csport(cp);
 	} else if (!strcasecmp(buf,"metaserver_port")) {
 	    int port = atoi(cp);
 
@@ -718,11 +722,43 @@ static void load_settings(void)
             } else {
                 LOG(llevError, "load_settings: unknown value for allow_denied_spells_writing: %s\n", cp);
         }
+    } else if ( !strcasecmp( buf, "allow_broken_converters" ) ) {
+            if (!strcasecmp(cp, "on") || !strcasecmp(cp, "true")) {
+                settings.allow_broken_converters = TRUE;
+            } else if (!strcasecmp(cp, "off") || !strcasecmp(cp, "false")) {
+                settings.allow_broken_converters = FALSE;
+            } else {
+                LOG(llevError, "load_settings: unknown value for allow_broken_converters: %s\n", cp);
+        }
+    } else if ( !strcasecmp( buf, "log_timestamp" ) ) {
+            if (!strcasecmp(cp, "on") || !strcasecmp(cp, "true")) {
+                settings.log_timestamp = TRUE;
+            } else if (!strcasecmp(cp, "off") || !strcasecmp(cp, "false")) {
+                settings.log_timestamp = FALSE;
+            } else {
+                LOG(llevError, "load_settings: unknown value for log_timestamp: %s\n", cp);
+        }
+    } else if ( !strcasecmp( buf, "log_timestamp_format" ) ) {
+        free(settings.log_timestamp_format);
+        settings.log_timestamp_format = strdup_local(cp);
     } else {
 	    LOG(llevError,"Unknown value in settings file: %s\n", buf);
 	}
     }
     close_and_delete(fp, comp);
+    if (settings.log_timestamp_format == NULL)
+        settings.log_timestamp_format = strdup_local("%y/%m/%d %H:%M:%S");
+
+    /*
+    * The who formats are defined in config to be blank. They should have been
+    * overridden by the settings file, if there are no entries however, it will
+    * have stayed blank. Since this probably isn't what is wanted, we will check if
+    * new formats have been specified, and if not we will use the old defaults.
+    */
+    if (!strcmp(settings.who_format,""))
+        strcpy(settings.who_format, "%N_%T%t%h%d%b%n<%m>");
+    if (!strcmp(settings.who_wiz_format,""))
+        strcpy(settings.who_wiz_format, "%N_%T%t%h%d%b%nLevel %l <%m>(@%i)(%c)");
 }
 
 
@@ -740,7 +776,6 @@ void init(int argc, char **argv) {
 
     init_library();	/* Must be called early */
     load_settings();	/* Load the settings file */
-    init_weather();
     load_materials();
     parse_args(argc, argv, 2);
     fprintf(logfile,"Welcome to CrossFire, v%s\n",VERSION);
@@ -764,12 +799,14 @@ void init(int argc, char **argv) {
 
 #ifndef WIN32 /* ***win32: no become_daemon in windows */
     if (settings.daemonmode)
-	logfile = become_daemon(settings.logfilename[0]=='\0'?"logfile":settings.logfilename);
+	become_daemon();
 #endif
 
     init_beforeplay();
+    init_weather();
     init_ericserver();
     metaserver_init();
+    metaserver2_init();
     reset_sleep();
     init_done=1;
 }
@@ -797,6 +834,7 @@ static void help(void) {
     printf(" -s          Display the high-score list.\n");
     printf(" -score <name or class> Displays all high scores with matching name/class.\n");
     printf(" -v          Print version and contributors.\n");
+    printf(" -conf       Sets the configuration dir (settings, motd, etc.)\n");
     printf(" -data       Sets the lib dir (archetypes, treasures, etc.)\n");
     printf(" -local      Read/write local data (hiscore, unique items, etc.)\n");
     printf(" -maps       Sets the directory for maps.\n");
@@ -965,13 +1003,19 @@ void rec_sigint(int i) {
   fatal_signal(0, 1);
 }
 
+/* SIGHUP handlers on daemons typically make them reread the config
+ * files and reinitialize itself.  This behaviour is better left for
+ * an explicit shutdown and restart with Crossfire, as there is just
+ * too much persistent runtime state.  However, another function of
+ * SIGHUP handlers is to reopen the log file for logrotate's benefit.
+ * We can do that here.
+ */
 void rec_sighup(int i) {
-  LOG(llevInfo,"\nSIGHUP received\n");
-  if(init_done) {
-    emergency_save(0);
-    cleanup();
+  /* Don't call LOG().  It calls non-reentrant functions.  The other
+   * signal handlers shouldn't really call LOG() either. */
+  if(logfile != stderr) {
+    reopen_logfile = 1;
   }
-  exit(0);
 }
 
 void rec_sigquit(int i) {
@@ -1022,7 +1066,12 @@ void fatal_signal(int make_core, int close_sockets) {
 
 static void init_signals(void) {
 #ifndef WIN32 /* init_signals() remove signals */
-  signal(SIGHUP,rec_sighup);
+  struct sigaction sa;
+  sa.sa_sigaction=NULL;
+  sigemptyset(&sa.sa_mask);
+  sa.sa_flags=0;
+  sa.sa_handler = rec_sighup;
+  sigaction(SIGHUP, &sa, NULL);
   signal(SIGINT,rec_sigint);
 #ifndef DEBUG
   signal(SIGQUIT,rec_sigquit);
