@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_login_c =
- *   "$Id: login.c 4961 2006-09-21 05:14:18Z mwedel $";
+ *   "$Id: login.c 7278 2007-09-28 05:18:42Z mwedel $";
  */
 
 /*
@@ -43,8 +43,9 @@ static void copy_file(const char *filename, FILE *fpout);
  * keep the game running.  Thus, we don't want to free any information.
  */
 void emergency_save(int flag) {
-  player *pl;
 #ifndef NO_EMERGENCY_SAVE
+  player *pl;
+
   trying_emergency_save = 1;
   if(editor)
     return;
@@ -72,36 +73,27 @@ void emergency_save(int flag) {
       LOG(llevError, "(failed) ");
       new_draw_info(NDI_UNIQUE, 0,pl->ob,"Emergency save failed, checking score...");
     }
-    check_score(pl->ob);
+    check_score(pl->ob,1);
   }
   LOG(llevError,"\n");
 #else
   LOG(llevInfo,"Emergency saves disabled, no save attempted\n");
 #endif
-  /* If the game is exiting, remove the player locks */
-  if (!flag) {
-    for(pl=first_player;pl!=NULL;pl=pl->next) {
-      if(pl->ob) {
-      }
-    }
-  }
 }
 
-/* Delete character with name.  if new is set, also delete the new
- * style directory, otherwise, just delete the old style playfile
- * (needed for transition)
+/**
+ * Totally deletes a character. The contents of its directory are effectively totally removed.
+ * Used when a player 'quits' the game, or dies on a server with permadeath and no resurrect.
+ *
+ * @param name
+ * player to delete.
  */
-void delete_character(const char *name, int new) {
+void delete_character(const char *name) {
     char buf[MAX_BUF];
 
-    sprintf(buf,"%s/%s/%s.pl",settings.localdir,settings.playerdir,name);
-    if(unlink(buf)== -1)
-	LOG(llevDebug, "Cannot delete character file %s: %s\n", buf, strerror_local(errno));
-    if (new) {
-	sprintf(buf,"%s/%s/%s",settings.localdir,settings.playerdir,name);
-	/* this effectively does an rm -rf on the directory */
-	remove_directory(buf);
-    }
+    snprintf(buf, sizeof(buf), "%s/%s/%s",settings.localdir,settings.playerdir,name);
+    /* this effectively does an rm -rf on the directory */
+    remove_directory(buf);
 }
 
 /* This verify that a character of name exits, and that it matches
@@ -335,6 +327,11 @@ int save_player(object *op, int flag) {
     fprintf(fp,"%d\n",pl->levsp[i]);
 	 fprintf(fp,"%d\n",pl->levgrace[i]);
   }
+  fprintf(fp, "party_rejoin_mode %d\n", pl->rejoin_party);
+  if (pl->party != NULL) {
+      fprintf(fp, "party_rejoin_name %s\n", pl->party->partyname);
+      fprintf(fp, "party_rejoin_password %s\n", pl->party->passwd);
+  }
   fprintf(fp,"endplst\n");
 
   SET_FLAG(op, FLAG_NO_FIX_PLAYER);
@@ -349,13 +346,13 @@ int save_player(object *op, int flag) {
   /* Save objects, but not unpaid objects.  Don't remove objects from
    * inventory.
    */
-  save_object(fp, op, 2);
+  save_object(fp, op, SAVE_FLAG_NO_REMOVE);
   if (flag) {
     op->x = backup_x;
     op->y = backup_y;
   }
 #else
-  save_object(fp, op, 3); /* don't check and don't remove */
+  save_object(fp, op, SAVE_FLAG_SAVE_UNPAID | SAVE_FLAG_NO_REMOVE); /* don't check and don't remove */
 #endif
 
   CLEAR_FLAG(op, FLAG_NO_FIX_PLAYER);
@@ -450,22 +447,12 @@ void check_login(object *op) {
     int correct = 0;
     time_t    elapsed_save_time=0;
     struct stat	statbuf;
+    char *party_name = NULL, party_password[9];
 
     strcpy (pl->maplevel,first_map_path);
+    party_password[0] = 0;
 
-    /* First, lets check for newest form of save */
-    sprintf(filename,"%s/%s/%s/%s.pl",settings.localdir,settings.playerdir,op->name,op->name);
-    if (access(filename, F_OK)==-1) {
-	/* not there,  Try the old style */
-
-	sprintf(filename,"%s/%s/%s.pl",settings.localdir,settings.playerdir,op->name);
-	/* Ok - old style exists.  Lets make the new style directory */
-	if (access(filename, F_OK)==0) {
-	    sprintf(buf,"%s/%s/%s",settings.localdir,settings.playerdir,op->name);
-	    make_path_to_file(buf);
-	}
-    }
-
+    /* Check if this matches a connected player, and if yes disconnect old / connect new. */
     for(pltmp=first_player; pltmp!=NULL; pltmp=pltmp->next) {
 	if(pltmp!=pl && pltmp->ob->name != NULL && !strcmp(pltmp->ob->name,op->name)) {
 	    if (check_password(pl->write_buf+1, pltmp->password)) {
@@ -498,6 +485,8 @@ void check_login(object *op) {
 	}
     }
 
+    sprintf(filename,"%s/%s/%s/%s.pl",settings.localdir,settings.playerdir,op->name,op->name);
+
     /* If no file, must be a new player, so lets get confirmation of
      * the password.  Return control to the higher level dispatch,
      * since the rest of this just deals with loading of the file.
@@ -664,8 +653,21 @@ void check_login(object *op) {
 		LOG(llevDebug, "Error: unknown spell (%s)\n",cp);
 #endif
 	}
+        else if (!strcmp(buf, "party_rejoin_mode")) {
+            pl->rejoin_party = value;
+        }
+        else if (!strcmp(buf, "party_rejoin_name")) {
+            party_name = strdup_local(bufall + strlen("party_rejoin_name") + 1);
+            if (party_name && strlen(party_name) > 0)
+                party_name[strlen(party_name) - 1] = '\0';
+        }
+        else if (!strcmp(buf, "party_rejoin_password")) {
+            snprintf(party_password, sizeof(party_password), bufall + strlen("party_rejoin_password") + 1);
+            if (strlen(party_password) > 0)
+                party_password[strlen(party_password) - 1] = '\0';
+        }
     } /* End of loop loading the character file */
-    leave_map(op);
+    remove_ob(op);
     op->speed=0;
     update_ob_speed(op);
     /*FIXME dangerous call, reset_object should be used to init freshly allocated obj struct!*/
@@ -713,19 +715,12 @@ void check_login(object *op) {
     pl->last_save_tick = pticks;
 #endif
     op->carrying = sum_weight (op);
-    /* Need to call fix_player now - program modified so that it is not
-     * called during the load process (FLAG_NO_FIX_PLAYER set when
-     * saved)
-     * Moved ahead of the esrv functions, so proper weights will be
-     * sent to the client.
-     */
+
     link_player_skills(op);
 
     if ( ! legal_range (op, op->contr->shoottype))
         op->contr->shoottype = range_none;
     
-    fix_player (op);
-    
     /* if it's a dragon player, set the correct title here */
     if (is_dragon_pl(op) && op->inv != NULL) {
         object *tmp, *abil=NULL, *skin=NULL;
@@ -765,9 +760,22 @@ void check_login(object *op) {
      * the data isn't needed.
      */
     esrv_new_player(op->contr,op->weight+op->carrying);
-    esrv_send_inventory(op, op);
+    /* Need to do these after esvr_new_player, as once the client
+     * sees that, it wipes any info it has about the player.
+     */
     esrv_add_spells(op->contr, NULL);
 
+    /* Need to call fix_object now - program modified so that it is not
+     * called during the load process (FLAG_NO_FIX_PLAYER set when
+     * saved)
+     * Moved ahead of the esrv functions, so proper weights will be
+     * sent to the client.  Needs to be after esvr_add_spells, otherwise
+     * we'll try to update spells from fix_object.
+     */
+    fix_player(op);
+
+    esrv_send_inventory(op, op);
+
     CLEAR_FLAG(op, FLAG_FRIENDLY);
 
     /* can_use_shield is a new flag.  However, the can_use.. seems to largely come
@@ -778,5 +786,33 @@ void check_login(object *op) {
      * should be able to use a shield.  However, old Q's won't get that advantage.
      */
     if (QUERY_FLAG(op, FLAG_USE_ARMOUR)) SET_FLAG(op, FLAG_USE_SHIELD);
+
+    /* Rejoin party if needed. */
+    if (pl->rejoin_party != party_rejoin_no && party_name != NULL) {
+        partylist* party;
+        for (party = get_firstparty(); party; party = party->next) {
+            if (strcmp(party_name, party->partyname) == 0)
+                break;
+        }
+        if (!party && pl->rejoin_party == party_rejoin_always) {
+            party = form_party(op, party_name);
+            snprintf(party->passwd, sizeof(party->passwd), party_password);
+        }
+        if (party && strcmp(party->passwd, party_password) == 0) {
+            pl->party = party;
+            snprintf(buf, MAX_BUF, "%s joins party %s", op->name, party->partyname);
+            send_party_message(op, buf);
+        }
+
+        if (pl->party)
+            snprintf(buf, MAX_BUF, "Rejoined party %s.", party->partyname);
+        else
+            snprintf(buf, MAX_BUF, "Couldn't rejoined party %s: %s.", party_name, party ? "invalid password." : "no such party.");
+        draw_ext_info(NDI_UNIQUE,0,op,MSG_TYPE_COMMAND, MSG_TYPE_COMMAND_SUCCESS,
+            buf, NULL);
+    }
+    if (party_name)
+        free(party_name);
+
     return;
 }
