@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_main_c =
- *    "$Id: main.c 4961 2006-09-21 05:14:18Z mwedel $";
+ *    "$Id: main.c 7244 2007-09-26 06:00:09Z mwedel $";
  */
 
 /*
@@ -27,7 +27,9 @@
 */
 
 #include <global.h>
+#ifndef __CEXTRACT__
 #include <sproto.h>
+#endif
 
 int main(int argc, char **argv)
 {
