@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_monster_c =
- *    "$Id: monster.c 5544 2007-02-15 19:44:29Z ryo_saeba $";
+ *    "$Id: monster.c 7568 2007-11-24 21:17:09Z akirschbaum $";
  */
 
 /*
@@ -993,20 +993,66 @@ static int monster_use_range(object *head,object *part,object *pl,int dir)
 	return 0;
     }
 
+/**
+ * Tries to make a (part of a) monster fire a bow.
+ *
+ * Handles confusion effect.
+ *
+ * @param head
+ * head of the monster.
+ * @param part
+ * part of the monster that we use to cast.
+ * @param pl
+ * target.
+ * @param dir
+ * direction to cast.
+ * @return
+ * 1 if monster fired something, 0 else.
+ */
 static int monster_use_bow(object *head, object *part, object *pl, int dir) {
     object *owner;
+    rv_vector rv;
+    sint16 x, y;
+    mapstruct* map;
 
-    if(!(dir=path_to_player(part,pl,0)))
-	return 0;
-    if(QUERY_FLAG(head,FLAG_CONFUSED))
-	dir = absdir(dir + RANDOM()%3 + RANDOM()%3 - 2);
+    get_rangevector(part, pl, &rv, 1);
+    if (rv.distance > 100)
+        /* Too far */
+        return 0;
+    if (rv.distance_x != 0 && rv.distance_y != 0 && abs(rv.distance_x) != abs(rv.distance_y))
+        /* Player must be on same horizontal, vertical or diagonal line. */
+        return 0;
+    dir = absdir(find_dir_2(rv.distance_x, rv.distance_y) + 4);
 
-    if(QUERY_FLAG(head,FLAG_FRIENDLY) && (owner = get_owner(head)) != NULL) {
-	int dir2 = find_dir_2(head->x-owner->x, head->y-owner->y);
-	if(dirdiff(dir,dir2) < 1)
-	    return 0; /* Might hit owner with arrow */
+    if (QUERY_FLAG(head,FLAG_FRIENDLY))
+        owner = get_owner(head);
+    else
+        owner = NULL;
+
+    /* The monster can possibly fire, let's see if the path is ok for an arrow. */
+    x = part->x;
+    y = part->y;
+    map = part->map;
+    while (x != pl->x || y != pl->y || map != pl->map) {
+        x += freearr_x[dir];
+        y += freearr_y[dir];
+        map = get_map_from_coord(map, &x, &y);
+        if (!map) {
+            LOG(llevError, "monster_use_bow: no map but still path exists??\n");
+            return 0;
+        }
+        if ((GET_MAP_MOVE_BLOCK(map, x, y) & MOVE_FLY_LOW) == MOVE_FLY_LOW)
+            return 0;
+        if (owner && owner->x == x && owner->y == y && owner->map == map)
+            /* Don't hit owner! */
+            return 0;
     }
 
+    /* Finally, path is clear, can fire. */
+
+    if(QUERY_FLAG(head,FLAG_CONFUSED))
+        dir = absdir(dir + RANDOM()%3 + RANDOM()%3 - 2);
+
     /* in server/player.c */
     return fire_bow(head, part, NULL, dir, 0, part->x, part->y);
 
@@ -1523,7 +1569,7 @@ void check_doors(object *op, mapstruct *m, int x, int y) {
  * unneeded complication - since the setup of that data is never re-used
  * (say 'hi' to monster, then 'yes', it would re-do the entire parse-message)
  * it seems to me to make more sense to just have simple function that returns
- * the 'text' portion of the message that it matches - this savees us a bunch
+ * the 'text' portion of the message that it matches - this saves us a bunch
  * of malloc's and free's, as well as that setup.
  * This function takes the message to be parsed in 'msg', the text to
  * match in 'match', and returns the portion of the message.  This
@@ -1666,10 +1712,9 @@ static int do_talk_npc(object* op, object* npc, object* override, const char* tx
 }
 
 void npc_say(object *npc, char *cp) {
-    char buf[MAX_BUF];
-    snprintf(buf, sizeof(buf), "%s says:", query_name(npc));
+    char buf[HUGE_BUF];
+    snprintf(buf, sizeof(buf), "%s says: %s", query_name(npc), cp);
     new_info_map(NDI_NAVY|NDI_UNIQUE, npc->map, buf);
-    new_info_map(NDI_NAVY|NDI_UNIQUE, npc->map, cp);
 }
 
 static int talk_to_npc(object *op, object *npc, const char *txt) {
