@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_move_c =
- *    "$Id: move.c 4961 2006-09-21 05:14:18Z mwedel $";
+ *    "$Id: move.c 6719 2007-06-27 20:47:47Z akirschbaum $";
  */
 
 /*
@@ -276,7 +276,10 @@ int teleport (object *teleporter, uint8 tele_type, object *user)
 	    if (k==9) {
 		LOG(llevError,"Shop mat %s (%d, %d) is in solid rock?\n",
 		    other_teleporter->name, other_teleporter->x, other_teleporter->y);
-		return 0;
+		// Teleport player on top of blocked destination: this prevents
+		// players from being trapped inside shops if the destination
+		// is blocked with earth walls.
+		k = 0;
 	    }
 	}
 	else return 0;
