@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_pets_c =
- *    "$Id: pets.c 5206 2006-12-26 19:22:49Z ryo_saeba $";
+ *    "$Id: pets.c 7294 2007-10-03 06:48:38Z mwedel $";
  */
 
 /*
@@ -31,6 +31,20 @@
 #include <sproto.h>
 #endif
 
+/**
+ * Mark all inventory items as FLAG_NO_DROP.
+ *
+ * @param ob
+ * the object to modify.
+ */
+static void mark_inventory_as_no_drop(object *ob) {
+    object *tmp;
+
+    for (tmp = ob->inv; tmp != NULL; tmp = tmp->below) {
+        SET_FLAG(tmp, FLAG_NO_DROP);
+    }
+}
+
 /* given that 'pet' is a friendly object, this function returns a
  * monster the pet should attack, NULL if nothing appropriate is
  * found.  it basically looks for nasty things around the owner
@@ -78,7 +92,7 @@ object *get_pet_enemy(object * pet, rv_vector *rv){
 	if(tmp == owner && !QUERY_FLAG(pet,FLAG_CONFUSED)
 	    && QUERY_FLAG(pet,FLAG_FRIENDLY))
 	    /* without this check, you can actually get pets with
-	     * enemy set to owner! 
+	     * enemy set to owner!
 	     */
             pet->enemy = NULL;
 	else
@@ -103,7 +117,7 @@ object *get_pet_enemy(object * pet, rv_vector *rv){
 	}
     }
 
-    /* Since the pet has no existing enemy, look for anything nasty 
+    /* Since the pet has no existing enemy, look for anything nasty
      * around the owner that it should go and attack.
      */
     tmp3 = NULL;
@@ -119,10 +133,10 @@ object *get_pet_enemy(object * pet, rv_vector *rv){
 
 	    if (QUERY_FLAG(tmp2,FLAG_ALIVE) && ((
 				!QUERY_FLAG(tmp2, FLAG_FRIENDLY) &&
-				(tmp2->type != PLAYER)) || 
+				(tmp2->type != PLAYER)) ||
 			should_arena_attack(pet, owner, tmp2))
 		&& !QUERY_FLAG(tmp2,FLAG_UNAGGRESSIVE) &&
-		tmp2 != pet && tmp2 != owner && 
+		tmp2 != pet && tmp2 != owner &&
 		can_detect_enemy(pet, tmp2, rv)) {
 
 			if (!can_see_enemy(pet, tmp2)) {
@@ -186,11 +200,11 @@ object *get_pet_enemy(object * pet, rv_vector *rv){
 		for (tmp = get_map_ob(nm, x, y); tmp != NULL; tmp = tmp->above) {
 		    object *tmp2 = tmp->head == NULL?tmp:tmp->head;
 		    if (QUERY_FLAG(tmp2,FLAG_ALIVE) && ((
-					!QUERY_FLAG(tmp2, FLAG_FRIENDLY) && 
-					(tmp2->type != PLAYER)) || 
+					!QUERY_FLAG(tmp2, FLAG_FRIENDLY) &&
+					tmp2->type != PLAYER) ||
 				should_arena_attack(pet, owner, tmp2))
 			&& !QUERY_FLAG(tmp2,FLAG_UNAGGRESSIVE) &&
-			tmp2 != pet && tmp2 != owner && 
+			tmp2 != pet && tmp2 != owner &&
 			can_detect_enemy(pet, tmp2, rv)) {
 
 			    if (!can_see_enemy(pet, tmp2)) {
@@ -405,14 +419,14 @@ void pet_move(object * ob)
  ****************************************************************************/
 
 /* fix_summon_pet() - this makes multisquare/single square monsters
- * proper for map insertion. 
+ * proper for map insertion.
  * at is the archetype, op is the caster of the spell, dir is the
  * direction the monster should be placed in.
  * is_golem is to note that this is a golem spell.
  */
 static object *fix_summon_pet(archetype *at, object *op, int dir, int is_golem) {
     archetype *atmp;
-    object *tmp=NULL, *prev=NULL, *head=NULL; 
+    object *tmp=NULL, *prev=NULL, *head=NULL;
 
     for(atmp = at; atmp!=NULL; atmp = atmp->more) {
         tmp = arch_to_object(atmp);
@@ -454,19 +468,10 @@ static object *fix_summon_pet(archetype *at, object *op, int dir, int is_golem)
     }
     head->direction = dir;
 
-    if ((head->type != GOLEM)&&(op->type!= PLAYER)) {
-        /* It isn't a golem, and the caster wasn't a player; that's probably
-         * a monster generated from a trap, thus we should give it treasures
-         * and equipment - gros, 12th August 2006
-         */
-        if (head && head->randomitems) {
-            object *htmp;
-            create_treasure(head->randomitems, head, GT_APPLY | GT_STARTEQUIP,
-                6, 0);
-            for (htmp=head->inv; htmp; htmp=htmp->below)
-                if (!htmp->nrof) SET_FLAG(htmp, FLAG_NO_DROP);
-        }
+    if (head->randomitems) {
+        create_treasure(head->randomitems, head, GT_APPLY | GT_STARTEQUIP, 6, 0);
     }
+    mark_inventory_as_no_drop(head);
 
     /* need to change some monster attr to prevent problems/crashing */
     head->last_heal=0;
@@ -517,9 +522,9 @@ void move_golem(object *op) {
 	return;
     }
 
-    /* Do golem attacks/movement for single & multisq golems. 
-     * Assuming here that op is the 'head' object. Pass only op to 
-     * move_ob (makes recursive calls to other parts) 
+    /* Do golem attacks/movement for single & multisq golems.
+     * Assuming here that op is the 'head' object. Pass only op to
+     * move_ob (makes recursive calls to other parts)
      * move_ob returns 0 if the creature was not able to move.
      */
     tag = op->count;
@@ -527,7 +532,7 @@ void move_golem(object *op) {
     if (was_destroyed (op, tag))
         return;
 
-    for(tmp=op;tmp;tmp=tmp->more) { 
+    for(tmp=op;tmp;tmp=tmp->more) {
 	sint16 x=tmp->x+freearr_x[op->direction],y=tmp->y+freearr_y[op->direction];
 	object *victim;
 	mapstruct *m;
@@ -538,7 +543,7 @@ void move_golem(object *op) {
 
 	if (mflags & P_OUT_OF_MAP) continue;
 
-	for(victim=get_map_ob(op->map,x,y);victim;victim=victim->above) 
+	for(victim=get_map_ob(op->map,x,y);victim;victim=victim->above)
 	    if(QUERY_FLAG(victim,FLAG_ALIVE)) break;
 
 	/* We used to call will_hit_self to make sure we don't
@@ -551,7 +556,7 @@ void move_golem(object *op) {
 	if(victim && victim!=op && victim->head!=op) {
 
 	    /* for golems with race fields, we don't attack
-	     * aligned races 
+	     * aligned races
 	     */
 
 	    if(victim->race && op->race && strstr(op->race,victim->race)) {
@@ -592,8 +597,8 @@ int summon_golem(object *op,object *caster,int dir,object *spob) {
     /* Because there can be different golem spells, player may want to
      * 'lose' their old golem.
      */
-    if(op->type==PLAYER && 
-       op->contr->ranges[range_golem]!=NULL && 
+    if(op->type==PLAYER &&
+       op->contr->ranges[range_golem]!=NULL &&
        op->contr->golem_count == op->contr->ranges[range_golem]->count) {
 	    new_draw_info(NDI_UNIQUE, 0, op, "You dismiss your existing golem.");
 	    remove_ob(op->contr->ranges[range_golem]);
@@ -604,7 +609,7 @@ int summon_golem(object *op,object *caster,int dir,object *spob) {
 
     if (spob->other_arch)
 	at = spob->other_arch;
-    else if (spob->race) { 
+    else if (spob->race) {
 	god = find_god(determine_god(caster));
 
 	if (!god) {
@@ -683,7 +688,7 @@ int summon_golem(object *op,object *caster,int dir,object *spob) {
      * this is a bit simplistic - we are basically just looking at how
      * often the sp doubles and use that as the ratio.
      */
-    tmp->stats.exp *= 1 + (MAX(spob->stats.maxgrace, spob->stats.sp) / 
+    tmp->stats.exp *= 1 + (MAX(spob->stats.maxgrace, spob->stats.sp) /
 			   caster_level(caster, spob));
     tmp->speed_left= 0;
     tmp->direction=dir;
@@ -736,20 +741,20 @@ static object *choose_cult_monster(object *pl, object *god, int summon_level) {
     /* Determine the number of races available */
     racenr=0;
     strcpy(buf,god->race);
-    race = strtok(buf,","); 
-    while(race) { 
-	racenr++; 
-	race = strtok(NULL,",");  
+    race = strtok(buf,",");
+    while(race) {
+	racenr++;
+	race = strtok(NULL,",");
      }
- 
+
     /* next, randomly select a race from the aligned_races string */
-    if(racenr>1) { 
+    if(racenr>1) {
 	racenr = rndm(0, racenr-1);
 	strcpy(buf,god->race);
         race = strtok(buf,",");
-        for(i=0;i<racenr;i++) 
+        for(i=0;i<racenr;i++)
 	     race = strtok(NULL,",");
-    } else 
+    } else
         race = god->race;
 
 
@@ -758,15 +763,15 @@ static object *choose_cult_monster(object *pl, object *god, int summon_level) {
      * checking is good for cases where the god archetypes mismatch the
      * race file
      */
-    if((list=find_racelink(race))==NULL) { 
+    if((list=find_racelink(race))==NULL) {
 	new_draw_info_format(NDI_UNIQUE, 0,pl,
 	    "The spell fails! %s's creatures are beyond the range of your summons",
 	     god->name);
 	LOG(llevDebug,"choose_cult_monster() requested non-existent aligned race!\n");
-	return 0; 
+	return 0;
     }
 
-    /* search for an apprplritate monster on this race list */ 
+    /* search for an apprplritate monster on this race list */
     mon_nr=0;
     for(tobl=list->member;tobl;tobl=tobl->next) {
 	otmp=tobl->ob;
@@ -806,7 +811,7 @@ int summon_object(object *op, object *caster, object *spell_ob, int dir, const c
 	int level = caster_level(caster, spell_ob);
 	treasure *tr, *lasttr=NULL;;
 
-	/* In old code, this was a very convuluted for statement,   
+	/* In old code, this was a very convuluted for statement,
 	 * with all the checks in the 'for' portion itself.  Much
 	 * more readable to break some of the conditions out.
 	 */
@@ -880,7 +885,7 @@ int summon_object(object *op, object *caster, object *spell_ob, int dir, const c
     }
 
     if (dir) {
-        // Only fail if caster specified a blocked direction.
+        /* Only fail if caster specified a blocked direction. */
         x = freearr_x[dir];
         y = freearr_y[dir];
         if (ob_blocked(&summon_arch->clone, op->map, op->x + x, op->y + y)) {
@@ -926,7 +931,7 @@ int summon_object(object *op, object *caster, object *spell_ob, int dir, const c
 			    SET_FLAG(tmp, FLAG_FRIENDLY);
 			    add_friendly_object(tmp);
 			    tmp->stats.exp = 0;
-			    if (spell_ob->attack_movement) 
+			    if (spell_ob->attack_movement)
 				tmp->attack_movement = spell_ob->attack_movement;
 			    if (get_owner(op))
 				set_owner(tmp, get_owner(op));
@@ -943,16 +948,16 @@ int summon_object(object *op, object *caster, object *spell_ob, int dir, const c
 	    prev = tmp;
 	    tmp->x = op->x + x + tmp->arch->clone.x;
 	    tmp->y = op->y + y + tmp->arch->clone.y;
-	    tmp->map = op->map;
+	    tmp->map = get_map_from_coord(op->map, &tmp->x, &tmp->y);
 	}
 	head->direction = freedir[ndir];
 	head->stats.exp = 0;
 	head = insert_ob_in_map(head, head->map, op, 0);
 	if (head && head->randomitems) {
-	    object *tmp;
 	    create_treasure(head->randomitems, head, GT_APPLY | GT_STARTEQUIP, 6, 0);
-	    for (tmp=head->inv; tmp; tmp=tmp->below)
-		if (!tmp->nrof) SET_FLAG(tmp, FLAG_NO_DROP);
+	}
+	if (head != NULL) {
+	    mark_inventory_as_no_drop(head);
 	}
     } /* for i < nrof */
     return 1;
@@ -962,33 +967,33 @@ int summon_object(object *op, object *caster, object *spell_ob, int dir, const c
 is found */
 static object *get_real_owner(object *ob) {
 	object *realowner = ob;
-	
+
 	if (realowner == NULL) return NULL;
-	
+
 	while(get_owner(realowner) != NULL)
 	{
 		realowner = get_owner(realowner);
 	}
 	return realowner;
 }
-		    
+
 /* determines if checks so pets don't attack players or other pets should be
 overruled by the arena petmode */
 int should_arena_attack(object *pet,object *owner,object *target) {
 	object *rowner, *towner;
-	
+
 	/* exit if the target, pet, or owner is null. */
 	if ((target == NULL) || (pet == NULL) || (owner == NULL)) return 0;
-		
+
 	/* get the owners of itself and the target, this is to deal with pets of
 	pets */
 	rowner = get_real_owner(owner);
 	if (target->type != PLAYER) {
 		towner = get_real_owner(target);
 	} else {
-		towner = 0;	
+		towner = 0;
 	}
-	
+
 	/* if the pet has no owner, exit with error */
 	if (rowner == NULL) {
 		LOG(llevError,"Pet has no owner.\n");
@@ -997,32 +1002,32 @@ int should_arena_attack(object *pet,object *owner,object *target) {
 
 	/* if the target is not a player, and has no owner, we shouldn't be here
 	*/
-	if ((towner == NULL) && (target->type != PLAYER)) {
+	if (towner == NULL && target->type != PLAYER) {
 		LOG(llevError,"Target is not a player but has no owner. We should not be here.\n");
 		return 0;
 	}
-	
+
 	/* make sure that the owner is a player */
 	if (rowner->type != PLAYER) return 0;
-	
+
 	/* abort if the petmode is not arena */
 	if (rowner->contr->petmode != pet_arena) return 0;
-	
+
 	/* abort if the pet, it's owner, or the target is not on battleground*/
-	if (!(op_on_battleground(pet, NULL, NULL) && 
-	     op_on_battleground(owner, NULL, NULL) && 
+	if (!(op_on_battleground(pet, NULL, NULL) &&
+	     op_on_battleground(owner, NULL, NULL) &&
 	     op_on_battleground(target, NULL, NULL)))
 	     return 0;
-	
-	/* if the target is a monster, make sure it's owner is not the same */	
-	if ((target->type != PLAYER) && (rowner == towner)) return 0;
+
+	/* if the target is a monster, make sure it's owner is not the same */
+	if (target->type != PLAYER && rowner == towner) return 0;
 
 	/* check if the target is a player which affects how it will handle
 	parties */
 	if (target->type != PLAYER) {
 		/* if the target is owned by a player make sure than make sure
 		it's not in the same party */
-		if ((towner->type == PLAYER) && (rowner->contr->party != NULL)) {
+		if (towner->type == PLAYER && rowner->contr->party != NULL) {
 			if (rowner->contr->party == towner->contr->party) return 0;
 		}
 	} else {
@@ -1032,6 +1037,6 @@ int should_arena_attack(object *pet,object *owner,object *target) {
 			if (rowner->contr->party == target->contr->party) return 0;
 		}
 	}
-	
+
 	return 1;
 }
