@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_rune_c =
- *   "$Id: rune.c 5125 2006-11-19 07:46:37Z mwedel $";
+ *   "$Id: rune.c 6151 2007-04-29 21:35:05Z ryo_saeba $";
  */
 
 /*
@@ -252,7 +252,7 @@ void spring_trap(object *trap,object *victim)
 
     trap->stats.hp--;  /*decrement detcount */
 
-    if(victim && victim->type==PLAYER) 
+    if(victim && victim->type==PLAYER && trap->msg != NULL) 
 	new_draw_info(NDI_UNIQUE, 0,victim,trap->msg);
 
     /*  Flash an image of the trap on the map so the poor sod
