@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_shop_c =
- *   "$Id: shop.c 5550 2007-02-15 20:40:39Z ryo_saeba $";
+ *   "$Id: shop.c 7634 2007-12-01 11:42:44Z ryo_saeba $";
  */
 
 /*
@@ -26,6 +26,7 @@
     The authors can be reached via e-mail at crossfire-devel@real-time.com
 */
 
+#include <assert.h>
 #include <global.h>
 #include <spells.h>
 #include <skills.h>
@@ -339,7 +340,7 @@ static const char *cost_string_from_value(uint64 cost)
     static char buf[MAX_BUF];
     archetype *coin, *next_coin;
     char *endbuf;
-    int num;
+    uint32 num;
     int cointype = LARGEST_COIN_GIVEN;
 
     coin = find_next_coin(cost, &cointype);
@@ -360,7 +361,7 @@ static const char *cost_string_from_value(uint64 cost)
     if (num == 1)
 	sprintf(buf, "1 %s", coin->clone.name);
     else
-	sprintf(buf, "%d %ss", num, coin->clone.name);
+        sprintf(buf, "%u %ss", num, coin->clone.name);
 
     next_coin = find_next_coin(cost, &cointype);
     if (next_coin == NULL)
@@ -378,29 +379,66 @@ static const char *cost_string_from_value(uint64 cost)
 	else
 	    next_coin = find_next_coin(cost, &cointype);
 
-	if (next_coin) {
-	    /* There will be at least one more string to add to the list,
-	     * use a comma.
-	     */
-	    strcat(endbuf, ", "); endbuf += 2;
-	} else {
-	    strcat(endbuf, " and "); endbuf += 5;
-	}
-	if (num == 1)
-	    sprintf(endbuf, "1 %s", coin->clone.name);
-	else
-	    sprintf(endbuf, "%d %ss", num, coin->clone.name);
+        if (next_coin) {
+            /* There will be at least one more string to add to the list,
+             * use a comma.
+             */
+            strcat(endbuf, ", "); endbuf += 2;
+        } else {
+            strcat(endbuf, " and "); endbuf += 5;
+        }
+        if (num == 1)
+            sprintf(endbuf, "1 %s", coin->clone.name);
+        else
+            sprintf(endbuf, "%u %ss", num, coin->clone.name);
     } while (next_coin);
 
     return buf;
 }
 
+/**
+ * Returns a string representing the money's value, in plain coins.
+ *
+ * @param coin
+ * coin. Must be of type MONEY.
+ * @return
+ * value in string, static buffer.
+ * @todo remove static buffer.
+ */
+const char* real_money_value(const object* coin) {
+    static char buf[MAX_BUF];
+
+    assert(coin->type == MONEY);
+
+    snprintf(buf, sizeof(buf), "%ld %s", coin->nrof, coin->nrof == 1 ? coin->name : coin->name_pl);
+    return buf;
+}
+
+/**
+ * Finds the price of an item.
+ *
+ * Price will be either an approximate value or the real value.
+ * @param tmp
+ * object to get the price of.
+ * @param who
+ * who is getting the price.
+ * @param flag
+ * combination of @ref F_xxx "F_xxx" values.
+ * @return
+ * buffer containing the price.
+ * @todo remove evil static buffer thingy.
+ */
 const char *query_cost_string(const object *tmp,object *who,int flag) {
     uint64 real_value = query_cost(tmp,who,flag);
     int idskill1=0;
     int idskill2=0;
     const typedata *tmptype;
 
+    /* money it's pretty hard to not give the exact price, so skip all logic and just return the real value. */
+    if (tmp->type == MONEY) {
+        return real_money_value(tmp);
+    }
+
     tmptype=get_typedata(tmp->type);
     if (tmptype) {
 	idskill1=tmptype->identifyskill;
@@ -612,24 +650,18 @@ static void add_value(object *coin_objs[], sint64 value) {
  * @param objects_len the length of objects
  */
 static void insert_objects(object *pl, object *container, object *objects[], int objects_len) {
-    int i;
+    int i, one = 0;
 
     for (i = 0; i < objects_len; i++) {
         if (objects[i]->nrof > 0) {
             object *tmp = insert_ob_in_ob(objects[i], container);
-
-            esrv_send_item(pl, tmp);
-            esrv_send_item(pl, container);
-            if (pl != container) {
-                esrv_update_item(UPD_WEIGHT, pl, container);
-            }
-            if (pl->type != PLAYER) {
-                esrv_send_item(pl, pl);
-            }
+            one = 1;
         } else {
             free_object(objects[i]);
         }
     }
+    if (one)
+        esrv_update_item(UPD_WEIGHT, pl, container);
 }
 
 /* This pays for the item, and takes the proper amount of money off
@@ -863,12 +895,9 @@ int get_payment(object *pl, object *op) {
                     "You paid %s for %s.",
 		    buf,query_name(op));
             tmp=merge_ob(op,NULL);
-            if (pl->type == PLAYER) {
-                if (tmp) {      /* it was merged */
-                    esrv_del_item (pl->contr, c);
-                    op = tmp;
-                }
-                esrv_send_item(pl, op);
+            if (pl->type == PLAYER && !tmp) {
+                /* If item wasn't merged we update it. If merged, merge_ob handled everything for us. */
+                esrv_update_item(UPD_FLAGS | UPD_NAME, pl, op);
             }
         }
     }
@@ -937,28 +966,24 @@ void sell_item(object *op, object *pl) {
 			if (pouch->weight_limit && (pouch->weight_limit-pouch->carrying)/w<n)
 			    n = (pouch->weight_limit-pouch->carrying)/w;
 
-			tmp = get_object();
-			copy_object(&at->clone, tmp);
-			tmp->nrof = n;
-			i -= (uint64)tmp->nrof * (uint64)tmp->value;
-			tmp = insert_ob_in_ob(tmp, pouch);
-			esrv_send_item (pl, tmp);
-			esrv_send_item (pl, pouch);
-			esrv_update_item (UPD_WEIGHT, pl, pouch);
-			esrv_send_item (pl, pl);
-		    }
-		}
-	    }
-	    if (i/at->clone.value > 0) {
-		tmp = get_object();
-		copy_object(&at->clone, tmp);
-		tmp->nrof = i/tmp->value;
-		i -= (uint64)tmp->nrof * (uint64)tmp->value;
-		tmp = insert_ob_in_ob(tmp, pl);
-		esrv_send_item (pl, tmp);
-		esrv_send_item (pl, pl);
-	    }
-	}
+                        tmp = get_object();
+                        copy_object(&at->clone, tmp);
+                        tmp->nrof = n;
+                        i -= (uint64)tmp->nrof * (uint64)tmp->value;
+                        tmp = insert_ob_in_ob(tmp, pouch);
+                        esrv_update_item(UPD_WEIGHT, pl, pl);
+                    }
+                }
+            }
+            if (i/at->clone.value > 0) {
+                tmp = get_object();
+                copy_object(&at->clone, tmp);
+                tmp->nrof = i/tmp->value;
+                i -= (uint64)tmp->nrof * (uint64)tmp->value;
+                tmp = insert_ob_in_ob(tmp, pl);
+                esrv_update_item (UPD_WEIGHT, pl, pl);
+            }
+        }
     }
 
     if (i!=0) 
