@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_skill_util_c =
- *   "$Id: skill_util.c 5817 2007-03-18 00:06:07Z ryo_saeba $";
+ *   "$Id: skill_util.c 6233 2007-05-10 22:33:25Z ryo_saeba $";
  */
 /*
     CrossFire, A Multiplayer game for X-windows
@@ -70,7 +70,9 @@ void init_skills(void) {
 
     for(at = first_archetype;at!=NULL;at=at->next) {
 	if (at->clone.type == SKILL) {
-	    if (skill_names[at->clone.subtype] != NULL) {
+            if (at->clone.subtype >= sizeof(skill_names)/sizeof(*skill_names)) {
+                LOG(llevError, "init_skills: invalid skill subtype %d for skill %s\n", at->clone.subtype, at->clone.skill);
+            } else if (skill_names[at->clone.subtype] != NULL) {
 		LOG(llevError, "init_skills: multiple skill using same subtype %d, %s, %s\n",
 		    at->clone.subtype, skill_names[at->clone.subtype], at->clone.skill);
 	    } else {
@@ -92,7 +94,7 @@ void init_skills(void) {
 
 /* This function goes through the player inventory and sets
  * up the last_skills[] array in the player object.
- * the last_skills[] is used to more quickly lookup skills - 
+ * the last_skills[] is used to more quickly lookup skills -
  * mostly used for sending exp.
  */
 void link_player_skills(object *op)
@@ -117,12 +119,82 @@ void link_player_skills(object *op)
     }
 }
 
-/* This returns the skill pointer of the given name (the
+/**
+ * This returns specified skill if it can be used, potentially using tool to help.
+ *
+ * Skill and tool can't be NULL at the same time.
+ *
+ * Factored from find_skill_by_name() and find_skill_by_number().
+ *
+ * The skill tool will be applied if not NULL, so the player can benefit from its bonuses.
+ *
+ * @param who
+ * player trying to ready a skill.
+ * @param skill
+ * skill to ready. Can be NULL.
+ * @param skill_tool
+ * skill tool. Can be NULL.
+ * @return
+ * skill object if it can be used, NULL else.
+ * @note
+ * clawing skill is special, as claws are declared as SKILLTOOL but shouldn't be applied.
+ * @todo
+ * rewrite some.
+ */
+static object* adjust_skill_tool(object* who, object* skill, object* skill_tool) {
+
+    if (!skill && !skill_tool)
+        return NULL;
+
+    /* If this is a skill that can be used without a tool and no tool found, return it */
+    if (skill && QUERY_FLAG(skill, FLAG_CAN_USE_SKILL) && (!skill_tool || QUERY_FLAG(skill_tool, FLAG_APPLIED) || strcmp(skill->skill, "clawing")  == 0))
+        return skill;
+
+    /* Player has a tool to use the skill.  If not applied, apply it -
+     * if not successful, return skill if can be used.  If they do have the skill tool
+     * but not the skill itself, give it to them.
+     */
+    if (skill_tool) {
+        if (!QUERY_FLAG(skill_tool, FLAG_APPLIED)) {
+            if (apply_special(who, skill_tool, 0))
+                if (skill && QUERY_FLAG(skill, FLAG_CAN_USE_SKILL))
+                    return skill;
+                else
+                    return NULL;
+
+        }
+        if (!skill) {
+            skill = give_skill_by_name(who, skill_tool->skill);
+            link_player_skills(who);
+        }
+        return skill;
+    }
+    if (skill && QUERY_FLAG(skill, FLAG_CAN_USE_SKILL))
+        return skill;
+    else
+        return NULL;
+}
+
+/**
+ * This returns the skill pointer of the given name (the
  * one that accumlates exp, has the level, etc).
  *
  * It is presumed that the player will be needing to actually
- * use the skill, so thus if use of the skill requires a skill
- * tool, this code will equip it.
+ * use the skill, so a skill tool will be equipped if one if found
+ * to benefit from its bonuses.
+ *
+ * This code is basically the same as find_skill_by_number() below,
+ * but instead of a skill number, we search by the name.
+ *
+ * @param who
+ * Player to get skill.
+ * @param name
+ * skill to find. Needs not to be a shared string.
+ * @return
+ * pointer to skill object, or NULL if player doesn't have it.
+ *
+ * @todo
+ * check if name shouldn't be made a shared string.
  */
 object *find_skill_by_name(object *who, const char *name)
 {
@@ -148,37 +220,27 @@ object *find_skill_by_name(object *who, const char *name)
 		skill_tool = tmp;
 	}
     }
-    /* If this is a skill that can be used without a tool, return it */
-    if (skill && QUERY_FLAG(skill, FLAG_CAN_USE_SKILL)) return skill;
-
-    /* Player has a tool to use the skill.  IF not applied, apply it -
-     * if not successful, return null.  If they do have the skill tool
-     * but not the skill itself, give it to them.
-     */
-    if (skill_tool) {
-	if (!QUERY_FLAG(skill_tool, FLAG_APPLIED)) {
-	    if (apply_special(who, skill_tool, 0)) return NULL;
-	}
-	if (!skill) {
-	    skill = give_skill_by_name(who, skill_tool->skill);
-	    link_player_skills(who);
-	}
-	return skill;
-    }
-    return NULL;
+    return adjust_skill_tool(who, skill, skill_tool);
 }
 
 
-/* This returns the skill pointer of the given name (the
+/**
+ * This returns the skill pointer of the given name (the
  * one that accumlates exp, has the level, etc).
  *
  * It is presumed that the player will be needing to actually
- * use the skill, so thus if use of the skill requires a skill
- * tool, this code will equip it.
- * 
+ * use the skill, so a skill tool will be equipped if one if found
+ * to benefit from its bonuses.
+ *
  * This code is basically the same as find_skill_by_name() above,
- * but instead a skill name, we search by matching number.
- * this replaces find_skill.
+ * but instead of a skill name, we search by matching number.
+ *
+ * @param who
+ * player applying a skill.
+ * @param skillno
+ * skill subtype.
+ * @return
+ * skill object if player can use it, NULL else.
  */
 object *find_skill_by_number(object *who, int skillno)
 {
@@ -187,35 +249,19 @@ object *find_skill_by_number(object *who, int skillno)
     if (skillno < 1 || skillno >= NUM_SKILLS) return NULL;
 
     for (tmp=who->inv; tmp!=NULL; tmp=tmp->below) {
-	if (tmp->type == SKILL && tmp->subtype == skillno) skill = tmp;
+        if (tmp->type == SKILL && tmp->subtype == skillno) skill = tmp;
 
-	/* Try to find appropriate skilltool.  If the player has one already
-	 * applied, we try to keep using that one.
-	 */
-	else if (tmp->type == SKILL_TOOL && tmp->subtype == skillno) {
-	    if (QUERY_FLAG(tmp, FLAG_APPLIED)) skill_tool = tmp;
-	    else if (!skill_tool || !QUERY_FLAG(skill_tool, FLAG_APPLIED))
-		skill_tool = tmp;
-	}
+        /* Try to find appropriate skilltool.  If the player has one already
+         * applied, we try to keep using that one.
+         */
+        else if (tmp->type == SKILL_TOOL && tmp->subtype == skillno) {
+            if (QUERY_FLAG(tmp, FLAG_APPLIED)) skill_tool = tmp;
+            else if (!skill_tool || !QUERY_FLAG(skill_tool, FLAG_APPLIED))
+                skill_tool = tmp;
+        }
     }
-    /* If this is a skill that can be used without a tool, return it */
-    if (skill && QUERY_FLAG(skill, FLAG_CAN_USE_SKILL)) return skill;
 
-    /* Player has a tool to use the skill.  IF not applied, apply it -
-     * if not successful, return null.  If they do have the skill tool
-     * but not the skill itself, give it to them.
-     */
-    if (skill_tool) {
-	if (!QUERY_FLAG(skill_tool, FLAG_APPLIED)) {
-	    if (apply_special(who, skill_tool, 0)) return NULL;
-	}
-	if (!skill) {
-	    skill = give_skill_by_name(who, skill_tool->skill);
-	    link_player_skills(who);
-	}
-	return skill;
-    }
-    return NULL;
+    return adjust_skill_tool(who, skill, skill_tool);
 }
 
 /* This changes the objects skill to new_skill.
@@ -236,7 +282,7 @@ int change_skill (object *who, object *new_skill, int flag)
 
     if ( who->type != PLAYER )
         return 0;
-    
+
     old_range = who->contr->shoottype;
 
     if (who->chosen_skill && who->chosen_skill == new_skill)
@@ -247,7 +293,7 @@ int change_skill (object *who, object *new_skill, int flag)
         return 1;
     }
 
-    if (!new_skill || who->chosen_skill) 
+    if (!new_skill || who->chosen_skill)
 	if (who->chosen_skill) apply_special(who, who->chosen_skill, AP_UNAPPLY | (flag & AP_NOPRINT));
 
     /* Only goal in this case was to unapply a skill */
@@ -278,8 +324,8 @@ void clear_skill(object *who)
 
 /**
  * Main skills use function-similar in scope to cast_spell().
- * We handle all requests for skill use outside of some combat here. 
- * We require a separate routine outside of fire() so as to allow monsters 
+ * We handle all requests for skill use outside of some combat here.
+ * We require a separate routine outside of fire() so as to allow monsters
  * to utilize skills.
  * This is changed (2002-11-30) from the old method that returned
  * exp - no caller needed that info, but it also prevented the callers
@@ -320,7 +366,7 @@ int do_skill (object *op, object *part, object *skill, int dir, const char *stri
 	     * equipped, that should transfer to player, when not,
 	     * shouldn't.
 	     */
-	    if(QUERY_FLAG(skill,FLAG_APPLIED)) { 
+	    if(QUERY_FLAG(skill,FLAG_APPLIED)) {
 		CLEAR_FLAG(skill,FLAG_APPLIED);
 		new_draw_info(NDI_UNIQUE,0,op,"You come to earth.");
 	    }
@@ -440,28 +486,32 @@ int do_skill (object *op, object *part, object *skill, int dir, const char *stri
 	    new_draw_info(NDI_UNIQUE, 0,op,"This skill is already in effect.");
 	    break;
 
+    case SK_HARVESTING:
+        success = do_harvest(op, dir, skill);
+        break;
+
 	default:
 	    LOG(llevDebug,"%s attempted to use unknown skill: %d\n"
                 ,query_name(op), op->chosen_skill->stats.sp);
 	    break;
     }
 
-    /* For players we now update the speed_left from using the skill. 
-     * Monsters have no skill use time because of the random nature in 
+    /* For players we now update the speed_left from using the skill.
+     * Monsters have no skill use time because of the random nature in
      * which use_monster_skill is called already simulates this.
      * If certain skills should take more/less time, that should be
      * in the code for the skill itself.
      */
-    
+
     if(op->type==PLAYER) op->speed_left -= 1.0;
 
     /* this is a good place to add experience for successfull use of skills.
      * Note that add_exp() will figure out player/monster experience
      * gain problems.
      */
- 
+
     if(success && exp) change_exp(op,exp, skill->skill, SK_SUBTRACT_SKILL_EXP);
-            
+
     return success;
 }
 
@@ -504,22 +554,22 @@ int calc_skill_exp(object *who, object *op, object *skill) {
      * If an object is not alive and magical we set the base exp higher to
      * help out exp awards for skill_ident skills. Also, if
      * an item is type RUNE, we give out exp based on stats.Cha
-     * and level (this was the old system) -b.t. 
-     */ 
+     * and level (this was the old system) -b.t.
+     */
 
-    if(!op) { 		/* no item/creature */ 
+    if(!op) { 		/* no item/creature */
         op_lvl= who->map->difficulty < 1 ? 1: who->map->difficulty;
         op_exp = 0;
     } else if(op->type==RUNE || op->type==TRAP) { /* all traps. If stats.Cha > 1 we use that
 				 * for the amount of experience */
         op_exp = op->stats.Cha>1 ? op->stats.Cha : op->stats.exp;
         op_lvl = op->level;
-    } else { 		/* all other items/living creatures */ 
+    } else { 		/* all other items/living creatures */
         op_exp = op->stats.exp;
 	op_lvl = op->level;
-        if(!QUERY_FLAG(op,FLAG_ALIVE)) { /* for ident/make items */ 
+        if(!QUERY_FLAG(op,FLAG_ALIVE)) { /* for ident/make items */
 	    op_lvl += 5 * abs(op->magic);
-	} 
+	}
     }
 
     if(op_lvl<1) op_lvl = 1;
@@ -540,7 +590,7 @@ int calc_skill_exp(object *who, object *op, object *skill) {
 		    lvl_mult = 1.0;	/* no adjustment */
 	    }
 	    else {
-		if (skill->level) 
+		if (skill->level)
 		    lvl_mult = ((float) skill->arch->clone.level * (float) op_lvl) / ((float) skill->level * 100.0);
 		else
 		    lvl_mult = 1.0;
@@ -550,28 +600,28 @@ int calc_skill_exp(object *who, object *op, object *skill) {
 	    lvl_mult = (float) op_lvl / (float) (skill->level?skill->level:1);
 	}
     }
- 
+
     /* assemble the exp total, and return value */
- 
+
     value =  base * lvl_mult;
     if (value < 1) value=1;	/* Always give at least 1 exp point */
-    
+
 #ifdef SKILL_UTIL_DEBUG
       LOG(llevDebug,"calc_skill_exp(): who: %s(lvl:%d)  op:%s(lvl:%d)\n",
 		who->name,skill->level,op->name,op_lvl);
 #endif
-    return ( (int) value); 
+    return ( (int) value);
 }
 
 /* Learn skill. This inserts the requested skill in the player's
  * inventory. The skill field of the scroll should have the
  * exact name of the requested skill.
- * This one actually teaches the player the skill as something 
+ * This one actually teaches the player the skill as something
  * they can equip.
  * Return 0 if the player knows the skill, 1 if the
  * player learns the skill, 2 otherwise.
  */
- 
+
 int
 learn_skill (object *pl, object *scroll) {
     object *tmp;
@@ -623,12 +673,12 @@ static int clipped_percent(sint64 a, sint64 b)
     return 0;
 
   rv = (int)((100.0f * ((float)a) / ((float)b) ) + 0.5f);
-  
+
   if (rv < 0)
     return 0;
   else if (rv > 100)
     return 100;
-  
+
   return rv;
 }
 
@@ -642,7 +692,7 @@ static int clipped_percent(sint64 a, sint64 b)
  * just dumped this as we found it, this would be a bit
  * simpler.
  */
- 
+
 void show_skills(object *op, const char* search) {
     object *tmp=NULL;
     char buf[MAX_BUF];
@@ -694,21 +744,24 @@ void show_skills(object *op, const char* search) {
 	new_draw_info(NDI_UNIQUE, 0, op, skills[i]);
     }
 
-    new_draw_info_format(NDI_UNIQUE, 0, op, 
+    new_draw_info_format(NDI_UNIQUE, 0, op,
 	"You can handle %d weapon improvements.",op->level/5+5);
 
     cp = determine_god(op);
-    new_draw_info_format(NDI_UNIQUE, 0, op, 
+    if (strcmp(cp, "none") == 0)
+        cp = NULL;
+
+    new_draw_info_format(NDI_UNIQUE, 0, op,
 	"You worship %s.", cp?cp:"no god at current time");
 
     new_draw_info_format(NDI_UNIQUE,0,op, "Your equipped item power is %d out of %d\n",
 			 op->contr->item_power, op->level);
 }
 
-/* use_skill() - similar to invoke command, it executes the skill in the 
- * direction that the user is facing. Returns false if we are unable to 
+/* use_skill() - similar to invoke command, it executes the skill in the
+ * direction that the user is facing. Returns false if we are unable to
  * change to the requested skill, or were unable to use the skill properly.
- * This is tricky because skills can have spaces.  We basically roll 
+ * This is tricky because skills can have spaces.  We basically roll
  * our own find_skill_by_name so we can try to do better string matching.
  */
 
@@ -722,12 +775,12 @@ int use_skill(object *op, const char *string) {
 	if (skop->type == SKILL && QUERY_FLAG(skop, FLAG_CAN_USE_SKILL) &&
 	    !strncasecmp(string, skop->skill, MIN(strlen(string), strlen(skop->skill))))
 	    break;
-	else if (skop->type == SKILL_TOOL && 
+	else if (skop->type == SKILL_TOOL &&
 	    !strncasecmp(string, skop->skill, MIN(strlen(string), strlen(skop->skill))))
 	    break;
     }
     if (!skop) {
-	new_draw_info_format(NDI_UNIQUE, 0, op, 
+	new_draw_info_format(NDI_UNIQUE, 0, op,
 		     "Unable to find skill %s", string);
 	return 0;
     }
@@ -746,14 +799,14 @@ int use_skill(object *op, const char *string) {
 	while(*string==0x20) string++;
 	if(strlen(string)==0) string = NULL;
     }
-   
+
 #ifdef SKILL_UTIL_DEBUG
     LOG(llevDebug,"use_skill() got skill: %s\n",sknum>-1?skills[sknum].name:"none");
 #endif
 
     /* Change to the new skill, then execute it. */
     if(do_skill(op,op,skop, op->facing,string)) return 1;
- 
+
     return 0;
 }
 
@@ -818,9 +871,9 @@ static object *find_best_player_hth_skill(object *op)
  * string is passed along to describe what messages to describe
  * the damage.
  */
- 
+
 static int do_skill_attack(object *tmp, object *op, const char *string, object *skill) {
-    int success; 
+    int success;
 
    /* For Players only: if there is no ready weapon, and no "attack" skill
     * is readied either then try to find a skill for the player to use.
@@ -873,8 +926,6 @@ static int do_skill_attack(object *tmp, object *op, const char *string, object *
 		for (tmp=op->inv; tmp; tmp=tmp->below)
 		    if (tmp->type == WEAPON && QUERY_FLAG(tmp, FLAG_APPLIED)) break;
 
-		if (op->current_weapon_script)
-		    FREE_AND_CLEAR_STR(op->current_weapon_script);
 		if (!tmp) {
 		    LOG(llevError,"Could not find applied weapon on %s\n",
 			op->name);
@@ -882,7 +933,6 @@ static int do_skill_attack(object *tmp, object *op, const char *string, object *
 		    return 0;
 		} else {
 		    op->current_weapon = tmp;
-		    op->current_weapon_script=add_string(query_name(tmp));
 		}
 	    }
 
@@ -894,18 +944,18 @@ static int do_skill_attack(object *tmp, object *op, const char *string, object *
     }
 
     /* lose invisiblity/hiding status for running attacks */
- 
+
     if(op->type==PLAYER && op->contr->tmp_invis) {
 	op->contr->tmp_invis=0;
 	op->invisible=0;
 	op->hide=0;
 	update_object(op,UP_OBJ_FACE);
     }
- 
+
     success = attack_ob(tmp,op);
- 
+
     /* print appropriate  messages to the player */
- 
+
     if(success && string!=NULL && tmp && !QUERY_FLAG(tmp,FLAG_FREED)) {
 	if(op->type==PLAYER)
 	    new_draw_info_format(NDI_UNIQUE, 0,op,
@@ -915,7 +965,7 @@ static int do_skill_attack(object *tmp, object *op, const char *string, object *
 		 "%s %s you!",query_name(op),string);
     }
     return success;
-}                         
+}
 
 
 /* skill_attack() - Core routine for use when we attack using a skills
@@ -929,20 +979,20 @@ static int do_skill_attack(object *tmp, object *op, const char *string, object *
  *
  * Initial implementation by -bt thomas@astro.psu.edu
  */
- 
+
 int skill_attack (object *tmp, object *pl, int dir, const char *string, object *skill) {
     sint16 tx,ty;
     mapstruct *m;
     int mflags;
- 
+
     if(!dir) dir=pl->facing;
     tx=freearr_x[dir];
     ty=freearr_y[dir];
- 
+
     /* If we don't yet have an opponent, find if one exists, and attack.
      * Legal opponents are the same as outlined in move_player_attack()
      */
- 
+
     if(tmp==NULL) {
 	m = pl->map;
 	tx = pl->x + freearr_x[dir];
