@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_skills_c =
- *   "$Id: skills.c 5555 2007-02-16 00:06:50Z ryo_saeba $";
+ *   "$Id: skills.c 7629 2007-12-01 10:58:41Z ryo_saeba $";
  */
 /*
     CrossFire, A Multiplayer game for X-windows
@@ -563,9 +563,10 @@ static int do_skill_detect_curse(object *pl, object *skill) {
 		success+= calc_skill_exp(pl,tmp, skill);
 	}
 
-    /* Check ground, too, but only objects the player could pick up */
+    /* Check ground, too, but only objects the player could pick up. Cauldrons are exceptions,
+     * you definitely want to know if they are cursed */
     for(tmp=get_map_ob(pl->map,pl->x,pl->y);tmp;tmp=tmp->above)
-	if (can_pick(pl, tmp) &&
+	if ((can_pick(pl, tmp) || QUERY_FLAG(tmp, FLAG_IS_CAULDRON)) &&
 	    !QUERY_FLAG(tmp,FLAG_IDENTIFIED) &&
 	    !QUERY_FLAG(tmp,FLAG_KNOWN_CURSED)
             && (QUERY_FLAG(tmp,FLAG_CURSED) || QUERY_FLAG(tmp,FLAG_DAMNED)) &&
@@ -629,12 +630,6 @@ static int do_skill_ident2(object *tmp,object *pl, int obj_class, object *skill)
           		new_draw_info(NDI_UNIQUE, 0,pl, "The item has a story:");
           		new_draw_info(NDI_UNIQUE, 0,pl, tmp->msg);
         	    }
-		    /* identify will take care of updating the item if
-		     * it is in the players inventory.  IF on map, do it
-		     * here
-		     */
-		    if (tmp->map)
-			esrv_send_item(pl, tmp);
 		  }
 	          success += calc_skill_exp(pl,tmp, skill);
         	} else
@@ -643,19 +638,59 @@ static int do_skill_ident2(object *tmp,object *pl, int obj_class, object *skill)
 	return success;
 }
 
-/* do_skill_ident() - workhorse for skill_ident() -b.t.
+/**
+ * Workhorse for skill_ident() -b.t.
+ *
+ * @param pl
+ * player identifying.
+ * @param obj_class
+ * type of objects to identify.
+ * @param skill
+ * skill to give experience to.
+ * @return
+ * experience gained by identifying items.
  */
 static int do_skill_ident(object *pl, int obj_class, object *skill) {
     object *tmp;
-    int success=0;
+    int success=0, area, i;
 
+        /* check the player */
     for(tmp=pl->inv;tmp;tmp=tmp->below)
-	success+=do_skill_ident2(tmp,pl,obj_class, skill);
-	 /*  check the ground */
+        success+=do_skill_ident2(tmp,pl,obj_class, skill);
 
-    for(tmp=get_map_ob(pl->map,pl->x,pl->y);tmp;tmp=tmp->above)
-	success+=do_skill_ident2(tmp,pl,obj_class, skill);
+        /*  check the ground */
+        /* Altered to allow ident skills to increase in area with
+         * experience. -- Aaron Baugher
+         */
 
+    if( skill->level > 64 ){   /* Adjust these levels? */
+        area = 49;
+    }
+    else if( skill->level > 16 ){
+        area = 25;
+    }
+    else if( skill->level > 4 ){
+        area = 9;
+    }
+    else {
+        area = 1;
+    }
+
+    for(i=0; i<area; i++) {
+        sint16 x = pl->x+freearr_x[i];
+        sint16 y = pl->y+freearr_y[i];
+        mapstruct *m = pl->map;
+        int mflags;
+
+        mflags =get_map_flags(m, &m, x,y, &x, &y);
+        if (mflags & P_OUT_OF_MAP) continue;
+
+        if( can_see_monsterP(m,pl->x,pl->y,i) ){
+            for(tmp=get_map_ob(m,x,y);tmp;tmp=tmp->above){
+                success+=do_skill_ident2(tmp,pl,obj_class, skill);
+            }
+        }
+    }
     return success;
 }
 
@@ -1145,12 +1180,10 @@ static int write_note(object *pl, object *item, const char *msg, object *skill)
 	    newBook = get_object();
 	    copy_object(item, newBook);
 	    decrease_ob(item);
-	    esrv_send_item(pl, item);
 	    newBook->nrof = 1;
 	    if (newBook->msg) free_string(newBook->msg);
 	    newBook->msg = add_string(buf);
 	    newBook = insert_ob_in_ob(newBook, pl);
-	    esrv_send_item(pl, newBook);
 	} else {
 	    if (item->msg) free_string(item->msg);
 	    item->msg=add_string(buf);
@@ -1177,7 +1210,7 @@ static int write_note(object *pl, object *item, const char *msg, object *skill)
  */
 
 static int write_scroll (object *pl, object *scroll, object *skill) {
-    int success=0,confused=0;
+    int success=0,confused=0, grace_cost = 0;
     object *newscroll, *chosen_spell, *tmp;
 
     /* this is a sanity check */
@@ -1194,7 +1227,9 @@ static int write_scroll (object *pl, object *scroll, object *skill) {
             "You need a spell readied in order to inscribe!");
         return 0;
     }
-    if(SP_level_spellpoint_cost(pl,chosen_spell,SPELL_GRACE) > pl->stats.grace) {
+    /* grace can become negative, we don't want a sp spell to block writing. */
+    grace_cost = SP_level_spellpoint_cost(pl,chosen_spell,SPELL_GRACE);
+    if(grace_cost > 0 && grace_cost > pl->stats.grace) {
         new_draw_info_format(NDI_UNIQUE,0,pl,
             "You don't have enough grace to write a scroll of %s.",
             chosen_spell->name);
@@ -1268,6 +1303,9 @@ static int write_scroll (object *pl, object *scroll, object *skill) {
 	tmp = get_object();
 	copy_object(chosen_spell, tmp);
 	insert_ob_in_ob(tmp, newscroll);
+    /* This is needed so casting from the scroll correctly works with moving_ball types, which
+       check attunements. */
+    newscroll->path_attuned = tmp->path_repelled;
 
 	/* Same code as from treasure.c - so they can better merge.
 	 * if players want to sell them, so be it.
@@ -1281,10 +1319,8 @@ static int write_scroll (object *pl, object *scroll, object *skill) {
     {
         /* Remove to correctly merge with other items which may exist in inventory */
         remove_ob(newscroll);
-        esrv_del_item(pl->contr,newscroll->count);
     }
 	newscroll=insert_ob_in_ob(newscroll,pl);
-	esrv_send_item(pl, newscroll);
 	success = calc_skill_exp(pl,newscroll, skill);
 	if(!confused) success *= 2;
     success = success * skill->level;
