@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_spell_attack_c =
- *   "$Id: spell_attack.c 4961 2006-09-21 05:14:18Z mwedel $";
+ *   "$Id: spell_attack.c 5960 2007-04-07 13:38:59Z ryo_saeba $";
  */
 
 
@@ -1517,7 +1517,7 @@ int mood_change(object *op, object *caster, object *spell) {
 		add_friendly_object(head);
 		head->attack_movement = PETMOVE;
 		done_one = 1;
-		change_exp(op, head->stats.exp / 2, head->skill, SK_EXP_ADD_SKILL);
+		share_exp(op, head->stats.exp / 2, head->skill, SK_EXP_ADD_SKILL);
 		head->stats.exp = 0;
 	    }
 
