@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_spell_effect_c =
- *   "$Id: spell_effect.c 5704 2007-03-06 19:41:03Z ryo_saeba $";
+ *   "$Id: spell_effect.c 6930 2007-08-15 10:10:28Z ryo_saeba $";
  */
 
 
@@ -46,7 +46,7 @@ void cast_magic_storm(object *op, object *tmp, int lvl)
     if (!tmp) return;	/* error */
     tmp->level=op->level;
     tmp->x=op->x;
-    tmp->y=op->y; 
+    tmp->y=op->y;
     tmp->range+=lvl/5;  /* increase the area of destruction */
     tmp->duration+=lvl/5;
 
@@ -57,7 +57,7 @@ void cast_magic_storm(object *op, object *tmp, int lvl)
      */
     if (tmp->duration>=40) tmp->duration=40;
     tmp->stats.dam=lvl; /* nasty recoils! */
-    tmp->stats.maxhp=tmp->count; /* tract single parent */ 
+    tmp->stats.maxhp=tmp->count; /* tract single parent */
     insert_ob_in_map(tmp,op->map,op,0);
 
 }
@@ -125,7 +125,7 @@ int recharge(object *op, object *caster, object *spell_ob) {
  */
 
 /**
- * Takes a living object (op) and turns it into 
+ * Takes a living object (op) and turns it into
  * another monster of some sort.  Currently, we only deal with single
  * space monsters.
  */
@@ -165,7 +165,7 @@ static void polymorph_living(object *op) {
 	else choice--;
     }
 
-    /* Look through the monster.  Unapply anything they have applied, 
+    /* Look through the monster.  Unapply anything they have applied,
      * and remove any spells.  Note that if this is extended
      * to players, that would need to get fixed somehow.
      */
@@ -215,7 +215,7 @@ static void polymorph_living(object *op) {
 
 
 /**
- * Destroys item from polymorph failure 
+ * Destroys item from polymorph failure
  * who is the caster of the polymorph, op is the
  * object destroyed.  We should probably do something
  * more clever ala nethack - create an iron golem or
@@ -252,7 +252,7 @@ static void polymorph_item(object *who, object *op) {
      * certainly be too high below.
      */
     for(at = first_archetype ; at != NULL; at = at->next) {
-	if(at->clone.type == op->type && !at->clone.invisible && 
+	if(at->clone.type == op->type && !at->clone.invisible &&
 	   at->clone.value > 0 && at->clone.value < max_value &&
 	   !QUERY_FLAG(&at->clone, FLAG_NO_DROP) &&
 	   !QUERY_FLAG(&at->clone, FLAG_STARTEQUIP))
@@ -268,8 +268,8 @@ static void polymorph_item(object *who, object *op) {
     do {
 	choice = rndm(0, numat-1);
 	for(at = first_archetype ; at != NULL; at = at->next) {
-	    if(at->clone.type == op->type && !at->clone.invisible && 
-	       at->clone.value > 0 && at->clone.value < max_value && 
+	    if(at->clone.type == op->type && !at->clone.invisible &&
+	       at->clone.value > 0 && at->clone.value < max_value &&
 	       !QUERY_FLAG(&at->clone, FLAG_NO_DROP) &&
 	       !QUERY_FLAG(&at->clone, FLAG_STARTEQUIP)) {
 	      if (!choice) break;
@@ -283,6 +283,7 @@ static void polymorph_item(object *who, object *op) {
     if (new_ob->invisible) {
 	LOG(llevError,"polymorph_item: fix_generated_object made %s invisible?!\n", new_ob->name);
 	free_object(new_ob);
+	return;
     }
 
     /* Unable to generate an acceptable item?  Melt it */
@@ -299,7 +300,7 @@ static void polymorph_item(object *who, object *op) {
     }
 
     /* We don't want rings to keep sustenance/hungry status. There are propably
-     *  other cases too that should be checked. 
+     *  other cases too that should be checked.
      */
     if(charges && op->type != RING && op->type != FOOD)
 	op->stats.food = charges;
@@ -308,7 +309,7 @@ static void polymorph_item(object *who, object *op) {
     new_ob->y = op->y;
     remove_ob(op);
     free_object(op);
-    /* 
+    /*
      * Don't want objects merged or re-arranged, as it then messes up the
      * order
      */
@@ -343,8 +344,8 @@ void polymorph(object *op, object *who) {
      * without archetypes are not good.  As are a few other
      * cases.
      */
-    if(op->type == 0 || op->arch == NULL || 
-       QUERY_FLAG(op,FLAG_NO_PICK) 
+    if(op->type == 0 || op->arch == NULL ||
+       QUERY_FLAG(op,FLAG_NO_PICK)
        || op->move_block || op->type == TREASURE)
 	return;
 
@@ -421,7 +422,7 @@ int cast_polymorph(object *op, object *caster, object *spell_ob, int dir) {
  * create nonnmagic arrows, or even -1, etc...
  */
 
-int cast_create_missile(object *op, object *caster,object *spell, int dir, 
+int cast_create_missile(object *op, object *caster,object *spell, int dir,
     const char *stringarg)
 {
     int missile_plus=0, bonus_plus=0;
@@ -518,7 +519,7 @@ int cast_create_food(object *op,object *caster, object *spell_ob, int dir, const
     archetype *at=NULL;
     object *new_op;
 
-    food_value=spell_ob->stats.food + 
+    food_value=spell_ob->stats.food +
               + 50 * SP_level_duration_adjust(caster,spell_ob);
 
     if(stringarg) {
@@ -528,7 +529,7 @@ int cast_create_food(object *op,object *caster, object *spell_ob, int dir, const
 	if (at == NULL || at->clone.stats.food > food_value)
 	    stringarg = NULL;
     }
-  
+
     if(!stringarg) {
 	archetype *at_tmp;
 
@@ -546,7 +547,7 @@ int cast_create_food(object *op,object *caster, object *spell_ob, int dir, const
 		 * under the limits of the spell and it is higher than
 		 * the item we have now, take it instead.
 		 */
-		if (at_tmp->clone.stats.food<=food_value && 
+		if (at_tmp->clone.stats.food<=food_value &&
 		    (!at || at_tmp->clone.stats.food>at->clone.stats.food))
 			at=at_tmp;
 	    }
@@ -571,7 +572,7 @@ int cast_create_food(object *op,object *caster, object *spell_ob, int dir, const
     cast_create_obj(op, caster,new_op, dir);
     return 1;
 }
-  
+
 int probe(object *op, object *caster, object *spell_ob, int dir) {
     int r, mflags, maxrange;
     object *tmp;
@@ -679,9 +680,9 @@ int cast_invisible(object *op, object *caster, object *spell_ob) {
 
 	op->contr->hidden = 0;
     }
-    if (makes_invisible_to(op, op)) 
+    if (makes_invisible_to(op, op))
 	new_draw_info(NDI_UNIQUE, 0,op,"You can't see your hands!");
-    else 
+    else
 	new_draw_info(NDI_UNIQUE, 0,op,"You feel more transparent!");
 
     update_object(op,UP_OBJ_FACE);
@@ -754,7 +755,7 @@ void execute_word_of_recall(object *op) {
 
 /**
  * Word of recall causes the player to return 'home'.
- * we put a force into the player object, so that there is a 
+ * we put a force into the player object, so that there is a
  * time delay effect.
  */
 int cast_word_of_recall(object *op, object *caster, object *spell_ob) {
@@ -795,7 +796,7 @@ int cast_word_of_recall(object *op, object *caster, object *spell_ob) {
     EXIT_PATH(dummy) = add_string(op->contr->savebed_map);
     EXIT_X(dummy) = op->contr->bed_x;
     EXIT_Y(dummy) = op->contr->bed_y;
-  
+
     (void) insert_ob_in_ob(dummy,op);
     new_draw_info(NDI_UNIQUE, 0,op,"You feel a force starting to build up inside you.");
     return 1;
@@ -840,7 +841,7 @@ int perceive_self(object *op) {
     int i;
 
     tmp=find_god(determine_god(op));
-    if (tmp) 
+    if (tmp)
 	new_draw_info_format(NDI_UNIQUE, 0, op, "You worship %s", tmp->name);
     else
 	new_draw_info(NDI_UNIQUE, 0,op,"You worship no god");
@@ -855,7 +856,7 @@ int perceive_self(object *op) {
 	if (tmp!=NULL) {
 	    for (i=0; i<NUM_STATS; i++) {
 		if (get_attr_value(&tmp->stats, i)<0) {
-		    new_draw_info_format(NDI_UNIQUE, 0,op, 
+		    new_draw_info_format(NDI_UNIQUE, 0,op,
 			 "Your %s is depleted by %d", statname[i],
 			    -(get_attr_value(&tmp->stats,i)));
 		}
@@ -965,7 +966,7 @@ int cast_create_town_portal (object *op, object *caster, object *spell, int dir)
      *    - To create the portal with the name of the player+destination map
      *    - set the owner of the town portal
      *    - To mark the position of the portal in the player's inventory
-     *      for easier destruction. 
+     *      for easier destruction.
      *
      *  The mark works has follow:
      *   slaying: Existing town portal
@@ -1160,7 +1161,7 @@ int cast_create_town_portal (object *op, object *caster, object *spell, int dir)
  */
 
 int magic_wall(object *op,object *caster,int dir,object *spell_ob) {
-    object *tmp, *tmp2;  
+    object *tmp, *tmp2;
     int i,posblocked,negblocked, maxrange;
     sint16 x, y;
     mapstruct *m;
@@ -1247,7 +1248,7 @@ int magic_wall(object *op,object *caster,int dir,object *spell_ob) {
 	insert_ob_in_ob(arch_to_object(tmp->other_arch), tmp);
 
     /*  This code causes the wall to extend some distance in
-     * each direction, or until an obstruction is encountered. 
+     * each direction, or until an obstruction is encountered.
      * posblocked and negblocked help determine how far the
      * created wall can extend, it won't go extend through
      * blocked spaces.
@@ -1258,10 +1259,10 @@ int magic_wall(object *op,object *caster,int dir,object *spell_ob) {
 
     for(i=1; i<=maxrange; i++) {
 	int dir2;
-	 
+
 	dir2 = (dir<4)?(dir+2):dir-2;
-	 
-	x = tmp->x+i*freearr_x[dir2]; 
+
+	x = tmp->x+i*freearr_x[dir2];
 	y = tmp->y+i*freearr_y[dir2];
 	m = tmp->map;
 
@@ -1319,7 +1320,7 @@ int dimension_door(object *op,object *caster, object *spob, int dir) {
     /* Given the new outdoor maps, can't let players dimension door for
      * ever, so put limits in.
      */
-    maxdist = spob->range + 
+    maxdist = spob->range +
 	SP_level_range_adjust(caster, spob);
 
     if(op->contr->count) {
@@ -1329,13 +1330,13 @@ int dimension_door(object *op,object *caster, object *spob, int dir) {
 	}
 
 	for(dist=0;dist<op->contr->count; dist++) {
-	    mflags = get_map_flags(op->map, &m, 
+	    mflags = get_map_flags(op->map, &m,
 		op->x+freearr_x[dir]*(dist+1), op->y+freearr_y[dir]*(dist+1),
 		&sx, &sy);
 
 	    if (mflags & (P_NO_MAGIC | P_OUT_OF_MAP)) break;
 
-	    if ((mflags & P_BLOCKSVIEW) && 
+	    if ((mflags & P_BLOCKSVIEW) &&
 	      OB_TYPE_MOVE_BLOCK(op, GET_MAP_MOVE_BLOCK(m, sx, sy))) break;
 	}
 
@@ -1360,21 +1361,21 @@ int dimension_door(object *op,object *caster, object *spob, int dir) {
 		new_draw_info(NDI_UNIQUE, 0,op,"You cast your spell, but nothing happens.\n");
 		return 1; /* Maybe the penalty should be more severe... */
 	}
-    } else { 
+    } else {
 	/* Player didn't specify a distance, so lets see how far
 	 * we can move the player.  Don't know why this stopped on
 	 * spaces that blocked the players view.
 	 */
 
 	for(dist=0; dist < maxdist; dist++) {
-	    mflags = get_map_flags(op->map, &m, 
-		op->x+freearr_x[dir] * (dist+1), 
+	    mflags = get_map_flags(op->map, &m,
+		op->x+freearr_x[dir] * (dist+1),
 	        op->y+freearr_y[dir] * (dist+1),
 	        &sx, &sy);
 
 	    if (mflags & (P_NO_MAGIC | P_OUT_OF_MAP)) break;
 
-	    if ((mflags & P_BLOCKSVIEW) && 
+	    if ((mflags & P_BLOCKSVIEW) &&
 	      OB_TYPE_MOVE_BLOCK(op, GET_MAP_MOVE_BLOCK(m, sx, sy))) break;
 
 	}
@@ -1542,7 +1543,7 @@ int cast_change_ability(object *op,object *caster,object *spell_ob, int dir, int
     }
 
     if(tmp==NULL) return 0;
-  
+
     /* If we've already got a force of this type, don't add a new one. */
     for(tmp2=tmp->inv; tmp2!=NULL; tmp2=tmp2->below) {
 	if (tmp2->type==FORCE && tmp2->subtype == FORCE_CHANGE_ABILITY)  {
@@ -1563,7 +1564,7 @@ int cast_change_ability(object *op,object *caster,object *spell_ob, int dir, int
 	force=create_archetype(FORCE_NAME);
 	force->subtype = FORCE_CHANGE_ABILITY;
 	free_string(force->name);
-	if (spell_ob->race) 
+	if (spell_ob->race)
 	    force->name = add_refcount(spell_ob->race);
 	else
 	    force->name = add_refcount(spell_ob->name);
@@ -1596,7 +1597,7 @@ int cast_change_ability(object *op,object *caster,object *spell_ob, int dir, int
 	    if (force->resist[i] > 100) force->resist[i] = 100;
 	}
     }
-    if (spell_ob->stats.hp) 
+    if (spell_ob->stats.hp)
 	force->stats.hp = spell_ob->stats.hp + SP_level_dam_adjust(caster,spell_ob);
 
     if (tmp->type == PLAYER) {
@@ -1680,7 +1681,7 @@ int cast_bless(object *op,object *caster,object *spell_ob, int dir) {
 	force=create_archetype(FORCE_NAME);
 	force->subtype = FORCE_CHANGE_ABILITY;
 	free_string(force->name);
-	if (spell_ob->race) 
+	if (spell_ob->race)
 	    force->name = add_refcount(spell_ob->race);
 	else
 	    force->name = add_refcount(spell_ob->name);
@@ -1757,7 +1758,7 @@ int cast_bless(object *op,object *caster,object *spell_ob, int dir) {
  * for the object.  There is also a maximum weight that will be
  * alchemied.
  */
- 
+
 /* I didn't feel like passing these as arguements to the
  * two functions that need them.  Real values are put in them
  * when the spell is cast, and these are freed when the spell
@@ -1779,7 +1780,7 @@ static void alchemy_object(float value_adj, object *obj, int *small_nuggets,
     else
         value *= value_adj;
 
-    
+
     /* Give half of what value_adj says when we alchemy money (This should
      * hopefully make it so that it isn't worth it to alchemy money, sell
      * the nuggets, alchemy the gold from that, etc.
@@ -1819,7 +1820,7 @@ static void place_alchemy_objects(object *op, mapstruct *m, int small_nuggets, i
     int flag=0;
 
     /* Put any nuggets below the player, but we can only pass this
-     * flag if we are on the same space as the player 
+     * flag if we are on the same space as the player
      */
     if (x == op->x && y == op->y && op->map == m) flag = INS_BELOW_ORIGINATOR;
 
@@ -1862,14 +1863,14 @@ int alchemy(object *op, object *caster, object *spell_ob)
     large=create_archetype("largenugget");
     small_value = query_cost(small, NULL, F_TRUE);
     large_value = query_cost(large, NULL, F_TRUE);
-    
+
     /* Set value_adj to be a multiplier for how much of the original value
      * will be in the nuggets. Starts at 0.05, increasing by 0.01 per casting
      * level, maxing out at 0.40.
      */
-    value_adj = (SP_level_dam_adjust(caster, spell_ob) / 
+    value_adj = (SP_level_dam_adjust(caster, spell_ob) /
         100.00) + 0.05;
-        
+
     if (value_adj > 0.40) value_adj = 0.40;
 
     for(y= op->y-1;y<=op->y+1;y++) {
@@ -1912,7 +1913,7 @@ int alchemy(object *op, object *caster, object *spell_ob)
 			}
 		    }
 		    alchemy_object(value_adj, tmp, &small_nuggets, &large_nuggets, &weight);
-	    
+
 		    if (weight>weight_max) {
 			place_alchemy_objects(op, mp, small_nuggets, large_nuggets, nx, ny);
 			free_object(large);
@@ -1962,7 +1963,7 @@ int remove_curse(object *op, object *caster, object *spell) {
 		CLEAR_FLAG(tmp, FLAG_KNOWN_CURSED);
 		tmp->value = 0; /* Still can't sell it */
 		if (op->type == PLAYER)
-		    esrv_send_item(op, tmp);
+		    esrv_update_item(UPD_FLAGS, op, tmp);
 	    }
 	}
 
@@ -1972,7 +1973,7 @@ int remove_curse(object *op, object *caster, object *spell) {
 	} else {
 	    if (was_one)
 		new_draw_info(NDI_UNIQUE, 0,op, "You failed to remove the curse.");
-	    else 
+	    else
 		new_draw_info(NDI_UNIQUE, 0,op, "You are not using any cursed items.");
 	}
     }
@@ -2022,7 +2023,7 @@ int cast_identify(object *op, object *caster, object *spell) {
 		    new_draw_info(NDI_UNIQUE, 0,op, "The item has a story:");
 		    new_draw_info(NDI_UNIQUE, 0,op, tmp->msg);
 		}
-		esrv_send_item(op, tmp);
+		esrv_update_item(UPD_FLAGS | UPD_NAME, op, tmp);
 	    }
 	    num_ident--;
 	    success=1;
@@ -2080,9 +2081,9 @@ int cast_detection(object *op, object *caster, object *spell, object *skill) {
 		/* show invisible */
 		if (QUERY_FLAG(spell, FLAG_MAKE_INVIS) &&
 		    /* Might there be other objects that we can make visibile? */
-		    (tmp->invisible && (QUERY_FLAG(tmp, FLAG_MONSTER) || 
+		    (tmp->invisible && (QUERY_FLAG(tmp, FLAG_MONSTER) ||
 		    (tmp->type==PLAYER && !QUERY_FLAG(tmp, FLAG_WIZ)) ||
-		     tmp->type==CF_HANDLE || 
+		     tmp->type==CF_HANDLE ||
 		     tmp->type==TRAPDOOR || tmp->type==EXIT || tmp->type==HOLE ||
 		     tmp->type==BUTTON || tmp->type==TELEPORTER ||
 		     tmp->type==GATE || tmp->type==LOCKED_DOOR ||
@@ -2112,8 +2113,8 @@ int cast_detection(object *op, object *caster, object *spell, object *skill) {
 		 */
 
 		/* detect magic */
-		if (QUERY_FLAG(spell, FLAG_KNOWN_MAGICAL) && 
-		    !QUERY_FLAG(tmp,FLAG_KNOWN_MAGICAL) && 
+		if (QUERY_FLAG(spell, FLAG_KNOWN_MAGICAL) &&
+		    !QUERY_FLAG(tmp,FLAG_KNOWN_MAGICAL) &&
 		    !QUERY_FLAG(tmp, FLAG_IDENTIFIED) &&
 		    is_magical(tmp)) {
 			SET_FLAG(tmp,FLAG_KNOWN_MAGICAL);
@@ -2171,7 +2172,7 @@ int cast_detection(object *op, object *caster, object *spell, object *skill) {
 	done_one = 0;
 	for (tmp = op->inv; tmp; tmp = tmp->below) {
 	    if (!tmp->invisible && !QUERY_FLAG(tmp, FLAG_IDENTIFIED)) {
-		if (QUERY_FLAG(spell, FLAG_KNOWN_MAGICAL) && 
+		if (QUERY_FLAG(spell, FLAG_KNOWN_MAGICAL) &&
 		    is_magical(tmp) && !QUERY_FLAG(tmp,FLAG_KNOWN_MAGICAL)) {
 			SET_FLAG(tmp,FLAG_KNOWN_MAGICAL);
 			if (op->type==PLAYER)
@@ -2319,7 +2320,7 @@ void counterspell(object *op,int dir)
     m = op->map;
     mflags = get_map_flags(m, &m, sx, sy, &sx, &sy);
     if (mflags & P_OUT_OF_MAP) return;
-    
+
     for(tmp=get_map_ob(m,sx,sy); tmp!=NULL; tmp=next) {
 	next = tmp->above;
 
@@ -2333,19 +2334,19 @@ void counterspell(object *op,int dir)
        /* don't attack our own spells */
        if(tmp->owner && tmp->owner == op->owner) continue;
 
-	/* Basically, if the object is magical and not counterspell, 
+	/* Basically, if the object is magical and not counterspell,
 	 * we will more or less remove the object.  Don't counterspell
-	 * monsters either. 
+	 * monsters either.
          */
 
 	if (head->attacktype & AT_MAGIC &&
-	   !(head->attacktype & AT_COUNTERSPELL) && 
-	   !QUERY_FLAG(head,FLAG_MONSTER) && 
+	   !(head->attacktype & AT_COUNTERSPELL) &&
+	   !QUERY_FLAG(head,FLAG_MONSTER) &&
 	   (op->level > head->level)) {
 		remove_ob(head);
 		free_object(head);
 	} else switch(head->type) {
-	    case SPELL_EFFECT: 
+	    case SPELL_EFFECT:
 		if((op->level > head->level) && !op->stats.food && !op->speed_left) {
 		    remove_ob(head);
 		    free_object(head);
@@ -2409,10 +2410,6 @@ int cast_consecrate(object *op, object *caster, object *spell) {
                 new_altar->y = tmp->y;
                 new_altar->level = tmp->level;
                 insert_ob_in_map(new_altar,tmp->map,tmp,INS_BELOW_ORIGINATOR);
-                if(op->type==PLAYER) {
-                    esrv_del_item(op->contr, tmp->count);
-                    esrv_send_item(op, new_altar);
-                }
                 remove_ob(tmp);
                 new_draw_info_format(NDI_UNIQUE,0, op,"You consecrated the altar to %s!",god->name);
                 return 1;
@@ -2426,7 +2423,7 @@ int cast_consecrate(object *op, object *caster, object *spell) {
 /**
  * Generalization of staff_to_snake.  Makes a golem out of the caster's weapon.
  * The golem is based on the archetype specified, modified by the caster's level
- * and the attributes of the weapon.  The weapon is inserted in the golem's 
+ * and the attributes of the weapon.  The weapon is inserted in the golem's
  * inventory so that it falls to the ground when the golem dies.
  * This code was very odd - code early on would only let players use the spell,
  * yet the code wass full of player checks.  I've presumed that the code
@@ -2441,7 +2438,7 @@ int animate_weapon(object *op,object *caster,object *spell, int dir) {
     sint16 x, y;
     mapstruct *m;
     materialtype_t *mt;
- 
+
     if(!spell->other_arch){
 	new_draw_info(NDI_UNIQUE, 0,op,"Oops, program error!");
 	LOG(llevError,"animate_weapon failed: spell %s missing other_arch!\n", spell->name);
@@ -2457,7 +2454,7 @@ int animate_weapon(object *op,object *caster,object *spell, int dir) {
     }
 
     /* if no direction specified, pick one */
-    if(!dir) 
+    if(!dir)
 	dir=find_free_spot(NULL,op->map,op->x,op->y,1,9);
 
     m = op->map;
@@ -2494,7 +2491,7 @@ int animate_weapon(object *op,object *caster,object *spell, int dir) {
 
     if (weapon->nrof > 1) {
 	tmp = get_split_ob(weapon, 1);
-	esrv_send_item(op, weapon);
+	esrv_update_item(UPD_NROF, op, weapon);
 	weapon = tmp;
     }
 
@@ -2518,13 +2515,13 @@ int animate_weapon(object *op,object *caster,object *spell, int dir) {
      * used above.
      */
     if (!QUERY_FLAG(weapon, FLAG_REMOVED))
-	remove_ob (weapon);
+        remove_ob (weapon);
     insert_ob_in_ob (weapon, tmp);
-    esrv_send_item(op, weapon);
+
     /* To do everything necessary to let a golem use the weapon is a pain,
      * so instead, just set it as equipped (otherwise, we need to update
      * body_info, skills, etc)
-     */		   
+     */
     SET_FLAG (tmp, FLAG_USE_WEAPON);
     SET_FLAG(weapon, FLAG_APPLIED);
     fix_player(tmp);
@@ -2535,13 +2532,13 @@ int animate_weapon(object *op,object *caster,object *spell, int dir) {
      */
 
     /* modify weapon's animated wc */
-    tmp->stats.wc = tmp->stats.wc - SP_level_range_adjust(caster,spell) 
+    tmp->stats.wc = tmp->stats.wc - SP_level_range_adjust(caster,spell)
 	- 5 * weapon->stats.Dex - 2 * weapon->stats.Str - weapon->magic;
     if(tmp->stats.wc<-127) tmp->stats.wc = -127;
 
     /* Modify hit points for weapon */
-    tmp->stats.maxhp = tmp->stats.maxhp + spell->duration + 
-	SP_level_duration_adjust(caster, spell) + 
+    tmp->stats.maxhp = tmp->stats.maxhp + spell->duration +
+	SP_level_duration_adjust(caster, spell) +
 	+ 8 * weapon->magic + 12 * weapon->stats.Con;
     if(tmp->stats.maxhp<0) tmp->stats.maxhp=10;
     tmp->stats.hp = tmp->stats.maxhp;
@@ -2602,9 +2599,9 @@ int animate_weapon(object *op,object *caster,object *spell, int dir) {
 	tmp->last_anim = weapon->last_anim;
 	tmp->state = weapon->state;
 	if(QUERY_FLAG(weapon, FLAG_ANIMATE)) {
-	    SET_FLAG(tmp,FLAG_ANIMATE); 
+	    SET_FLAG(tmp,FLAG_ANIMATE);
 	} else {
-	    CLEAR_FLAG(tmp,FLAG_ANIMATE); 
+	    CLEAR_FLAG(tmp,FLAG_ANIMATE);
 	}
 	update_ob_speed(tmp);
     }
@@ -2627,14 +2624,14 @@ int animate_weapon(object *op,object *caster,object *spell, int dir) {
 int cast_change_map_lightlevel( object *op, object *caster, object *spell ) {
     int success;
 
-    if(!op->map) return 0;   /* shouldnt happen */ 
+    if(!op->map) return 0;   /* shouldnt happen */
 
     success=change_map_light(op->map,spell->stats.dam);
     if(!success) {
 	if (spell->stats.dam < 0)
-	    new_draw_info(NDI_UNIQUE,0,op,"It can be no brighter here."); 
+	    new_draw_info(NDI_UNIQUE,0,op,"It can be no brighter here.");
 	else
-	    new_draw_info(NDI_UNIQUE,0,op,"It can be no darker here."); 
+	    new_draw_info(NDI_UNIQUE,0,op,"It can be no darker here.");
     }
     return success;
 }
@@ -2657,7 +2654,7 @@ int create_aura(object *op, object *caster, object *spell)
     if (new_aura) refresh=1;
     else new_aura = arch_to_object(spell->other_arch);
 
-    new_aura->duration  = spell->duration + 
+    new_aura->duration  = spell->duration +
                   10* SP_level_duration_adjust(caster,spell);
     if (op->type == PLAYER)
         store_spell_expiry(new_aura);
@@ -2670,7 +2667,7 @@ int create_aura(object *op, object *caster, object *spell)
     new_aura->attacktype= spell->attacktype;
 
     new_aura->level = caster_level(caster, spell);
-    if (refresh) 
+    if (refresh)
 	new_draw_info(NDI_UNIQUE, 0, op, "You recast the spell while in effect.");
     else
 	insert_ob_in_ob(new_aura, op);
@@ -2683,8 +2680,8 @@ int create_aura(object *op, object *caster, object *spell)
  * which he carries with him, but which acts on the map immediately
  * around him.
  * Aura parameters:
- * duration:  duration counter.   
- * attacktype:  aura's attacktype 
+ * duration:  duration counter.
+ * attacktype:  aura's attacktype
  * other_arch:  archetype to drop where we attack
  */
 
@@ -2697,7 +2694,7 @@ void move_aura(object *aura) {
     env = aura->env;
 
     /* no matter what we've gotta remove the aura...
-     * we'll put it back if its time isn't up.  
+     * we'll put it back if its time isn't up.
      */
     remove_ob(aura);
 
@@ -2716,11 +2713,11 @@ void move_aura(object *aura) {
     aura->y = env->y;
 
     /* we need to jump out of the inventory for a bit
-     * in order to hit the map conveniently. 
+     * in order to hit the map conveniently.
      */
     insert_ob_in_map(aura,env->map,aura,0);
 
-    for(i=1;i<9;i++) { 
+    for(i=1;i<9;i++) {
 	sint16 nx, ny;
 	nx = aura->x + freearr_x[i];
 	ny = aura->y + freearr_y[i];
@@ -2752,7 +2749,7 @@ void move_aura(object *aura) {
 /**
  * moves the peacemaker spell.
  * op is the piece object.
- */      
+ */
 
 void move_peacemaker(object *op) {
     object *tmp;
@@ -2793,8 +2790,8 @@ void move_peacemaker(object *op) {
 	    }
 	}
     }
-}   
-      
+}
+
 
 /**
  * This writes a rune that contains the appropriate message.
