@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_time_c =
- *    "$Id: time.c 5704 2007-03-06 19:41:03Z ryo_saeba $";
+ *    "$Id: time.c 7568 2007-11-24 21:17:09Z akirschbaum $";
  */
 
 /*
@@ -37,7 +37,7 @@
 #include <sproto.h>
 #endif
 
-/** The following removes doors.  The functions check to see if similar 
+/** The following removes doors.  The functions check to see if similar
  * doors are next to the one that is being removed, and if so, set it
  * so those will be removed shortly (in a cascade like fashion.)
  */
@@ -103,7 +103,7 @@ static void generate_monster_inv(object *gen) {
 	LOG(llevError,"Generator (%s) not on a map?\n", gen->name);
 	return;
     }
-    /*First count numer of objects in inv*/
+    /*First count number of objects in inv*/
     for (op=gen->inv;op;op=op->below)
         qty++;
     if (!qty){
@@ -209,7 +209,7 @@ static void remove_blindness(object *op) {
   if(--op->stats.food > 0)
     return;
   CLEAR_FLAG(op, FLAG_APPLIED);
-  if(op->env!=NULL) { 
+  if(op->env!=NULL) {
      change_abil(op->env,op);
      fix_player(op->env);
   }
@@ -289,7 +289,7 @@ static void move_gate(object *op) { /* 1 = going down, 0 = goind up */
 	 * the gate slightly.
 	 */
 
-	for (tmp=op->above; tmp!=NULL; tmp=tmp->above) 
+	for (tmp=op->above; tmp!=NULL; tmp=tmp->above)
 	    if (!QUERY_FLAG(tmp, FLAG_NO_PICK)
 		|| QUERY_FLAG(tmp, FLAG_CAN_ROLL)
 		|| QUERY_FLAG(tmp, FLAG_ALIVE))
@@ -328,10 +328,10 @@ static void move_gate(object *op) { /* 1 = going down, 0 = goind up */
 	    if(tmp!=NULL) {
 		if(QUERY_FLAG(tmp, FLAG_ALIVE)) {
 		    hit_player(tmp, random_roll(1, op->stats.dam, tmp, PREFER_LOW), op, AT_PHYSICAL, 1);
-		    if(tmp->type==PLAYER) 
+		    if(tmp->type==PLAYER)
 			new_draw_info_format(NDI_UNIQUE, 0, tmp,
 					     "You are crushed by the %s!",op->name);
-		} else 
+		} else
 		    /* If the object is not alive, and the object either can
 		     * be picked up or the object rolls, move the object
 		     * off the gate.
@@ -352,7 +352,7 @@ static void move_gate(object *op) { /* 1 = going down, 0 = goind up */
 	    }
 
 	    /* See if there is still anything blocking the gate */
-	    for (tmp=op->above; tmp!=NULL; tmp=tmp->above) 
+	    for (tmp=op->above; tmp!=NULL; tmp=tmp->above)
 		if (!QUERY_FLAG(tmp, FLAG_NO_PICK)
 			|| QUERY_FLAG(tmp, FLAG_CAN_ROLL)
 			|| QUERY_FLAG(tmp, FLAG_ALIVE))
@@ -387,7 +387,7 @@ static void move_timed_gate(object *op)
     if (op->value != v)   /* change direction ? */
       op->stats.sp = 0;
     return;
-  } 
+  }
   if (--op->stats.hp <= 0) { /* keep gate down */
     move_gate(op);
     if (op->value != v) {  /* ready ? */
@@ -403,26 +403,50 @@ static void move_timed_gate(object *op)
  *  sp:         1 if detection sets buttons
  *              -1 if detection unsets buttons
  */
-static void move_detector(object *op) 
+static void move_detector(object *op)
 {
-    object *tmp;
+    object *tmp, *tmp2;
     int last = op->value;
     int detected;
     detected = 0;
 
-    for(tmp = get_map_ob(op->map,op->x,op->y);tmp!=NULL&&!detected;tmp=tmp->above) {
-		object *tmp2;
-		if(op->stats.hp) {
-		  for(tmp2= tmp->inv;tmp2;tmp2=tmp2->below) {
-			 if(op->slaying && !strcmp(op->slaying,tmp->name)) detected=1;
-			 if(tmp2->type==FORCE &&tmp2->slaying && !strcmp(tmp2->slaying,op->slaying)) detected=1;
-		  }
-		}
-	if (op->slaying && !strcmp(op->slaying,tmp->name)) {
-	  detected = 1;
-	}
-	else if (tmp->type==SPECIAL_KEY && tmp->slaying==op->slaying)
-	    detected=1;
+    if (!op->slaying) {
+        if (op->map)
+            LOG(llevError, "Detector with no slaying set at %s (%d,%d)\n", op->map->path, op->x, op->y);
+        else if (op->env)
+            LOG(llevError, "Detector with no slaying in %s\n", op->env->name);
+        else
+            LOG(llevError, "Detector with no slaying nowhere?\n");
+        op->speed = 0;
+        update_ob_speed(op);
+        return;
+    }
+
+    for(tmp = get_map_ob(op->map, op->x, op->y); tmp!=NULL ;tmp = tmp->above) {
+        if (op->stats.hp) {
+            for(tmp2= tmp->inv;tmp2;tmp2=tmp2->below) {
+                if (op->slaying == tmp2->name) {
+                    detected=1;
+                    break;
+                }
+                if (tmp2->type==FORCE && tmp2->slaying == op->slaying) {
+                    detected=1;
+                    break;
+                }
+            }
+        }
+        if (op->slaying == tmp->name) {
+            detected = 1;
+            break;
+        }
+        if (tmp->type == PLAYER && !strcmp(op->slaying, "player")) {
+            detected = 1;
+            break;
+        }
+        if (tmp->type==SPECIAL_KEY && tmp->slaying==op->slaying) {
+            detected=1;
+            break;
+        }
     }
 
     /* the detector sets the button if detection is found */
@@ -510,6 +534,9 @@ static void move_hole(object *op) { /* 1 = opening, 0 = closing */
  */
 object *stop_item (object *op)
 {
+    if (free_no_drop(op))
+        return NULL;
+
     if (op->map == NULL)
         return op;
 
@@ -554,6 +581,9 @@ void fix_stopped_item (object *op, mapstruct *map, object *originator)
 
 object *fix_stopped_arrow (object *op)
 {
+    if (free_no_drop(op))
+        return NULL;
+
     if(rndm(0, 99) < op->stats.food) {
 	/* Small chance of breaking */
         remove_ob (op);
@@ -594,6 +624,28 @@ object *fix_stopped_arrow (object *op)
     return op;
 }
 
+/**
+ * Check whether the given object is FLAG_NO_DROP. If so, (optionally) remove
+ * and free it.
+ *
+ * @param op
+ * the object to check
+ * @return
+ * whether the object was freed
+ */
+int free_no_drop(object *op) {
+    if (!QUERY_FLAG(op, FLAG_NO_DROP)) {
+        return 0;
+    }
+
+    if(!QUERY_FLAG(op, FLAG_REMOVED)) {
+        remove_ob(op);
+    }
+
+    free_object2(op, 1);
+    return 1;
+}
+
 /** stop_arrow() - what to do when a non-living flying object
  * has to stop. Sept 96 - I added in thrown object code in
  * here too. -b.t.
@@ -636,7 +688,7 @@ void move_arrow(object *op) {
 	return;
     }
 
-    /* we need to stop thrown objects at some point. Like here. */ 
+    /* we need to stop thrown objects at some point. Like here. */
     if(op->type==THROWN_OBJ) {
 	/* If the object that the THROWN_OBJ encapsulates disappears,
 	 * we need to have this object go away also - otherwise, you get
@@ -651,9 +703,9 @@ void move_arrow(object *op) {
 	    free_object(op);
             return;
 	}
-	if(op->last_sp-- < 0) { 
+	if(op->last_sp-- < 0) {
 	    stop_arrow (op);
-	    return; 
+	    return;
 	}
     }
 
@@ -682,7 +734,7 @@ void move_arrow(object *op) {
 	for (tmp = get_map_ob(m, new_x, new_y); tmp != NULL; tmp=tmp->above)
 	     if (QUERY_FLAG(tmp, FLAG_ALIVE)) break;
 
-    
+
 	/* Not really fair, but don't let monsters hit themselves with
 	 * their own arrow - this can be because they fire it then
 	 * move into it.
@@ -698,7 +750,7 @@ void move_arrow(object *op) {
 		(rndm(0, 99)) < (90-op->level/10)) {
 
 		int number = op->face->number;
-	    
+
 		op->direction = absdir (op->direction + 4);
 		op->state = 0;
 		if (GET_ANIM_ID (op)) {
@@ -773,7 +825,7 @@ void move_arrow(object *op) {
 		/* If this space is not out of the map and not blocked, valid space -
 		 * don't need to retry again.
 		 */
-		if (!(mflags & P_OUT_OF_MAP) && 
+		if (!(mflags & P_OUT_OF_MAP) &&
 		  !OB_TYPE_MOVE_BLOCK(op, GET_MAP_MOVE_BLOCK(m1, x1, y1))) break;
 
 	    }
@@ -804,7 +856,7 @@ void move_arrow(object *op) {
 }
 
 /** This routine doesnt seem to work for "inanimate" objects that
- * are being carried, ie a held torch leaps from your hands!. 
+ * are being carried, ie a held torch leaps from your hands!.
  * Modified this routine to allow held objects. b.t. */
 
 static void change_object(object *op) { /* Doesn`t handle linked objs yet */
@@ -818,7 +870,7 @@ static void change_object(object *op) { /* Doesn`t handle linked objs yet */
 
   /* In non-living items only change when food value is 0 */
   if(!QUERY_FLAG(op,FLAG_ALIVE)) {
-	if(op->stats.food-- > 0) return; 
+	if(op->stats.food-- > 0) return;
 	else op->stats.food=1; /* so 1 other_arch is made */
   }
   env=op->env;
@@ -831,14 +883,6 @@ static void change_object(object *op) { /* Doesn`t handle linked objs yet */
     if(env) {
         tmp->x=env->x,tmp->y=env->y;
 	tmp=insert_ob_in_ob(tmp,env);
-	/* If this object is the players inventory, we need to tell the
-	 * client of the change.  Insert_ob_in_map takes care of the
-	 * updating the client, so we don't need to do that below.
-	 */
-	if ((pl=get_player_container(env))!=NULL) {
-	    esrv_del_item(pl->contr, op->count);
-	    esrv_send_item(pl, tmp);
-	}
     } else {
         j=find_first_free_spot(tmp,op->map,op->x,op->y);
 	if (j==-1)  /* No free spot */
@@ -931,7 +975,7 @@ void move_player_changer(object *op) {
 	fix_player(player);
 	esrv_send_inventory(op->above,op->above);
 	esrv_update_item(UPD_FACE, op->above, op->above);
-	
+
 	/* update players death & WoR home-position */
 	sscanf(EXIT_PATH(op), "%c", &c);
 	if (c == '/') {
@@ -943,7 +987,7 @@ void move_player_changer(object *op) {
             LOG(llevDebug,
                 "WARNING: destination '%s' in player_changer must be an absolute path!\n",
 		EXIT_PATH(op));
-	
+
 	enter_exit(op->above,op);
 	save_player(player, 1);
     }
@@ -992,7 +1036,7 @@ void move_player_mover(object *op) {
     if (!dir) dir=rndm(1, 8);
 
     for(victim=get_map_ob(op->map,op->x,op->y); victim !=NULL; victim=victim->above) {
-	if(QUERY_FLAG(victim, FLAG_ALIVE) && !QUERY_FLAG(victim, FLAG_WIZPASS) && 
+	if(QUERY_FLAG(victim, FLAG_ALIVE) && !QUERY_FLAG(victim, FLAG_WIZPASS) &&
 	   (victim->move_type & op->move_type || !victim->move_type)) {
 
 	    if (victim->head) victim = victim->head;
@@ -1010,18 +1054,18 @@ void move_player_mover(object *op) {
 		    m->path, op->x, op->y);
 		return ;
 	    }
-	    
+
 	    if (should_director_abort(op, victim)) return ;
 
 	    for(nextmover=get_map_ob(m,nx, ny); nextmover !=NULL; nextmover=nextmover->above) {
-		if(nextmover->type == PLAYERMOVER) 
+		if(nextmover->type == PLAYERMOVER)
 		    nextmover->speed_left=-.99;
 		if(QUERY_FLAG(nextmover,FLAG_ALIVE)) {
 		    op->speed_left=-1.1;  /* wait until the next thing gets out of the way */
 		}
 	    }
 
-	    if(victim->type==PLAYER) { 
+	    if(victim->type==PLAYER) {
 		/*  only level >=1 movers move people */
 		if(op->level) {
 		    /* Following is a bit of hack.  We need to make sure it
@@ -1045,7 +1089,7 @@ void move_player_mover(object *op) {
 		/* Not sure why, but for some chars on metalforge, they
 		 * would sometimes get -inf speed_left, and from the
 		 * description, it could only happen here, so just put
-		 * a lower sanity limit.  My only guess is that the 
+		 * a lower sanity limit.  My only guess is that the
 		 * mover has 0 speed.
 		 */
 		if (victim->speed_left < -5.0) victim->speed_left=-5.0;
@@ -1088,7 +1132,7 @@ void move_duplicator(object *op) {
 }
 
 /**
- * move_creator (by peterm) 
+ * move_creator (by peterm)
  * it has the creator object create it's other_arch right on top of it.
  * connected:  what will trigger it
  * hp:  how many times it may create before stopping
@@ -1164,7 +1208,7 @@ void move_creator(object *creator) {
  * unless hp was zero to start with, in which case it is infinite.*/
 void move_marker(object *op) {
     object *tmp,*tmp2;
-  
+
     for(tmp=get_map_ob(op->map,op->x,op->y);tmp!=NULL;tmp=tmp->above) {
 	if(tmp->type == PLAYER) { /* we've got someone to MARK */
 
@@ -1181,13 +1225,13 @@ void move_marker(object *op) {
 		free_object(tmp2);
 	    }
 
-	    /* cycle through his inventory to look for the MARK we want to 
-	     * place 
+	    /* cycle through his inventory to look for the MARK we want to
+	     * place
 	     */
 	    for(tmp2=tmp->inv;tmp2 !=NULL; tmp2=tmp2->below) {
 		if(tmp2->type == FORCE && tmp2->slaying && !strcmp(tmp2->slaying,op->slaying)) break;
 	    }
-      
+
 	    /* if we didn't find our own MARK */
 	    if(tmp2==NULL) {
 		object *force = create_archetype(FORCE_NAME);
@@ -1208,7 +1252,7 @@ void move_marker(object *op) {
 		if(op->msg)
 		    new_draw_info(NDI_UNIQUE|NDI_NAVY,0,tmp,op->msg);
 
-		if(op->stats.hp > 0) { 
+		if(op->stats.hp > 0) {
 		    op->stats.hp--;
 		    if(op->stats.hp==0) {
 			/* marker expires--granted mark number limit */
@@ -1221,13 +1265,17 @@ void move_marker(object *op) {
 	} /* if tmp->type == PLAYER */
     } /* For all objects on this space */
 }
- 
+
 int process_object(object *op) {
     if (QUERY_FLAG(op, FLAG_IS_A_TEMPLATE))
 	return 0;
 
+    /* Lauwenmark: Handle for plugin time event */
+    if (execute_event(op, EVENT_TIME,NULL,NULL,NULL,SCRIPT_FIX_NOTHING) != 0)
+        return 0;
+
     if(QUERY_FLAG(op, FLAG_MONSTER))
-	if(move_monster(op) || QUERY_FLAG(op, FLAG_FREED)) 
+	if(move_monster(op) || QUERY_FLAG(op, FLAG_FREED))
 	    return 1;
 
     if(QUERY_FLAG(op, FLAG_ANIMATE) && op->anim_speed==0) {
@@ -1261,8 +1309,6 @@ int process_object(object *op) {
 	}
 	return 1;
     }
-    /* Lauwenmark: Handle for plugin time event */
-    execute_event(op, EVENT_TIME,NULL,NULL,NULL,SCRIPT_FIX_NOTHING);
     switch(op->type) {
 	case TRANSPORT:
 	    /* Transports are directed by players - thus, there
