@@ -1,7 +1,7 @@
 
 /*
  * static char *rcsid_item_c =
- *    "$Id: item.c 5290 2007-01-02 22:59:21Z ryo_saeba $";
+ *    "$Id: item.c 6930 2007-08-15 10:10:28Z ryo_saeba $";
  */
 
 /*
@@ -573,6 +573,7 @@ static object *esrv_get_ob_from_count(object *pl, tag_t count)
 void examine_cmd(char *buf, int len,player *pl)
 {
     long tag;
+    object* op;
 
     if (len <= 0 || !buf) {
         LOG(llevDebug, "Player '%s' sent bogus examine_cmd information", pl->ob->name);
@@ -580,7 +581,7 @@ void examine_cmd(char *buf, int len,player *pl)
     }
 
     tag = atoi(buf);
-    object *op = esrv_get_ob_from_count(pl->ob, tag);
+    op = esrv_get_ob_from_count(pl->ob, tag);
 
     if (!op) {
 	LOG(llevDebug, "Player '%s' tried to examine the unknown object (%ld)\n",
@@ -650,12 +651,8 @@ void lock_item_cmd(uint8 *data, int len,player *pl)
 
     tmp = merge_ob(op, NULL);
     if (tmp == NULL) {
-	/* object was not merged */
-	esrv_update_item(UPD_FLAGS, pl->ob, op);
-    } else {
-	/* object was merged into tmp */
-	esrv_del_item(pl, tag);
-	esrv_send_item(pl->ob, tmp);
+            /* object was not merged - if it was, merge_ob sent updates for us. */
+        esrv_update_item(UPD_FLAGS, pl->ob, op);
     }
 }
 
@@ -841,4 +838,57 @@ void esrv_move_object (object *pl, tag_t to, tag_t tag, long nrof)
     }
 }
 
+void inscribe_scroll_cmd(char* buf, int len, player* pl) {
+    object *scroll, *spell, *marked, *inscription, *currentspell;
+    tag_t tscroll, tspell, tmarked;
+    char type;
 
+    if (len < 1) {
+        LOG(llevDebug, "Player %s sent an invalid inscribe command.\n", pl->ob->name);
+        return;
+    }
+
+    type = buf[0];
+
+    inscription = find_skill_by_name(pl->ob, "inscription");
+    if (!inscription) {
+        draw_ext_info(NDI_UNIQUE, 0, pl->ob, MSG_TYPE_SKILL, MSG_TYPE_SKILL_FAILURE, "You don't know how to write!", NULL);
+        return;
+    }
+
+    if (type == 0) {
+        if (len != 9) {
+            LOG(llevDebug, "Player %s sent an invalid inscribe command.\n", pl->ob->name);
+            return;
+        }
+        tscroll = GetInt_String(buf+1);
+        tspell = GetInt_String(buf+5);
+
+        scroll = esrv_get_ob_from_count(pl->ob, tscroll);
+        if (!scroll) {
+            LOG(llevDebug, "Player %s sent an invalid scroll for inscribe command.\n", pl->ob->name);
+            return;
+        }
+
+        spell = esrv_get_ob_from_count(pl->ob, tspell);
+        if (!spell) {
+            LOG(llevDebug, "Player %s sent an invalid spell for inscribe command.\n", pl->ob->name);
+            return;
+        }
+
+        tmarked = pl->mark_count;
+        marked = pl->mark;
+        currentspell = pl->ranges[range_magic];
+
+        pl->mark_count = tscroll;
+        pl->mark = scroll;
+        pl->ranges[range_magic] = spell;
+
+        write_on_item(pl->ob, "", inscription);
+
+        pl->mark_count = tmarked;
+        pl->mark = marked;
+        pl->ranges[range_magic] = currentspell;
+    } else {
+    }
+}
