@@ -1,7 +1,7 @@
 
 /*
  * static char *rcsid_loop_c =
- *    "$Id: loop.c 5074 2006-11-04 07:37:28Z mwedel $";
+ *    "$Id: loop.c 8087 2008-01-04 09:37:52Z ryo_saeba $";
  */
 
 /*
@@ -116,6 +116,7 @@ static struct player_cmd_mapping player_commands[] = {
     { "lock",		(func_uint8_int_pl)lock_item_cmd,	1},
     { "mark",		(func_uint8_int_pl)mark_item_cmd,	1},
     { "mapredraw",	map_redraw_cmd,	0},	/* Added: phil */
+    { "inscribe",   inscribe_scroll_cmd,    0},
     { NULL, NULL, 0}	/* terminator */
 };
 
@@ -507,6 +508,17 @@ static void block_until_new_connection(void)
     reset_sleep(); /* Or the game would go too fast */
 }
 
+/**
+ * Checks if file descriptor is valid.
+ *
+ * @param fd
+ * file descriptor to check.
+ * @return
+ * 1 if fd is valid, 0 else.
+ */
+static int is_fd_valid(int fd) {
+    return fcntl(fd, F_GETFL) != -1 || errno != EBADF;
+}
 
 /**
  * This checks the sockets for input and exceptions, does the right thing.
@@ -533,6 +545,10 @@ void doeric_server(void)
     FD_ZERO(&tmp_exceptions);
 
     for(i=0;i<socket_info.allocated_sockets;i++) {
+        if (init_sockets[i].status == Ns_Add && !is_fd_valid(init_sockets[i].fd)) {
+            LOG(llevError, "doeric_server: invalid waiting fd %d\n", i);
+            init_sockets[i].status = Ns_Dead;
+        }
 	if (init_sockets[i].status == Ns_Dead) {
 	    free_newsocket(&init_sockets[i]);
 	    init_sockets[i].status = Ns_Avail;
@@ -548,7 +564,12 @@ void doeric_server(void)
      * since we may remove some
      */
     for (pl=first_player; pl!=NULL; ) {
-	if (pl->socket.status == Ns_Dead) {
+        if (pl->socket.status != Ns_Dead && !is_fd_valid(pl->socket.fd)) {
+            LOG(llevError, "doeric_server: invalid file descriptor for player %s [%s]: %d\n", (pl->ob && pl->ob->name) ? pl->ob->name : "(unnamed player?)", (pl->socket.host) ? pl->socket.host : "(unknown ip?)", pl->socket.fd);
+            pl->socket.status = Ns_Dead;
+        }
+
+        if (pl->socket.status == Ns_Dead) {
 	    player *npl=pl->next;
 
 	    save_player(pl->ob, 0);
@@ -602,7 +623,7 @@ void doeric_server(void)
 	    newsocknum = socket_info.allocated_sockets;
 	    socket_info.allocated_sockets++;
 	    init_sockets[newsocknum].faces_sent_len = nrofpixmaps;
-	    init_sockets[newsocknum].faces_sent = malloc(nrofpixmaps*sizeof(*init_sockets[newsocknum].faces_sent));
+	    init_sockets[newsocknum].faces_sent = calloc(1, nrofpixmaps*sizeof(*init_sockets[newsocknum].faces_sent));
 	    if (!init_sockets[newsocknum].faces_sent) fatal(OUT_OF_MEMORY);
 	    init_sockets[newsocknum].status = Ns_Avail;
 	}
@@ -690,6 +711,18 @@ void doeric_server(void)
 	}
 	else {
 	    handle_client(&pl->socket, pl);
+
+	    /* There seems to be rare cases where next points to a removed/freed player.
+	     * My belief is that this player does something (shout, move, whatever)
+	     * that causes data to be sent to the next player on the list, but
+	     * that player is defunct, so the socket codes removes that player.
+	     * End result is that next now points at the removed player, and
+	     * that has garbage data so we crash.  So update the next pointer
+	     * while pl is still valid.  MSW 2007-04-21
+	     */
+	    next = pl->next;
+
+
 	    /* If the player has left the game, then the socket status
 	     * will be set to this be the leave function.  We don't
 	     * need to call leave again, as it has already been called
@@ -720,6 +753,7 @@ void doeric_server(void)
 		draw_client_map(pl->ob);
 		if (pl->socket.update_look) esrv_draw_look(pl->ob);
 		if (pl->socket.tick) send_tick(pl);
+
 	    }
 	}
     }
