@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_init_c =
- *    "$Id: request.c 5149 2006-12-02 19:17:49Z ryo_saeba $";
+ *    "$Id: request.c 7568 2007-11-24 21:17:09Z akirschbaum $";
  */
 
 /*
@@ -246,12 +246,14 @@ void set_up_cmd(char *buf, int len, socket_struct *ns)
 	} else if (!strcmp(cmd,"bot")) {
 		ns->is_bot = ( atoi(param) != 0 ? 1 : 0 );
 		safe_strcat(cmdback, ns->is_bot ? "1" : "0", &slen, HUGE_BUF);
-	} else {
-	    /* Didn't get a setup command we understood -
-	     * report a failure to the client.
-	     */
-	    safe_strcat(cmdback, "FALSE", &slen, HUGE_BUF);
-	}
+        } else if (!strcmp(cmd,"inscribe")) {
+            safe_strcat(cmdback, atoi(param) != 0 ? "1" : "0", &slen, HUGE_BUF);
+        } else {
+                /* Didn't get a setup command we understood -
+                 * report a failure to the client.
+                 */
+            safe_strcat(cmdback, "FALSE", &slen, HUGE_BUF);
+        }
     } /* for processing all the setup commands */
     LOG(llevInfo,"SendBack SetupCmd:: %s\n", cmdback);
     Write_String_To_Socket(ns, cmdback, strlen(cmdback));
@@ -640,7 +642,7 @@ void version_cmd(char *buf, int len,socket_struct *ns)
 
 	/* these are old dxclients */
 	/* Version 1024 added support for singular + plural name values - 
-	 * requiing this minimal value reduces complexity of that code, and it
+	 * requiring this minimal value reduces complexity of that code, and it
 	 * has been around for a long time.
 	 */
 	if(!strcmp(" CF DX CLIENT", cp) || ns->sc_version < 1024 )
@@ -2321,7 +2323,17 @@ void esrv_update_spells(player *pl) {
     int flags=0;
     object *spell;
     client_spell* spell_info;
+
     if (!pl->socket.monitor_spells) return;
+
+    /* Handles problem at login, where this is called from fix_object
+     * before we have had a chance to send spells to the player.  It does seem
+     * to me that there should never be a case where update_spells is called
+     * before add_spells has been called.  Add_spells() will update the
+     * spell_state to non null.
+     */
+    if (!pl->spell_state) return;
+
     for (spell=pl->ob->inv; spell!=NULL; spell=spell->below) {
 	if (spell->type == SPELL) {
         spell_info = get_client_spell_state(pl, spell);
