@@ -1,6 +1,6 @@
 /*
  * static char *rcsid_check_object_c =
- *   "$Id: check_object.c 4661 2006-07-02 15:32:35Z tchize $";
+ *   "$Id: check_object.c 6683 2007-06-20 20:07:39Z ryo_saeba $";
  */
 
 /*
@@ -94,7 +94,6 @@ void teardown(void)
  * get_ob_key_link
  * get_ob_key_value
  * set_ob_key_value
- * find_best_weapon_used_match
  * item_matched_string
  */
 /** This is the test to check the behaviour of the method
@@ -372,7 +371,6 @@ START_TEST (test_reset_object)
   fail_unless(ob1->msg == NULL,"Field msg of ob1 was not NULLified by reset_object");
   fail_unless(ob1->materialname == NULL,"Field materialname of ob1 was not NULLified by reset_object");
   fail_unless(ob1->lore == NULL,"Field lore of ob1 was not NULLified by reset_object");
-  fail_unless(ob1->current_weapon_script == NULL,"Field current_weapon_script of ob1 was not NULLified by reset_object");
 }
 END_TEST
 
@@ -397,7 +395,6 @@ START_TEST (test_clear_object)
   fail_unless(ob1->msg == NULL,"Field msg of ob1 was not cleaned by clear_object");
   fail_unless(ob1->materialname == NULL,"Field materialname of ob1 was not cleaned by clear_object");
   fail_unless(ob1->lore == NULL,"Field lore of ob1 was not cleaned by clear_object");
-  fail_unless(ob1->current_weapon_script == NULL,"Field current_weapon_script of ob1 was not cleaned by clear_object");
   fail_unless(query_refcount(reference)==1,
               "The number of references to string should drop back to 1 but was %d",query_refcount(reference));
 }
@@ -427,7 +424,6 @@ START_TEST (test_copy_object)
   fail_unless(ob1->msg==ob2->msg,"Field msg of ob1 should match ob2");
   fail_unless(ob1->materialname==ob2->materialname,"Field materialname of ob1 should match ob2");
   fail_unless(ob1->lore==ob2->lore,"Field lore of ob1 should match ob2");
-  fail_unless(ob1->current_weapon_script==ob2->current_weapon_script,"Field current_weapon_script of ob1 should match ob2");
   fail_unless(query_refcount(reference)==1, "refcount of marker string is not dropped to 1 after copy object, some string field were not cleaned. refcount: %d",query_refcount(reference));
 }
 END_TEST
@@ -1028,23 +1024,44 @@ START_TEST (test_set_ob_key_value)
 }
 END_TEST
 
-
-/** This is the test to check the behaviour of the method
- *  object *find_best_weapon_used_match(object *pl, const char *params);
- */
-START_TEST (test_find_best_weapon_used_match)
-{
-    /*TESTME*/
-}
-END_TEST
-
-
 /** This is the test to check the behaviour of the method
  *  int item_matched_string(object *pl, object *op, const char *name);
  */
 START_TEST (test_item_matched_string)
 {
-    /*TESTME*/
+    object *pl;
+    object *o1, *o2, *o3, *o4;
+    int val;
+
+    pl = cctk_create_game_object("kobold");
+    fail_unless(pl != NULL, "couldn't create kobold");
+    pl->contr = (player*)calloc(1, sizeof(player));
+    fail_unless(pl->contr != NULL, "couldn't alloc contr");
+
+    o1 = cctk_create_game_object("cloak");
+    fail_unless(o1 != NULL, "couldn't find cloak archetype");
+    o1->title = add_string("of Gorokh");
+    CLEAR_FLAG(o1, FLAG_IDENTIFIED);
+					    
+    val = item_matched_string(pl, o1, "all");
+    fail_unless(val == 1, "all didn't match cloak");
+    val = item_matched_string(pl, o1, "Gorokh");
+    fail_unless(val == 0, "unidentified cloak matched title with value %d", val);
+    val = item_matched_string(pl, o1, "random");
+    fail_unless(val == 0, "unidentified cloak matched random value with value %d", val);
+								    
+    SET_FLAG(o1, FLAG_IDENTIFIED);
+    val = item_matched_string(pl, o1, "Gorokh");
+    fail_unless(val != 0, "identified cloak didn't match title with value %d", val);
+
+    o2 = cctk_create_game_object("cloak");
+    SET_FLAG(o2, FLAG_UNPAID);
+    val = item_matched_string(pl, o2, "unpaid");
+    fail_unless(val == 2, "unpaid cloak didn't match unpaid");
+    val = item_matched_string(pl, o2, "cloak");
+    fail_unless(val != 0, "unpaid cloak didn't match cloak with %d", val);
+    val = item_matched_string(pl, o2, "wrong");
+    fail_unless(val == 0, "unpaid cloak matched wrong name %d", val);
 }
 END_TEST
 
@@ -1118,7 +1135,6 @@ Suite *object_suite(void)
   tcase_add_test(tc_core, test_get_ob_key_link);
   tcase_add_test(tc_core, test_get_ob_key_value);
   tcase_add_test(tc_core, test_set_ob_key_value);
-  tcase_add_test(tc_core, test_find_best_weapon_used_match);
   tcase_add_test(tc_core, test_item_matched_string);
 
   return s;
